/*     */ package org.apache.batik.gvt;
/*     */ 
/*     */ import java.awt.Graphics2D;
/*     */ import java.awt.Shape;
/*     */ import java.awt.geom.AffineTransform;
/*     */ import java.awt.geom.GeneralPath;
/*     */ import java.awt.geom.Point2D;
/*     */ import java.awt.geom.Rectangle2D;
/*     */ import java.io.Serializable;
/*     */ import java.text.AttributedCharacterIterator;
/*     */ import java.util.List;
/*     */ import org.apache.batik.gvt.renderer.StrokingTextPainter;
/*     */ import org.apache.batik.gvt.text.AttributedCharacterSpanIterator;
/*     */ import org.apache.batik.gvt.text.GVTAttributedCharacterIterator;
/*     */ import org.apache.batik.gvt.text.Mark;
/*     */ import org.apache.batik.gvt.text.TextHit;
/*     */ import org.apache.batik.gvt.text.TextPaintInfo;
/*     */ import org.apache.batik.gvt.text.TextSpanLayout;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class TextNode
/*     */   extends AbstractGraphicsNode
/*     */   implements Selectable
/*     */ {
/*  48 */   public static final AttributedCharacterIterator.Attribute PAINT_INFO = GVTAttributedCharacterIterator.TextAttribute.PAINT_INFO;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*  55 */   protected Point2D location = new Point2D.Float(0.0F, 0.0F);
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected AttributedCharacterIterator aci;
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected String text;
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*  70 */   protected Mark beginMark = null;
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*  75 */   protected Mark endMark = null;
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected List textRuns;
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*  85 */   protected TextPainter textPainter = StrokingTextPainter.getInstance();
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   private Rectangle2D geometryBounds;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   private Rectangle2D primitiveBounds;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   private Shape outline;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void setTextPainter(TextPainter textPainter) {
/* 117 */     if (textPainter == null) {
/* 118 */       this.textPainter = StrokingTextPainter.getInstance();
/*     */     } else {
/* 120 */       this.textPainter = textPainter;
/*     */     } 
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 128 */   public TextPainter getTextPainter() { return this.textPainter; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 135 */   public List getTextRuns() { return this.textRuns; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 144 */   public void setTextRuns(List textRuns) { this.textRuns = textRuns; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public String getText() {
/* 152 */     if (this.text != null) {
/* 153 */       return this.text;
/*     */     }
/* 155 */     if (this.aci == null) {
/* 156 */       this.text = "";
/*     */     } else {
/* 158 */       StringBuffer buf = new StringBuffer(this.aci.getEndIndex());
/* 159 */       char c = this.aci.first();
/* 160 */       for (; c != Character.MAX_VALUE; 
/* 161 */         c = this.aci.next()) {
/* 162 */         buf.append(c);
/*     */       }
/* 164 */       this.text = buf.toString();
/*     */     } 
/* 166 */     return this.text;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void setLocation(Point2D newLocation) {
/* 175 */     fireGraphicsNodeChangeStarted();
/* 176 */     invalidateGeometryCache();
/* 177 */     this.location = newLocation;
/* 178 */     fireGraphicsNodeChangeCompleted();
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 187 */   public Point2D getLocation() { return this.location; }
/*     */ 
/*     */ 
/*     */   
/*     */   public void swapTextPaintInfo(TextPaintInfo newInfo, TextPaintInfo oldInfo) {
/* 192 */     fireGraphicsNodeChangeStarted();
/* 193 */     invalidateGeometryCache();
/* 194 */     oldInfo.set(newInfo);
/* 195 */     fireGraphicsNodeChangeCompleted();
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void setAttributedCharacterIterator(AttributedCharacterIterator newAci) {
/* 206 */     fireGraphicsNodeChangeStarted();
/* 207 */     invalidateGeometryCache();
/* 208 */     this.aci = newAci;
/* 209 */     this.text = null;
/* 210 */     this.textRuns = null;
/* 211 */     fireGraphicsNodeChangeCompleted();
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 220 */   public AttributedCharacterIterator getAttributedCharacterIterator() { return this.aci; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected void invalidateGeometryCache() {
/* 233 */     super.invalidateGeometryCache();
/* 234 */     this.primitiveBounds = null;
/* 235 */     this.geometryBounds = null;
/* 236 */     this.outline = null;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public Rectangle2D getPrimitiveBounds() {
/* 243 */     if (this.primitiveBounds == null && 
/* 244 */       this.aci != null) {
/* 245 */       this.primitiveBounds = this.textPainter.getBounds2D(this);
/*     */     }
/*     */     
/* 248 */     return this.primitiveBounds;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public Rectangle2D getGeometryBounds() {
/* 258 */     if (this.geometryBounds == null && 
/* 259 */       this.aci != null) {
/* 260 */       this.geometryBounds = this.textPainter.getGeometryBounds(this);
/*     */     }
/*     */     
/* 263 */     return this.geometryBounds;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 272 */   public Rectangle2D getSensitiveBounds() { return getGeometryBounds(); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public Shape getOutline() {
/* 279 */     if (this.outline == null && 
/* 280 */       this.aci != null) {
/* 281 */       this.outline = this.textPainter.getOutline(this);
/*     */     }
/*     */     
/* 284 */     return this.outline;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 293 */   public Mark getMarkerForChar(int index, boolean beforeChar) { return this.textPainter.getMark(this, index, beforeChar); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void setSelection(Mark begin, Mark end) {
/* 300 */     if (begin.getTextNode() != this || end.getTextNode() != this)
/*     */     {
/* 302 */       throw new Error("Markers not from this TextNode");
/*     */     }
/* 304 */     this.beginMark = begin;
/* 305 */     this.endMark = end;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public boolean selectAt(double x, double y) {
/* 314 */     this.beginMark = this.textPainter.selectAt(x, y, this);
/* 315 */     return true;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public boolean selectTo(double x, double y) {
/* 324 */     Mark tmpMark = this.textPainter.selectTo(x, y, this.beginMark);
/* 325 */     if (tmpMark == null)
/* 326 */       return false; 
/* 327 */     if (tmpMark != this.endMark) {
/* 328 */       this.endMark = tmpMark;
/* 329 */       return true;
/*     */     } 
/* 331 */     return false;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public boolean selectAll(double x, double y) {
/* 340 */     this.beginMark = this.textPainter.selectFirst(this);
/* 341 */     this.endMark = this.textPainter.selectLast(this);
/* 342 */     return true;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public Object getSelection() {
/* 351 */     Object o = null;
/* 352 */     if (this.aci == null) return o;
/*     */     
/* 354 */     int[] ranges = this.textPainter.getSelected(this.beginMark, this.endMark);
/*     */ 
/*     */ 
/*     */     
/* 358 */     if (ranges != null && ranges.length > 1) {
/*     */       
/* 360 */       if (ranges[0] > ranges[1]) {
/* 361 */         int temp = ranges[1];
/* 362 */         ranges[1] = ranges[0];
/* 363 */         ranges[0] = temp;
/*     */       } 
/* 365 */       o = new AttributedCharacterSpanIterator(this.aci, ranges[0], ranges[1] + 1);
/*     */     } 
/*     */     
/* 368 */     return o;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public Shape getHighlightShape() {
/* 377 */     highlightShape = this.textPainter.getHighlightShape(this.beginMark, this.endMark);
/*     */     
/* 379 */     AffineTransform t = getGlobalTransform();
/* 380 */     return t.createTransformedShape(highlightShape);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void primitivePaint(Graphics2D g2d) {
/* 399 */     Shape clip = g2d.getClip();
/* 400 */     if (clip != null && !(clip instanceof GeneralPath)) {
/* 401 */       g2d.setClip(new GeneralPath(clip));
/*     */     }
/*     */     
/* 404 */     this.textPainter.paint(this, g2d);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public boolean contains(Point2D p) {
/* 421 */     if (!super.contains(p)) {
/* 422 */       return false;
/*     */     }
/* 424 */     List list = getTextRuns();
/*     */     
/* 426 */     for (int i = 0; i < list.size(); i++) {
/* 427 */       StrokingTextPainter.TextRun run = (StrokingTextPainter.TextRun)list.get(i);
/*     */       
/* 429 */       TextSpanLayout layout = run.getLayout();
/* 430 */       float x = (float)p.getX();
/* 431 */       float y = (float)p.getY();
/* 432 */       TextHit textHit = layout.hitTestChar(x, y);
/* 433 */       if (textHit != null && contains(p, layout.getBounds2D())) {
/* 434 */         return true;
/*     */       }
/*     */     } 
/* 437 */     return false;
/*     */   }
/*     */   
/*     */   protected boolean contains(Point2D p, Rectangle2D b) {
/* 441 */     if (b == null || !b.contains(p)) {
/* 442 */       return false;
/*     */     }
/* 444 */     switch (this.pointerEventType) {
/*     */       case 0:
/*     */       case 1:
/*     */       case 2:
/*     */       case 3:
/* 449 */         return this.isVisible;
/*     */       case 4:
/*     */       case 5:
/*     */       case 6:
/*     */       case 7:
/* 454 */         return true;
/*     */       case 8:
/* 456 */         return false;
/*     */     } 
/* 458 */     return false;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static final class Anchor
/*     */     implements Serializable
/*     */   {
/*     */     public static final int ANCHOR_START = 0;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/*     */     public static final int ANCHOR_MIDDLE = 1;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/*     */     public static final int ANCHOR_END = 2;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 488 */     public static final Anchor START = new Anchor(0);
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 495 */     public static final Anchor MIDDLE = new Anchor(1);
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 502 */     public static final Anchor END = new Anchor(2);
/*     */ 
/*     */ 
/*     */ 
/*     */     
/*     */     private int type;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 513 */     private Anchor(int type) { this.type = type; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 520 */     public int getType() { return this.type; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/*     */     private Object readResolve() {
/* 531 */       switch (this.type) {
/*     */         case 0:
/* 533 */           return START;
/*     */         case 1:
/* 535 */           return MIDDLE;
/*     */         case 2:
/* 537 */           return END;
/*     */       } 
/* 539 */       throw new Error("Unknown Anchor type");
/*     */     }
/*     */   }
/*     */ }


/* Location:              C:\Program Files (x86)\IBM\WebSphere\AppServer\profiles\ctgAppSrv01\installedApps\ctgCell01\MAXIMO.ear\maximouiweb.war\WEB-INF\classes\!\com\ibm\tivoli\maximo\report\control\svgtools\batik-gvt.jar!\org\apache\batik\gvt\TextNode.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       1.0.7
 */