/*     */ package org.apache.batik.gvt.flow;
/*     */ 
/*     */ import java.awt.font.FontRenderContext;
/*     */ import java.awt.font.TextAttribute;
/*     */ import java.text.AttributedCharacterIterator;
/*     */ import java.util.ArrayList;
/*     */ import java.util.Arrays;
/*     */ import java.util.HashSet;
/*     */ import java.util.Iterator;
/*     */ import java.util.LinkedList;
/*     */ import java.util.List;
/*     */ import java.util.Set;
/*     */ import org.apache.batik.gvt.TextNode;
/*     */ import org.apache.batik.gvt.TextPainter;
/*     */ import org.apache.batik.gvt.font.GVTFont;
/*     */ import org.apache.batik.gvt.font.GVTGlyphVector;
/*     */ import org.apache.batik.gvt.font.GVTLineMetrics;
/*     */ import org.apache.batik.gvt.font.MultiGlyphVector;
/*     */ import org.apache.batik.gvt.renderer.StrokingTextPainter;
/*     */ import org.apache.batik.gvt.text.GVTAttributedCharacterIterator;
/*     */ import org.apache.batik.gvt.text.GlyphLayout;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class FlowTextPainter
/*     */   extends StrokingTextPainter
/*     */ {
/*  55 */   protected static TextPainter singleton = new FlowTextPainter();
/*     */   public static final char SOFT_HYPHEN = '­';
/*     */   public static final char ZERO_WIDTH_SPACE = '​';
/*     */   public static final char ZERO_WIDTH_JOINER = '‍';
/*     */   public static final char SPACE = ' ';
/*     */   
/*  61 */   public static TextPainter getInstance() { return singleton; }
/*     */ 
/*     */   
/*     */   public List getTextRuns(TextNode node, AttributedCharacterIterator aci) {
/*  65 */     List textRuns = node.getTextRuns();
/*  66 */     if (textRuns != null) {
/*  67 */       return textRuns;
/*     */     }
/*     */     
/*  70 */     AttributedCharacterIterator[] chunkACIs = getTextChunkACIs(aci);
/*  71 */     textRuns = computeTextRuns(node, aci, chunkACIs);
/*     */     
/*  73 */     aci.first();
/*  74 */     List rgns = (List)aci.getAttribute(FLOW_REGIONS);
/*     */     
/*  76 */     if (rgns != null) {
/*  77 */       Iterator i = textRuns.iterator();
/*  78 */       List chunkLayouts = new ArrayList();
/*  79 */       StrokingTextPainter.TextRun tr = (StrokingTextPainter.TextRun)i.next();
/*  80 */       List layouts = new ArrayList();
/*  81 */       chunkLayouts.add(layouts);
/*  82 */       layouts.add(tr.getLayout());
/*  83 */       while (i.hasNext()) {
/*  84 */         tr = (StrokingTextPainter.TextRun)i.next();
/*  85 */         if (tr.isFirstRunInChunk()) {
/*  86 */           layouts = new ArrayList();
/*  87 */           chunkLayouts.add(layouts);
/*     */         } 
/*  89 */         layouts.add(tr.getLayout());
/*     */       } 
/*     */       
/*  92 */       textWrap(chunkACIs, chunkLayouts, rgns, this.fontRenderContext);
/*     */     } 
/*     */ 
/*     */     
/*  96 */     node.setTextRuns(textRuns);
/*  97 */     return node.getTextRuns();
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 105 */   public static final AttributedCharacterIterator.Attribute WORD_LIMIT = TextLineBreaks.WORD_LIMIT;
/*     */ 
/*     */   
/* 108 */   public static final AttributedCharacterIterator.Attribute FLOW_REGIONS = GVTAttributedCharacterIterator.TextAttribute.FLOW_REGIONS;
/*     */ 
/*     */   
/* 111 */   public static final AttributedCharacterIterator.Attribute FLOW_LINE_BREAK = GVTAttributedCharacterIterator.TextAttribute.FLOW_LINE_BREAK;
/*     */   
/* 113 */   public static final AttributedCharacterIterator.Attribute LINE_HEIGHT = GVTAttributedCharacterIterator.TextAttribute.LINE_HEIGHT;
/*     */ 
/*     */   
/* 116 */   public static final AttributedCharacterIterator.Attribute GVT_FONT = GVTAttributedCharacterIterator.TextAttribute.GVT_FONT;
/*     */ 
/*     */   
/* 119 */   protected static Set szAtts = new HashSet();
/*     */   
/*     */   static  {
/* 122 */     szAtts.add(TextAttribute.SIZE);
/* 123 */     szAtts.add(GVT_FONT);
/* 124 */     szAtts.add(LINE_HEIGHT);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static boolean textWrap(AttributedCharacterIterator[] acis, List chunkLayouts, List flowRects, FontRenderContext frc) {
/* 138 */     GVTGlyphVector[] gvs = new GVTGlyphVector[acis.length];
/* 139 */     WordInfo[][] wordInfos = new WordInfo[acis.length][];
/* 140 */     Iterator clIter = chunkLayouts.iterator();
/*     */     
/* 142 */     float prevBotMargin = 0.0F;
/* 143 */     int numWords = 0;
/* 144 */     BlockInfo[] blockInfos = new BlockInfo[acis.length];
/* 145 */     float[] topSkip = new float[acis.length];
/* 146 */     for (int chunk = 0; clIter.hasNext(); chunk++) {
/*     */       
/* 148 */       AttributedCharacterIterator aci = acis[chunk];
/* 149 */       List gvl = new LinkedList();
/* 150 */       List layouts = (List)clIter.next();
/* 151 */       Iterator iter = layouts.iterator();
/* 152 */       while (iter.hasNext()) {
/* 153 */         GlyphLayout gl = (GlyphLayout)iter.next();
/* 154 */         gvl.add(gl.getGlyphVector());
/*     */       } 
/* 156 */       MultiGlyphVector multiGlyphVector = new MultiGlyphVector(gvl);
/* 157 */       gvs[chunk] = multiGlyphVector;
/* 158 */       wordInfos[chunk] = doWordAnalysis(multiGlyphVector, aci, numWords, frc);
/* 159 */       aci.first();
/* 160 */       BlockInfo bi = (BlockInfo)aci.getAttribute(StrokingTextPainter.FLOW_PARAGRAPH);
/* 161 */       bi.initLineInfo(frc);
/* 162 */       blockInfos[chunk] = bi;
/* 163 */       if (prevBotMargin > bi.getTopMargin()) {
/* 164 */         topSkip[chunk] = prevBotMargin;
/*     */       } else {
/* 166 */         topSkip[chunk] = bi.getTopMargin();
/* 167 */       }  prevBotMargin = bi.getBottomMargin();
/* 168 */       numWords += wordInfos[chunk].length;
/*     */     } 
/*     */     
/* 171 */     Iterator frIter = flowRects.iterator();
/* 172 */     RegionInfo currentRegion = null;
/* 173 */     int currWord = 0;
/* 174 */     int chunk = 0;
/* 175 */     List lineInfos = new LinkedList();
/* 176 */     while (frIter.hasNext()) {
/* 177 */       currentRegion = (RegionInfo)frIter.next();
/* 178 */       FlowRegions fr = new FlowRegions(currentRegion.getShape());
/*     */       
/* 180 */       while (chunk < wordInfos.length) {
/* 181 */         WordInfo[] chunkInfo = wordInfos[chunk];
/* 182 */         BlockInfo bi = blockInfos[chunk];
/* 183 */         WordInfo wi = chunkInfo[currWord];
/* 184 */         Object flowLine = wi.getFlowLine();
/* 185 */         double lh = Math.max(wi.getLineHeight(), bi.getLineHeight());
/* 186 */         LineInfo li = new LineInfo(fr, bi, true);
/* 187 */         double newY = li.getCurrentY() + topSkip[chunk];
/* 188 */         topSkip[chunk] = 0.0F;
/* 189 */         if (li.gotoY(newY))
/*     */           break; 
/* 191 */         while (!li.addWord(wi)) {
/*     */           
/* 193 */           newY = li.getCurrentY() + lh * 0.1D;
/* 194 */           if (li.gotoY(newY))
/*     */             break; 
/* 196 */         }  if (fr.done())
/*     */           break; 
/* 198 */         currWord++;
/* 199 */         for (; currWord < chunkInfo.length; currWord++) {
/* 200 */           wi = chunkInfo[currWord];
/* 201 */           if (wi.getFlowLine() != flowLine || !li.addWord(wi))
/*     */           
/*     */           { 
/*     */ 
/*     */             
/* 206 */             li.layout();
/* 207 */             lineInfos.add(li);
/* 208 */             li = null;
/*     */             
/* 210 */             flowLine = wi.getFlowLine();
/* 211 */             lh = Math.max(wi.getLineHeight(), bi.getLineHeight());
/* 212 */             if (!fr.newLine(lh))
/*     */               break; 
/* 214 */             li = new LineInfo(fr, bi, false);
/* 215 */             while (!li.addWord(wi)) {
/* 216 */               newY = li.getCurrentY() + lh * 0.1D;
/* 217 */               if (li.gotoY(newY))
/*     */                 break; 
/* 219 */             }  if (fr.done())
/*     */               break;  } 
/* 221 */         }  if (li != null) {
/* 222 */           li.setParaEnd(true);
/* 223 */           li.layout();
/*     */         } 
/*     */         
/* 226 */         if (fr.done())
/*     */           break; 
/* 228 */         chunk++;
/* 229 */         currWord = 0;
/*     */         
/* 231 */         if (bi.isFlowRegionBreak()) {
/*     */           break;
/*     */         }
/* 234 */         if (!fr.newLine(lh))
/*     */           break; 
/*     */       } 
/* 237 */       if (chunk == wordInfos.length) {
/*     */         break;
/*     */       }
/*     */     } 
/* 241 */     boolean overflow = (chunk < wordInfos.length);
/*     */     
/* 243 */     while (chunk < wordInfos.length) {
/* 244 */       WordInfo[] chunkInfo = wordInfos[chunk];
/* 245 */       while (currWord < chunkInfo.length) {
/* 246 */         WordInfo wi = chunkInfo[currWord];
/* 247 */         int numGG = wi.getNumGlyphGroups();
/* 248 */         for (int gg = 0; gg < numGG; gg++) {
/* 249 */           GlyphGroupInfo ggi = wi.getGlyphGroup(gg);
/* 250 */           GVTGlyphVector gv = ggi.getGlyphVector();
/* 251 */           int end = ggi.getEnd();
/* 252 */           for (int g = ggi.getStart(); g <= end; g++) {
/* 253 */             gv.setGlyphVisible(g, false);
/*     */           }
/*     */         } 
/* 256 */         currWord++;
/*     */       } 
/* 258 */       chunk++;
/* 259 */       currWord = 0;
/*     */     } 
/*     */     
/* 262 */     return overflow;
/*     */   }
/*     */   
/*     */   static int[] allocWordMap(int[] wordMap, int sz) {
/* 266 */     if (wordMap != null) {
/* 267 */       if (sz <= wordMap.length) {
/* 268 */         return wordMap;
/*     */       }
/* 270 */       if (sz < wordMap.length * 2) {
/* 271 */         sz = wordMap.length * 2;
/*     */       }
/*     */     } 
/*     */ 
/*     */     
/* 276 */     int[] ret = new int[sz];
/* 277 */     int ext = (wordMap != null) ? wordMap.length : 0;
/* 278 */     if (sz < ext) {
/* 279 */       ext = sz;
/*     */     }
/* 281 */     if (ext != 0) {
/* 282 */       System.arraycopy(wordMap, 0, ret, 0, ext);
/*     */     }
/* 284 */     Arrays.fill(ret, ext, sz, -1);
/*     */     
/* 286 */     return ret;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   static WordInfo[] doWordAnalysis(GVTGlyphVector gv, AttributedCharacterIterator aci, int numWords, FontRenderContext frc) {
/* 297 */     int numGlyphs = gv.getNumGlyphs();
/* 298 */     int[] glyphWords = new int[numGlyphs];
/* 299 */     int[] wordMap = allocWordMap(null, 10);
/* 300 */     int maxWord = 0;
/* 301 */     int aciIdx = aci.getBeginIndex();
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 306 */     for (int i = 0; i < numGlyphs; i++) {
/* 307 */       int cnt = gv.getCharacterCount(i, i);
/* 308 */       aci.setIndex(aciIdx);
/* 309 */       Integer integer = (Integer)aci.getAttribute(WORD_LIMIT);
/* 310 */       int minWord = integer.intValue() - numWords;
/* 311 */       if (minWord > maxWord) {
/* 312 */         maxWord = minWord;
/* 313 */         wordMap = allocWordMap(wordMap, maxWord + 1);
/*     */       } 
/* 315 */       aciIdx++;
/* 316 */       for (int c = 1; c < cnt; c++) {
/* 317 */         aci.setIndex(aciIdx);
/* 318 */         integer = (Integer)aci.getAttribute(WORD_LIMIT);
/* 319 */         int cWord = integer.intValue() - numWords;
/* 320 */         if (cWord > maxWord) {
/* 321 */           maxWord = cWord;
/* 322 */           wordMap = allocWordMap(wordMap, maxWord + 1);
/*     */         } 
/*     */ 
/*     */         
/* 326 */         if (cWord < minWord) {
/* 327 */           wordMap[minWord] = cWord;
/* 328 */           minWord = cWord;
/* 329 */         } else if (cWord > minWord) {
/* 330 */           wordMap[cWord] = minWord;
/*     */         } 
/* 332 */         aciIdx++;
/*     */       } 
/* 334 */       glyphWords[i] = minWord;
/*     */     } 
/* 336 */     int words = 0;
/* 337 */     WordInfo[] cWordMap = new WordInfo[maxWord + 1];
/* 338 */     for (int i = 0; i <= maxWord; i++) {
/* 339 */       int nw = wordMap[i];
/* 340 */       if (nw == -1) {
/*     */         
/* 342 */         cWordMap[i] = new WordInfo(words++);
/*     */       } else {
/* 344 */         int word = nw;
/* 345 */         nw = wordMap[i];
/* 346 */         while (nw != -1) {
/* 347 */           word = nw;
/* 348 */           nw = wordMap[word];
/*     */         } 
/* 350 */         wordMap[i] = word;
/* 351 */         cWordMap[i] = cWordMap[word];
/*     */       } 
/*     */     } 
/* 354 */     wordMap = null;
/* 355 */     WordInfo[] wordInfos = new WordInfo[words];
/* 356 */     for (int i = 0; i <= maxWord; i++) {
/* 357 */       WordInfo wi = cWordMap[i];
/* 358 */       wordInfos[wi.getIndex()] = cWordMap[i];
/*     */     } 
/*     */     
/* 361 */     aciIdx = aci.getBeginIndex();
/* 362 */     int aciEnd = aci.getEndIndex();
/* 363 */     char ch = aci.setIndex(aciIdx);
/*     */     
/* 365 */     int aciWordStart = aciIdx;
/* 366 */     GVTFont gvtFont = (GVTFont)aci.getAttribute(GVT_FONT);
/* 367 */     float lineHeight = 1.0F;
/* 368 */     Float lineHeightFloat = (Float)aci.getAttribute(LINE_HEIGHT);
/* 369 */     if (lineHeightFloat != null)
/* 370 */       lineHeight = lineHeightFloat.floatValue(); 
/* 371 */     int runLimit = aci.getRunLimit(szAtts);
/* 372 */     WordInfo prevWI = null;
/* 373 */     float[] lastAdvAdj = new float[numGlyphs];
/* 374 */     float[] advAdj = new float[numGlyphs];
/* 375 */     boolean[] hideLast = new boolean[numGlyphs];
/* 376 */     boolean[] hide = new boolean[numGlyphs];
/* 377 */     boolean[] space = new boolean[numGlyphs];
/* 378 */     float[] glyphPos = gv.getGlyphPositions(0, numGlyphs + 1, null);
/* 379 */     for (int i = 0; i < numGlyphs; i++) {
/* 380 */       char pch = ch;
/* 381 */       ch = aci.setIndex(aciIdx);
/* 382 */       Integer integer = (Integer)aci.getAttribute(WORD_LIMIT);
/* 383 */       WordInfo theWI = cWordMap[integer.intValue() - numWords];
/* 384 */       if (theWI.getFlowLine() == null) {
/* 385 */         theWI.setFlowLine(aci.getAttribute(FLOW_LINE_BREAK));
/*     */       }
/* 387 */       if (prevWI == null) {
/* 388 */         prevWI = theWI;
/* 389 */       } else if (prevWI != theWI) {
/* 390 */         GVTLineMetrics lm = gvtFont.getLineMetrics(aci, aciWordStart, aciIdx, frc);
/*     */         
/* 392 */         prevWI.addLineMetrics(gvtFont, lm);
/* 393 */         prevWI.addLineHeight(lineHeight);
/* 394 */         aciWordStart = aciIdx;
/* 395 */         prevWI = theWI;
/*     */       } 
/*     */       
/* 398 */       int chCnt = gv.getCharacterCount(i, i);
/* 399 */       if (chCnt == 1) {
/*     */         float kern;
/*     */         char nch;
/* 402 */         switch (ch) {
/*     */           case '­':
/* 404 */             hideLast[i] = true;
/* 405 */             nch = aci.next(); aci.previous();
/* 406 */             kern = gvtFont.getHKern(pch, nch);
/* 407 */             advAdj[i] = -(glyphPos[2 * i + 2] - glyphPos[2 * i] + kern);
/*     */             break;
/*     */           case '‍':
/* 410 */             hide[i] = true;
/*     */             break;
/*     */           case '​':
/* 413 */             hide[i] = true;
/*     */             break;
/*     */           case ' ':
/* 416 */             space[i] = true;
/* 417 */             nch = aci.next(); aci.previous();
/* 418 */             kern = gvtFont.getHKern(pch, nch);
/* 419 */             lastAdvAdj[i] = -(glyphPos[2 * i + 2] - glyphPos[2 * i] + kern);
/*     */             break;
/*     */         } 
/*     */       
/*     */       } 
/* 424 */       aciIdx += chCnt;
/* 425 */       if (aciIdx > runLimit && aciIdx < aciEnd) {
/*     */ 
/*     */         
/* 428 */         GVTLineMetrics lm = gvtFont.getLineMetrics(aci, aciWordStart, runLimit, frc);
/*     */         
/* 430 */         prevWI.addLineMetrics(gvtFont, lm);
/* 431 */         prevWI.addLineHeight(lineHeight);
/* 432 */         prevWI = null;
/* 433 */         aciWordStart = aciIdx;
/* 434 */         aci.setIndex(aciIdx);
/* 435 */         gvtFont = (GVTFont)aci.getAttribute(GVT_FONT);
/* 436 */         Float f = (Float)aci.getAttribute(LINE_HEIGHT);
/* 437 */         lineHeight = f.floatValue();
/* 438 */         runLimit = aci.getRunLimit(szAtts);
/*     */       } 
/*     */     } 
/* 441 */     GVTLineMetrics lm = gvtFont.getLineMetrics(aci, aciWordStart, runLimit, frc);
/*     */     
/* 443 */     prevWI.addLineMetrics(gvtFont, lm);
/* 444 */     prevWI.addLineHeight(lineHeight);
/*     */     
/* 446 */     int[] wordGlyphCounts = new int[words];
/*     */     
/* 448 */     for (int i = 0; i < numGlyphs; i++) {
/* 449 */       int word = glyphWords[i];
/* 450 */       int cWord = cWordMap[word].getIndex();
/* 451 */       glyphWords[i] = cWord;
/* 452 */       wordGlyphCounts[cWord] = wordGlyphCounts[cWord] + 1;
/*     */     } 
/*     */     
/* 455 */     cWordMap = null;
/* 456 */     int[][] wordGlyphs = new int[words][];
/* 457 */     int[] wordGlyphGroupsCounts = new int[words];
/* 458 */     for (int i = 0; i < numGlyphs; i++) {
/* 459 */       int cWord = glyphWords[i];
/*     */       
/* 461 */       int[] wgs = wordGlyphs[cWord];
/* 462 */       if (wgs == null) {
/* 463 */         wgs = wordGlyphs[cWord] = new int[wordGlyphCounts[cWord]];
/*     */ 
/*     */ 
/*     */ 
/*     */         
/* 468 */         wordGlyphCounts[cWord] = 0;
/*     */       } 
/* 470 */       int cnt = wordGlyphCounts[cWord];
/* 471 */       wgs[cnt] = i;
/*     */       
/* 473 */       if (cnt == 0) {
/* 474 */         wordGlyphGroupsCounts[cWord] = wordGlyphGroupsCounts[cWord] + 1;
/*     */       }
/* 476 */       else if (wgs[cnt - 1] != i - 1) {
/* 477 */         wordGlyphGroupsCounts[cWord] = wordGlyphGroupsCounts[cWord] + 1;
/*     */       } 
/* 479 */       wordGlyphCounts[cWord] = wordGlyphCounts[cWord] + 1;
/*     */     } 
/*     */     
/* 482 */     for (int i = 0; i < words; i++) {
/* 483 */       int cnt = wordGlyphGroupsCounts[i];
/*     */       
/* 485 */       GlyphGroupInfo[] wordGlyphGroups = new GlyphGroupInfo[cnt];
/* 486 */       if (cnt == 1) {
/* 487 */         int[] glyphs = wordGlyphs[i];
/* 488 */         int start = glyphs[0];
/* 489 */         int end = glyphs[glyphs.length - 1];
/* 490 */         wordGlyphGroups[0] = new GlyphGroupInfo(gv, start, end, hide, hideLast[end], glyphPos, advAdj, lastAdvAdj, space);
/*     */       }
/*     */       else {
/*     */         
/* 494 */         int glyphGroup = 0;
/* 495 */         int[] glyphs = wordGlyphs[i];
/* 496 */         int prev = glyphs[0];
/* 497 */         int start = prev;
/* 498 */         for (int j = 1; j < glyphs.length; j++) {
/* 499 */           if (prev + 1 != glyphs[j]) {
/* 500 */             int end = glyphs[j - 1];
/* 501 */             wordGlyphGroups[glyphGroup] = new GlyphGroupInfo(gv, start, end, hide, hideLast[end], glyphPos, advAdj, lastAdvAdj, space);
/*     */ 
/*     */             
/* 504 */             start = glyphs[j];
/* 505 */             glyphGroup++;
/*     */           } 
/* 507 */           prev = glyphs[j];
/*     */         } 
/* 509 */         int end = glyphs[glyphs.length - 1];
/* 510 */         wordGlyphGroups[glyphGroup] = new GlyphGroupInfo(gv, start, end, hide, hideLast[end], glyphPos, advAdj, lastAdvAdj, space);
/*     */       } 
/*     */ 
/*     */       
/* 514 */       wordInfos[i].setGlyphGroups(wordGlyphGroups);
/*     */     } 
/* 516 */     return wordInfos;
/*     */   }
/*     */ }


/* Location:              C:\Program Files (x86)\IBM\WebSphere\AppServer\profiles\ctgAppSrv01\installedApps\ctgCell01\MAXIMO.ear\maximouiweb.war\WEB-INF\classes\!\com\ibm\tivoli\maximo\report\control\svgtools\batik-gvt.jar!\org\apache\batik\gvt\flow\FlowTextPainter.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       1.0.7
 */