/*     */ package org.apache.batik.gvt.font;
/*     */ 
/*     */ import java.awt.Font;
/*     */ import java.awt.Shape;
/*     */ import java.awt.font.FontRenderContext;
/*     */ import java.awt.font.GlyphMetrics;
/*     */ import java.awt.font.GlyphVector;
/*     */ import java.awt.font.TextAttribute;
/*     */ import java.awt.geom.AffineTransform;
/*     */ import java.awt.geom.Point2D;
/*     */ import java.awt.geom.Rectangle2D;
/*     */ import java.text.AttributedCharacterIterator;
/*     */ import java.text.CharacterIterator;
/*     */ import java.text.StringCharacterIterator;
/*     */ import java.util.HashMap;
/*     */ import java.util.Map;
/*     */ import org.apache.batik.gvt.text.ArabicTextHandler;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class AWTGVTFont
/*     */   implements GVTFont
/*     */ {
/*     */   protected Font awtFont;
/*     */   protected float size;
/*     */   protected float scale;
/*     */   public static final float FONT_SIZE = 48.0F;
/*     */   
/*     */   public AWTGVTFont(Font font) {
/*  57 */     this.size = font.getSize2D();
/*  58 */     this.awtFont = font.deriveFont(48.0F);
/*  59 */     this.scale = this.size / this.awtFont.getSize2D();
/*  60 */     initializeFontCache(this.awtFont);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public AWTGVTFont(Font font, float scale) {
/*  70 */     this.size = font.getSize2D() * scale;
/*  71 */     this.awtFont = font.deriveFont(48.0F);
/*  72 */     this.scale = this.size / this.awtFont.getSize2D();
/*  73 */     initializeFontCache(this.awtFont);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public AWTGVTFont(Map attributes) {
/*  82 */     Float sz = (Float)attributes.get(TextAttribute.SIZE);
/*  83 */     if (sz != null) {
/*  84 */       this.size = sz.floatValue();
/*  85 */       attributes.put(TextAttribute.SIZE, new Float(48.0F));
/*  86 */       this.awtFont = new Font(attributes);
/*     */     } else {
/*  88 */       this.awtFont = new Font(attributes);
/*  89 */       this.size = this.awtFont.getSize2D();
/*     */     } 
/*  91 */     this.scale = this.size / this.awtFont.getSize2D();
/*  92 */     initializeFontCache(this.awtFont);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public AWTGVTFont(String name, int style, int size) {
/* 103 */     this.awtFont = new Font(name, style, 48);
/* 104 */     this.size = size;
/* 105 */     this.scale = size / this.awtFont.getSize2D();
/* 106 */     initializeFontCache(this.awtFont);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 116 */   public boolean canDisplay(char c) { return this.awtFont.canDisplay(c); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 131 */   public int canDisplayUpTo(char[] text, int start, int limit) { return this.awtFont.canDisplayUpTo(text, start, limit); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 139 */   public int canDisplayUpTo(CharacterIterator iter, int start, int limit) { return this.awtFont.canDisplayUpTo(iter, start, limit); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 146 */   public int canDisplayUpTo(String str) { return this.awtFont.canDisplayUpTo(str); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public GVTGlyphVector createGlyphVector(FontRenderContext frc, char[] chars) {
/* 156 */     StringCharacterIterator sci = new StringCharacterIterator(new String(chars));
/*     */     
/* 158 */     GlyphVector gv = this.awtFont.createGlyphVector(frc, chars);
/* 159 */     return new AWTGVTGlyphVector(gv, this, this.scale, sci);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public GVTGlyphVector createGlyphVector(FontRenderContext frc, CharacterIterator ci) {
/* 169 */     if (ci instanceof AttributedCharacterIterator) {
/* 170 */       AttributedCharacterIterator aci = (AttributedCharacterIterator)ci;
/* 171 */       if (ArabicTextHandler.containsArabic(aci)) {
/* 172 */         String str = ArabicTextHandler.createSubstituteString(aci);
/*     */         
/* 174 */         return createGlyphVector(frc, str);
/*     */       } 
/*     */     } 
/* 177 */     GlyphVector gv = this.awtFont.createGlyphVector(frc, ci);
/* 178 */     return new AWTGVTGlyphVector(gv, this, this.scale, ci);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 188 */   public GVTGlyphVector createGlyphVector(FontRenderContext frc, int[] glyphCodes, CharacterIterator ci) { return new AWTGVTGlyphVector(this.awtFont.createGlyphVector(frc, glyphCodes), this, this.scale, ci); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public GVTGlyphVector createGlyphVector(FontRenderContext frc, String str) {
/* 200 */     StringCharacterIterator sci = new StringCharacterIterator(str);
/*     */     
/* 202 */     return new AWTGVTGlyphVector(this.awtFont.createGlyphVector(frc, str), this, this.scale, sci);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 211 */   public GVTFont deriveFont(float size) { return new AWTGVTFont(this.awtFont, size / this.size); }
/*     */ 
/*     */ 
/*     */   
/* 215 */   public String getFamilyName() { return this.awtFont.getFamily(); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 225 */   public GVTLineMetrics getLineMetrics(char[] chars, int beginIndex, int limit, FontRenderContext frc) { return new GVTLineMetrics(this.awtFont.getLineMetrics(chars, beginIndex, limit, frc), this.scale); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 236 */   public GVTLineMetrics getLineMetrics(CharacterIterator ci, int beginIndex, int limit, FontRenderContext frc) { return new GVTLineMetrics(this.awtFont.getLineMetrics(ci, beginIndex, limit, frc), this.scale); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 245 */   public GVTLineMetrics getLineMetrics(String str, FontRenderContext frc) { return new GVTLineMetrics(this.awtFont.getLineMetrics(str, frc), this.scale); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 255 */   public GVTLineMetrics getLineMetrics(String str, int beginIndex, int limit, FontRenderContext frc) { return new GVTLineMetrics(this.awtFont.getLineMetrics(str, beginIndex, limit, frc), this.scale); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 263 */   public float getSize() { return this.size; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 270 */   public float getHKern(int glyphCode1, int glyphCode2) { return 0.0F; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 277 */   public float getVKern(int glyphCode1, int glyphCode2) { return 0.0F; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static AWTGlyphGeometryCache.Value getGlyphGeometry(AWTGVTFont font, char c, GlyphVector gv, int glyphIndex, Point2D glyphPos) {
/* 296 */     AWTGlyphGeometryCache glyphCache = (AWTGlyphGeometryCache)fontCache.get(font.awtFont);
/*     */ 
/*     */     
/* 299 */     AWTGlyphGeometryCache.Value v = glyphCache.get(c);
/* 300 */     if (v == null) {
/* 301 */       Shape outline = gv.getGlyphOutline(glyphIndex);
/* 302 */       GlyphMetrics metrics = gv.getGlyphMetrics(glyphIndex);
/* 303 */       Rectangle2D gmB = metrics.getBounds2D();
/* 304 */       if (AWTGVTGlyphVector.outlinesPositioned()) {
/* 305 */         AffineTransform tr = AffineTransform.getTranslateInstance(-glyphPos.getX(), -glyphPos.getY());
/*     */         
/* 307 */         outline = tr.createTransformedShape(outline);
/*     */       } 
/* 309 */       v = new AWTGlyphGeometryCache.Value(outline, gmB);
/*     */       
/* 311 */       glyphCache.put(c, v);
/*     */     } 
/* 313 */     return v;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 320 */   static Map fontCache = new HashMap(11);
/*     */   
/*     */   static void initializeFontCache(Font awtFont) {
/* 323 */     if (!fontCache.containsKey(awtFont)) {
/* 324 */       fontCache.put(awtFont, new AWTGlyphGeometryCache());
/*     */     }
/*     */   }
/*     */ 
/*     */   
/* 329 */   static void putAWTGVTFont(AWTGVTFont font) { fontCache.put(font.awtFont, font); }
/*     */ 
/*     */ 
/*     */   
/* 333 */   static AWTGVTFont getAWTGVTFont(Font awtFont) { return (AWTGVTFont)fontCache.get(awtFont); }
/*     */ }


/* Location:              C:\Program Files (x86)\IBM\WebSphere\AppServer\profiles\ctgAppSrv01\installedApps\ctgCell01\MAXIMO.ear\maximouiweb.war\WEB-INF\classes\!\com\ibm\tivoli\maximo\report\control\svgtools\batik-gvt.jar!\org\apache\batik\gvt\font\AWTGVTFont.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       1.0.7
 */