/*     */ package org.apache.batik.gvt.font;
/*     */ 
/*     */ import java.awt.Graphics2D;
/*     */ import java.awt.Paint;
/*     */ import java.awt.RenderingHints;
/*     */ import java.awt.Shape;
/*     */ import java.awt.Stroke;
/*     */ import java.awt.font.FontRenderContext;
/*     */ import java.awt.font.GlyphJustificationInfo;
/*     */ import java.awt.font.GlyphVector;
/*     */ import java.awt.geom.AffineTransform;
/*     */ import java.awt.geom.GeneralPath;
/*     */ import java.awt.geom.Point2D;
/*     */ import java.awt.geom.Rectangle2D;
/*     */ import java.text.AttributedCharacterIterator;
/*     */ import java.text.CharacterIterator;
/*     */ import org.apache.batik.gvt.text.ArabicTextHandler;
/*     */ import org.apache.batik.gvt.text.GVTAttributedCharacterIterator;
/*     */ import org.apache.batik.gvt.text.TextPaintInfo;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class AWTGVTGlyphVector
/*     */   implements GVTGlyphVector
/*     */ {
/*  50 */   public static final AttributedCharacterIterator.Attribute PAINT_INFO = GVTAttributedCharacterIterator.TextAttribute.PAINT_INFO;
/*     */   
/*     */   private GlyphVector awtGlyphVector;
/*     */   
/*     */   private AWTGVTFont gvtFont;
/*     */   
/*     */   private CharacterIterator ci;
/*     */   
/*     */   private Point2D[] defaultGlyphPositions;
/*     */   
/*     */   private Point2D.Float[] glyphPositions;
/*     */   
/*     */   private AffineTransform[] glyphTransforms;
/*     */   
/*     */   private Shape[] glyphOutlines;
/*     */   
/*     */   private Shape[] glyphVisualBounds;
/*     */   
/*     */   private Shape[] glyphLogicalBounds;
/*     */   
/*     */   private boolean[] glyphVisible;
/*     */   
/*     */   private GVTGlyphMetrics[] glyphMetrics;
/*     */   
/*     */   private GeneralPath outline;
/*     */   
/*     */   private Rectangle2D visualBounds;
/*     */   
/*     */   private Rectangle2D logicalBounds;
/*     */   
/*     */   private Rectangle2D bounds2D;
/*     */   
/*     */   private float scaleFactor;
/*     */   
/*     */   private float ascent;
/*     */   
/*     */   private float descent;
/*     */   
/*     */   private TextPaintInfo cacheTPI;
/*     */   
/*     */   private static final boolean outlinesPositioned;
/*     */   
/*     */   private static final boolean drawGlyphVectorWorks;
/*     */   
/*     */   private static final boolean glyphVectorTransformWorks;
/*     */ 
/*     */   
/*     */   public AWTGVTGlyphVector(GlyphVector glyphVector, AWTGVTFont font, float scaleFactor, CharacterIterator ci) {
/*  98 */     this.awtGlyphVector = glyphVector;
/*  99 */     this.gvtFont = font;
/* 100 */     this.scaleFactor = scaleFactor;
/* 101 */     this.ci = ci;
/*     */     
/* 103 */     GVTLineMetrics lineMetrics = this.gvtFont.getLineMetrics("By", this.awtGlyphVector.getFontRenderContext());
/*     */ 
/*     */     
/* 106 */     this.ascent = lineMetrics.getAscent();
/* 107 */     this.descent = lineMetrics.getDescent();
/*     */     
/* 109 */     this.outline = null;
/* 110 */     this.visualBounds = null;
/* 111 */     this.logicalBounds = null;
/* 112 */     this.bounds2D = null;
/* 113 */     int numGlyphs = glyphVector.getNumGlyphs();
/* 114 */     this.glyphPositions = new Point2D.Float[numGlyphs + 1];
/* 115 */     this.glyphTransforms = new AffineTransform[numGlyphs];
/* 116 */     this.glyphOutlines = new Shape[numGlyphs];
/* 117 */     this.glyphVisualBounds = new Shape[numGlyphs];
/* 118 */     this.glyphLogicalBounds = new Shape[numGlyphs];
/* 119 */     this.glyphVisible = new boolean[numGlyphs];
/* 120 */     this.glyphMetrics = new GVTGlyphMetrics[numGlyphs];
/*     */     
/* 122 */     for (int i = 0; i < numGlyphs; i++) {
/* 123 */       this.glyphVisible[i] = true;
/*     */     }
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 131 */   public GVTFont getFont() { return this.gvtFont; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 138 */   public FontRenderContext getFontRenderContext() { return this.awtGlyphVector.getFontRenderContext(); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 145 */   public int getGlyphCode(int glyphIndex) { return this.awtGlyphVector.getGlyphCode(glyphIndex); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 153 */   public int[] getGlyphCodes(int beginGlyphIndex, int numEntries, int[] codeReturn) { return this.awtGlyphVector.getGlyphCodes(beginGlyphIndex, numEntries, codeReturn); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 162 */   public GlyphJustificationInfo getGlyphJustificationInfo(int glyphIndex) { return this.awtGlyphVector.getGlyphJustificationInfo(glyphIndex); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public Rectangle2D getBounds2D(AttributedCharacterIterator aci) {
/* 169 */     aci.first();
/* 170 */     TextPaintInfo tpi = (TextPaintInfo)aci.getAttribute(PAINT_INFO);
/* 171 */     if (this.bounds2D != null && TextPaintInfo.equivilent(tpi, this.cacheTPI))
/*     */     {
/* 173 */       return this.bounds2D;
/*     */     }
/* 175 */     if (tpi == null)
/* 176 */       return null; 
/* 177 */     if (!tpi.visible) {
/* 178 */       return null;
/*     */     }
/* 180 */     this.cacheTPI = new TextPaintInfo(tpi);
/* 181 */     Shape outline = null;
/* 182 */     if (tpi.fillPaint != null) {
/* 183 */       outline = getOutline();
/* 184 */       this.bounds2D = outline.getBounds2D();
/*     */     } 
/*     */ 
/*     */ 
/*     */     
/* 189 */     Stroke stroke = tpi.strokeStroke;
/* 190 */     Paint paint = tpi.strokePaint;
/* 191 */     if (stroke != null && paint != null) {
/* 192 */       if (outline == null)
/* 193 */         outline = getOutline(); 
/* 194 */       Rectangle2D strokeBounds = stroke.createStrokedShape(outline).getBounds2D();
/*     */       
/* 196 */       if (this.bounds2D == null) {
/* 197 */         this.bounds2D = strokeBounds;
/*     */       } else {
/*     */         
/* 200 */         this.bounds2D.add(strokeBounds);
/*     */       } 
/* 202 */     }  if (this.bounds2D == null) {
/* 203 */       return null;
/*     */     }
/* 205 */     if (this.bounds2D.getWidth() == 0.0D || this.bounds2D.getHeight() == 0.0D)
/*     */     {
/* 207 */       this.bounds2D = null;
/*     */     }
/* 209 */     return this.bounds2D;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public Rectangle2D getLogicalBounds() {
/* 217 */     if (this.logicalBounds == null)
/*     */     {
/* 219 */       computeGlyphLogicalBounds();
/*     */     }
/* 221 */     return this.logicalBounds;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public Shape getGlyphLogicalBounds(int glyphIndex) {
/* 229 */     if (this.glyphLogicalBounds[glyphIndex] == null && this.glyphVisible[glyphIndex])
/*     */     {
/*     */       
/* 232 */       computeGlyphLogicalBounds();
/*     */     }
/* 234 */     return this.glyphLogicalBounds[glyphIndex];
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   private void computeGlyphLogicalBounds() {
/* 244 */     Shape[] tempLogicalBounds = new Shape[getNumGlyphs()];
/* 245 */     boolean[] rotated = new boolean[getNumGlyphs()];
/*     */     
/* 247 */     double maxWidth = -1.0D;
/* 248 */     double maxHeight = -1.0D;
/* 249 */     for (int i = 0; i < getNumGlyphs(); i++) {
/*     */       
/* 251 */       if (!this.glyphVisible[i]) {
/*     */         
/* 253 */         tempLogicalBounds[i] = null;
/*     */       }
/*     */       else {
/*     */         
/* 257 */         AffineTransform glyphTransform = getGlyphTransform(i);
/* 258 */         GVTGlyphMetrics glyphMetrics = getGlyphMetrics(i);
/*     */         
/* 260 */         float glyphX = 0.0F;
/* 261 */         float glyphY = -this.ascent / this.scaleFactor;
/* 262 */         float glyphWidth = glyphMetrics.getHorizontalAdvance() / this.scaleFactor;
/*     */         
/* 264 */         float glyphHeight = glyphMetrics.getVerticalAdvance() / this.scaleFactor;
/*     */         
/* 266 */         Rectangle2D glyphBounds = new Rectangle2D.Double(glyphX, glyphY, glyphWidth, glyphHeight);
/*     */ 
/*     */ 
/*     */ 
/*     */         
/* 271 */         if (glyphBounds.isEmpty()) {
/* 272 */           if (i > 0) {
/*     */ 
/*     */             
/* 275 */             rotated[i] = rotated[i - 1];
/*     */           } else {
/* 277 */             rotated[i] = true;
/*     */           }
/*     */         
/*     */         } else {
/*     */           
/* 282 */           Point2D p1 = new Point2D.Double(glyphBounds.getMinX(), glyphBounds.getMinY());
/*     */           
/* 284 */           Point2D p2 = new Point2D.Double(glyphBounds.getMaxX(), glyphBounds.getMinY());
/*     */           
/* 286 */           Point2D p3 = new Point2D.Double(glyphBounds.getMinX(), glyphBounds.getMaxY());
/*     */ 
/*     */           
/* 289 */           Point2D gpos = getGlyphPosition(i);
/* 290 */           AffineTransform tr = AffineTransform.getTranslateInstance(gpos.getX(), gpos.getY());
/*     */ 
/*     */           
/* 293 */           if (glyphTransform != null)
/* 294 */             tr.concatenate(glyphTransform); 
/* 295 */           tr.scale(this.scaleFactor, this.scaleFactor);
/*     */           
/* 297 */           tempLogicalBounds[i] = tr.createTransformedShape(glyphBounds);
/*     */           
/* 299 */           Point2D tp1 = new Point2D.Double();
/* 300 */           Point2D tp2 = new Point2D.Double();
/* 301 */           Point2D tp3 = new Point2D.Double();
/* 302 */           tr.transform(p1, tp1);
/* 303 */           tr.transform(p2, tp2);
/* 304 */           tr.transform(p3, tp3);
/* 305 */           double tdx12 = tp1.getX() - tp2.getX();
/* 306 */           double tdx13 = tp1.getX() - tp3.getX();
/* 307 */           double tdy12 = tp1.getY() - tp2.getY();
/* 308 */           double tdy13 = tp1.getY() - tp3.getY();
/*     */           
/* 310 */           if ((Math.abs(tdx12) < 0.001D && Math.abs(tdy13) < 0.001D) || (Math.abs(tdx13) < 0.001D && Math.abs(tdy12) < 0.001D)) {
/*     */ 
/*     */             
/* 313 */             rotated[i] = false;
/*     */           } else {
/* 315 */             rotated[i] = true;
/*     */           } 
/*     */ 
/*     */           
/* 319 */           Rectangle2D rectBounds = tempLogicalBounds[i].getBounds2D();
/* 320 */           if (rectBounds.getWidth() > maxWidth)
/* 321 */             maxWidth = rectBounds.getWidth(); 
/* 322 */           if (rectBounds.getHeight() > maxHeight) {
/* 323 */             maxHeight = rectBounds.getHeight();
/*     */           }
/*     */         } 
/*     */       } 
/*     */     } 
/* 328 */     GeneralPath logicalBoundsPath = new GeneralPath();
/* 329 */     for (int i = 0; i < getNumGlyphs(); i++) {
/* 330 */       if (tempLogicalBounds[i] != null) {
/* 331 */         logicalBoundsPath.append(tempLogicalBounds[i], false);
/*     */       }
/*     */     } 
/*     */     
/* 335 */     this.logicalBounds = logicalBoundsPath.getBounds2D();
/*     */     
/* 337 */     if (this.logicalBounds.getHeight() < maxHeight * 1.5D) {
/*     */       
/* 339 */       for (int i = 0; i < getNumGlyphs(); i++) {
/*     */ 
/*     */         
/* 342 */         if (!rotated[i] && 
/* 343 */           tempLogicalBounds[i] != null)
/*     */         {
/* 345 */           Rectangle2D glyphBounds = tempLogicalBounds[i].getBounds2D();
/*     */           
/* 347 */           double x = glyphBounds.getMinX();
/* 348 */           double width = glyphBounds.getWidth();
/*     */           
/* 350 */           if (i < getNumGlyphs() - 1 && tempLogicalBounds[i + true] != null) {
/*     */ 
/*     */             
/* 353 */             Rectangle2D ngb = tempLogicalBounds[i + 1].getBounds2D();
/*     */             
/* 355 */             if (ngb.getX() > x) {
/* 356 */               double nw = ngb.getX() - x;
/* 357 */               if (nw < width * 1.15D && nw > width * 0.85D) {
/* 358 */                 double delta = (nw - width) * 0.5D;
/* 359 */                 width += delta;
/* 360 */                 ngb.setRect(ngb.getX() - delta, ngb.getY(), ngb.getWidth() + delta, ngb.getHeight());
/*     */               } 
/*     */             } 
/*     */           } 
/*     */           
/* 365 */           tempLogicalBounds[i] = new Rectangle2D.Double(x, this.logicalBounds.getMinY(), width, this.logicalBounds.getHeight());
/*     */         }
/*     */       
/*     */       } 
/* 369 */     } else if (this.logicalBounds.getWidth() < maxWidth * 1.5D) {
/*     */       
/* 371 */       for (int i = 0; i < getNumGlyphs(); i++) {
/*     */ 
/*     */         
/* 374 */         if (!rotated[i] && 
/* 375 */           tempLogicalBounds[i] != null) {
/*     */           
/* 377 */           Rectangle2D glyphBounds = tempLogicalBounds[i].getBounds2D();
/* 378 */           double y = glyphBounds.getMinY();
/* 379 */           double height = glyphBounds.getHeight();
/*     */           
/* 381 */           if (i < getNumGlyphs() - 1 && tempLogicalBounds[i + true] != null) {
/*     */ 
/*     */             
/* 384 */             Rectangle2D ngb = tempLogicalBounds[i + 1].getBounds2D();
/* 385 */             if (ngb.getY() > y) {
/* 386 */               double nh = ngb.getY() - y;
/* 387 */               if (nh < height * 1.15D && nh > height * 0.85D) {
/* 388 */                 double delta = (nh - height) * 0.5D;
/* 389 */                 height += delta;
/* 390 */                 ngb.setRect(ngb.getX(), ngb.getY() - delta, ngb.getWidth(), ngb.getHeight() + delta);
/*     */               } 
/*     */             } 
/*     */           } 
/*     */           
/* 395 */           tempLogicalBounds[i] = new Rectangle2D.Double(this.logicalBounds.getMinX(), y, this.logicalBounds.getWidth(), height);
/*     */         } 
/*     */       } 
/*     */     } 
/*     */ 
/*     */     
/* 401 */     System.arraycopy(tempLogicalBounds, 0, this.glyphLogicalBounds, 0, getNumGlyphs());
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public GVTGlyphMetrics getGlyphMetrics(int glyphIndex) {
/* 409 */     if (this.glyphMetrics[glyphIndex] != null) {
/* 410 */       return this.glyphMetrics[glyphIndex];
/*     */     }
/*     */     
/* 413 */     Point2D glyphPos = this.defaultGlyphPositions[glyphIndex];
/* 414 */     char c = this.ci.setIndex(this.ci.getBeginIndex() + glyphIndex);
/* 415 */     this.ci.setIndex(this.ci.getBeginIndex());
/* 416 */     AWTGlyphGeometryCache.Value v = AWTGVTFont.getGlyphGeometry(this.gvtFont, c, this.awtGlyphVector, glyphIndex, glyphPos);
/*     */     
/* 418 */     Rectangle2D gmB = v.getBounds2D();
/*     */ 
/*     */     
/* 421 */     Rectangle2D bounds = new Rectangle2D.Double(gmB.getX() * this.scaleFactor, gmB.getY() * this.scaleFactor, gmB.getWidth() * this.scaleFactor, gmB.getHeight() * this.scaleFactor);
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 428 */     float adv = (float)(this.defaultGlyphPositions[glyphIndex + 1].getX() - this.defaultGlyphPositions[glyphIndex].getX());
/*     */     
/* 430 */     this.glyphMetrics[glyphIndex] = new GVTGlyphMetrics(adv * this.scaleFactor, this.ascent + this.descent, bounds, (byte)0);
/*     */ 
/*     */ 
/*     */     
/* 434 */     return this.glyphMetrics[glyphIndex];
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public Shape getGlyphOutline(int glyphIndex) {
/* 442 */     if (this.glyphOutlines[glyphIndex] == null) {
/*     */ 
/*     */ 
/*     */ 
/*     */       
/* 447 */       Point2D glyphPos = this.defaultGlyphPositions[glyphIndex];
/* 448 */       char c = this.ci.setIndex(this.ci.getBeginIndex() + glyphIndex);
/* 449 */       this.ci.setIndex(this.ci.getBeginIndex());
/* 450 */       AWTGlyphGeometryCache.Value v = AWTGVTFont.getGlyphGeometry(this.gvtFont, c, this.awtGlyphVector, glyphIndex, glyphPos);
/*     */       
/* 452 */       Shape glyphOutline = v.getOutline();
/*     */ 
/*     */       
/* 455 */       AffineTransform tr = AffineTransform.getTranslateInstance(getGlyphPosition(glyphIndex).getX(), getGlyphPosition(glyphIndex).getY());
/*     */ 
/*     */ 
/*     */       
/* 459 */       AffineTransform glyphTransform = getGlyphTransform(glyphIndex);
/*     */       
/* 461 */       if (glyphTransform != null) {
/* 462 */         tr.concatenate(glyphTransform);
/*     */       }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */       
/* 485 */       tr.scale(this.scaleFactor, this.scaleFactor);
/* 486 */       this.glyphOutlines[glyphIndex] = tr.createTransformedShape(glyphOutline);
/*     */     } 
/*     */     
/* 489 */     return this.glyphOutlines[glyphIndex];
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   static  {
/* 503 */     s = System.getProperty("java.specification.version");
/* 504 */     if ("1.4".compareTo(s) <= 0) {
/* 505 */       outlinesPositioned = true;
/* 506 */       drawGlyphVectorWorks = true;
/* 507 */       glyphVectorTransformWorks = true;
/* 508 */     } else if ("Mac OS X".equals(System.getProperty("os.name"))) {
/* 509 */       outlinesPositioned = true;
/* 510 */       drawGlyphVectorWorks = false;
/* 511 */       glyphVectorTransformWorks = false;
/*     */     } else {
/* 513 */       outlinesPositioned = false;
/* 514 */       drawGlyphVectorWorks = true;
/* 515 */       glyphVectorTransformWorks = false;
/*     */     } 
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 522 */   static boolean outlinesPositioned() { return outlinesPositioned; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 531 */   public Rectangle2D getGlyphCellBounds(int glyphIndex) { return getGlyphLogicalBounds(glyphIndex).getBounds2D(); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 538 */   public Point2D getGlyphPosition(int glyphIndex) { return this.glyphPositions[glyphIndex]; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public float[] getGlyphPositions(int beginGlyphIndex, int numEntries, float[] positionReturn) {
/* 548 */     if (positionReturn == null) {
/* 549 */       positionReturn = new float[numEntries * 2];
/*     */     }
/*     */     
/* 552 */     for (int i = beginGlyphIndex; i < beginGlyphIndex + numEntries; i++) {
/* 553 */       Point2D glyphPos = getGlyphPosition(i);
/* 554 */       positionReturn[(i - beginGlyphIndex) * 2] = (float)glyphPos.getX();
/* 555 */       positionReturn[(i - beginGlyphIndex) * 2 + 1] = (float)glyphPos.getY();
/*     */     } 
/*     */     
/* 558 */     return positionReturn;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 565 */   public AffineTransform getGlyphTransform(int glyphIndex) { return this.glyphTransforms[glyphIndex]; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public Shape getGlyphVisualBounds(int glyphIndex) {
/* 572 */     if (this.glyphVisualBounds[glyphIndex] == null) {
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */       
/* 578 */       Point2D glyphPos = this.defaultGlyphPositions[glyphIndex];
/* 579 */       char c = this.ci.setIndex(this.ci.getBeginIndex() + glyphIndex);
/* 580 */       this.ci.setIndex(this.ci.getBeginIndex());
/* 581 */       AWTGlyphGeometryCache.Value v = AWTGVTFont.getGlyphGeometry(this.gvtFont, c, this.awtGlyphVector, glyphIndex, glyphPos);
/*     */       
/* 583 */       Rectangle2D glyphBounds = v.getOutlineBounds2D();
/*     */ 
/*     */       
/* 586 */       AffineTransform tr = AffineTransform.getTranslateInstance(getGlyphPosition(glyphIndex).getX(), getGlyphPosition(glyphIndex).getY());
/*     */ 
/*     */ 
/*     */       
/* 590 */       AffineTransform glyphTransform = getGlyphTransform(glyphIndex);
/* 591 */       if (glyphTransform != null) {
/* 592 */         tr.concatenate(glyphTransform);
/*     */       }
/* 594 */       tr.scale(this.scaleFactor, this.scaleFactor);
/* 595 */       this.glyphVisualBounds[glyphIndex] = tr.createTransformedShape(glyphBounds);
/*     */     } 
/*     */ 
/*     */     
/* 599 */     return this.glyphVisualBounds[glyphIndex];
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 606 */   public int getNumGlyphs() { return this.awtGlyphVector.getNumGlyphs(); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public Shape getOutline() {
/* 614 */     if (this.outline != null) {
/* 615 */       return this.outline;
/*     */     }
/* 617 */     this.outline = new GeneralPath();
/* 618 */     for (int i = 0; i < getNumGlyphs(); i++) {
/* 619 */       if (this.glyphVisible[i]) {
/* 620 */         Shape glyphOutline = getGlyphOutline(i);
/* 621 */         this.outline.append(glyphOutline, false);
/*     */       } 
/*     */     } 
/* 624 */     return this.outline;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public Shape getOutline(float x, float y) {
/* 632 */     outline = getOutline();
/* 633 */     AffineTransform tr = AffineTransform.getTranslateInstance(x, y);
/* 634 */     return tr.createTransformedShape(outline);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public Rectangle2D getGeometricBounds() {
/* 644 */     if (this.visualBounds == null) {
/* 645 */       Shape outline = getOutline();
/* 646 */       this.visualBounds = outline.getBounds2D();
/*     */     } 
/* 648 */     return this.visualBounds;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void performDefaultLayout() {
/* 655 */     if (this.defaultGlyphPositions == null) {
/* 656 */       this.awtGlyphVector.performDefaultLayout();
/* 657 */       this.defaultGlyphPositions = new Point2D.Float[getNumGlyphs() + 1];
/* 658 */       for (int i = 0; i <= getNumGlyphs(); i++) {
/* 659 */         this.defaultGlyphPositions[i] = this.awtGlyphVector.getGlyphPosition(i);
/*     */       }
/*     */     } 
/* 662 */     this.outline = null;
/* 663 */     this.visualBounds = null;
/* 664 */     this.logicalBounds = null;
/* 665 */     this.bounds2D = null;
/* 666 */     float shiftLeft = 0.0F;
/* 667 */     int i = 0;
/* 668 */     for (; i < getNumGlyphs(); i++) {
/* 669 */       this.glyphTransforms[i] = null;
/* 670 */       this.glyphVisualBounds[i] = null;
/* 671 */       this.glyphLogicalBounds[i] = null;
/* 672 */       this.glyphOutlines[i] = null;
/* 673 */       this.glyphMetrics[i] = null;
/* 674 */       Point2D glyphPos = this.defaultGlyphPositions[i];
/* 675 */       float x = (float)(glyphPos.getX() * this.scaleFactor - shiftLeft);
/* 676 */       float y = (float)(glyphPos.getY() * this.scaleFactor);
/*     */ 
/*     */ 
/*     */       
/* 680 */       this.ci.setIndex(i + this.ci.getBeginIndex());
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */       
/* 713 */       if (this.glyphPositions[i] == null) {
/* 714 */         this.glyphPositions[i] = new Point2D.Float(x, y);
/*     */       } else {
/* 716 */         (this.glyphPositions[i]).x = x;
/* 717 */         (this.glyphPositions[i]).y = y;
/*     */       } 
/*     */     } 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 724 */     Point2D glyphPos = this.defaultGlyphPositions[i];
/* 725 */     this.glyphPositions[i] = new Point2D.Float((float)(glyphPos.getX() * this.scaleFactor - shiftLeft), (float)(glyphPos.getY() * this.scaleFactor));
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void setGlyphPosition(int glyphIndex, Point2D newPos) {
/* 734 */     (this.glyphPositions[glyphIndex]).x = (float)newPos.getX();
/* 735 */     (this.glyphPositions[glyphIndex]).y = (float)newPos.getY();
/* 736 */     this.outline = null;
/* 737 */     this.visualBounds = null;
/* 738 */     this.logicalBounds = null;
/* 739 */     this.bounds2D = null;
/*     */     
/* 741 */     if (glyphIndex != getNumGlyphs()) {
/* 742 */       this.glyphVisualBounds[glyphIndex] = null;
/* 743 */       this.glyphLogicalBounds[glyphIndex] = null;
/* 744 */       this.glyphOutlines[glyphIndex] = null;
/* 745 */       this.glyphMetrics[glyphIndex] = null;
/*     */     } 
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void setGlyphTransform(int glyphIndex, AffineTransform newTX) {
/* 753 */     this.glyphTransforms[glyphIndex] = newTX;
/* 754 */     this.outline = null;
/* 755 */     this.visualBounds = null;
/* 756 */     this.logicalBounds = null;
/* 757 */     this.bounds2D = null;
/*     */     
/* 759 */     this.glyphVisualBounds[glyphIndex] = null;
/* 760 */     this.glyphLogicalBounds[glyphIndex] = null;
/* 761 */     this.glyphOutlines[glyphIndex] = null;
/* 762 */     this.glyphMetrics[glyphIndex] = null;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void setGlyphVisible(int glyphIndex, boolean visible) {
/* 769 */     if (visible == this.glyphVisible[glyphIndex])
/*     */       return; 
/* 771 */     this.glyphVisible[glyphIndex] = visible;
/* 772 */     this.outline = null;
/* 773 */     this.visualBounds = null;
/* 774 */     this.logicalBounds = null;
/* 775 */     this.bounds2D = null;
/*     */     
/* 777 */     this.glyphVisualBounds[glyphIndex] = null;
/* 778 */     this.glyphLogicalBounds[glyphIndex] = null;
/* 779 */     this.glyphOutlines[glyphIndex] = null;
/* 780 */     this.glyphMetrics[glyphIndex] = null;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 787 */   public boolean isGlyphVisible(int glyphIndex) { return this.glyphVisible[glyphIndex]; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public int getCharacterCount(int startGlyphIndex, int endGlyphIndex) {
/* 799 */     if (startGlyphIndex < 0) {
/* 800 */       startGlyphIndex = 0;
/*     */     }
/* 802 */     if (endGlyphIndex >= getNumGlyphs()) {
/* 803 */       endGlyphIndex = getNumGlyphs() - 1;
/*     */     }
/* 805 */     int charCount = 0;
/* 806 */     int start = startGlyphIndex + this.ci.getBeginIndex();
/* 807 */     int end = endGlyphIndex + this.ci.getBeginIndex();
/*     */     
/* 809 */     for (char c = this.ci.setIndex(start); this.ci.getIndex() <= end; c = this.ci.next()) {
/* 810 */       charCount += ArabicTextHandler.getNumChars(c);
/*     */     }
/*     */     
/* 813 */     return charCount;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void draw(Graphics2D graphics2D, AttributedCharacterIterator aci) {
/* 821 */     int numGlyphs = getNumGlyphs();
/*     */     
/* 823 */     aci.first();
/* 824 */     TextPaintInfo tpi = (TextPaintInfo)aci.getAttribute(GVTAttributedCharacterIterator.TextAttribute.PAINT_INFO);
/*     */     
/* 826 */     if (tpi == null)
/* 827 */       return;  if (!tpi.visible)
/*     */       return; 
/* 829 */     Paint fillPaint = tpi.fillPaint;
/* 830 */     Stroke stroke = tpi.strokeStroke;
/* 831 */     Paint strokePaint = tpi.strokePaint;
/*     */     
/* 833 */     if (fillPaint == null && (strokePaint == null || stroke == null)) {
/*     */       return;
/*     */     }
/*     */     
/* 837 */     boolean useHinting = drawGlyphVectorWorks;
/* 838 */     if (useHinting && stroke != null && strokePaint != null)
/*     */     {
/* 840 */       useHinting = false;
/*     */     }
/* 842 */     if (useHinting && fillPaint != null && !(fillPaint instanceof java.awt.Color))
/*     */     {
/*     */ 
/*     */       
/* 846 */       useHinting = false;
/*     */     }
/* 848 */     if (useHinting) {
/* 849 */       Object v1 = graphics2D.getRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING);
/*     */       
/* 851 */       Object v2 = graphics2D.getRenderingHint(RenderingHints.KEY_STROKE_CONTROL);
/*     */ 
/*     */       
/* 854 */       if (v1 == RenderingHints.VALUE_TEXT_ANTIALIAS_ON && v2 == RenderingHints.VALUE_STROKE_PURE)
/*     */       {
/* 856 */         useHinting = false;
/*     */       }
/*     */     } 
/* 859 */     int typeGRot = 16;
/* 860 */     int typeGTrans = 32;
/*     */     
/* 862 */     if (useHinting) {
/*     */ 
/*     */       
/* 865 */       AffineTransform at = graphics2D.getTransform();
/* 866 */       int type = at.getType();
/* 867 */       if ((type & 0x20) != 0 || (type & 0x10) != 0) {
/* 868 */         useHinting = false;
/*     */       }
/*     */     } 
/* 871 */     if (useHinting) {
/* 872 */       for (int i = 0; i < numGlyphs; i++) {
/* 873 */         if (!this.glyphVisible[i]) {
/* 874 */           useHinting = false;
/*     */           break;
/*     */         } 
/* 877 */         AffineTransform at = this.glyphTransforms[i];
/* 878 */         if (at != null) {
/* 879 */           int type = at.getType();
/* 880 */           if ((type & 0xFFFFFFFE) != 0)
/*     */           {
/* 882 */             if (!glyphVectorTransformWorks || (type & 0x20) != 0 || (type & 0x10) != 0) {
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */               
/* 890 */               useHinting = false;
/*     */               break;
/*     */             } 
/*     */           }
/*     */         } 
/*     */       } 
/*     */     }
/* 897 */     if (useHinting) {
/* 898 */       double sf = this.scaleFactor;
/* 899 */       double[] mat = new double[6];
/* 900 */       for (int i = 0; i < numGlyphs; i++) {
/* 901 */         Point2D pos = this.glyphPositions[i];
/* 902 */         double x = pos.getX();
/* 903 */         double y = pos.getY();
/* 904 */         AffineTransform at = this.glyphTransforms[i];
/* 905 */         if (at != null) {
/*     */ 
/*     */           
/* 908 */           at.getMatrix(mat);
/* 909 */           x += mat[4];
/* 910 */           y += mat[5];
/* 911 */           if (mat[0] != 1.0D || mat[1] != 0.0D || mat[2] != 0.0D || mat[3] != 1.0D) {
/*     */ 
/*     */             
/* 914 */             mat[4] = 0.0D; mat[5] = 0.0D;
/* 915 */             at = new AffineTransform(mat);
/*     */           } else {
/* 917 */             at = null;
/*     */           } 
/*     */         } 
/* 920 */         pos = new Point2D.Double(x / sf, y / sf);
/* 921 */         this.awtGlyphVector.setGlyphPosition(i, pos);
/* 922 */         this.awtGlyphVector.setGlyphTransform(i, at);
/*     */       } 
/* 924 */       graphics2D.scale(sf, sf);
/* 925 */       graphics2D.setPaint(fillPaint);
/* 926 */       graphics2D.drawGlyphVector(this.awtGlyphVector, 0.0F, 0.0F);
/* 927 */       graphics2D.scale(1.0D / sf, 1.0D / sf);
/*     */       
/* 929 */       for (int i = 0; i < numGlyphs; i++) {
/* 930 */         Point2D pos = this.defaultGlyphPositions[i];
/* 931 */         this.awtGlyphVector.setGlyphPosition(i, pos);
/* 932 */         this.awtGlyphVector.setGlyphTransform(i, null);
/*     */       } 
/*     */     } else {
/*     */       
/* 936 */       Shape outline = getOutline();
/*     */ 
/*     */       
/* 939 */       if (fillPaint != null) {
/* 940 */         graphics2D.setPaint(fillPaint);
/* 941 */         graphics2D.fill(outline);
/*     */       } 
/*     */ 
/*     */       
/* 945 */       if (stroke != null && strokePaint != null) {
/* 946 */         graphics2D.setStroke(stroke);
/* 947 */         graphics2D.setPaint(strokePaint);
/* 948 */         graphics2D.draw(outline);
/*     */       } 
/*     */     } 
/*     */   }
/*     */ }


/* Location:              C:\Program Files (x86)\IBM\WebSphere\AppServer\profiles\ctgAppSrv01\installedApps\ctgCell01\MAXIMO.ear\maximouiweb.war\WEB-INF\classes\!\com\ibm\tivoli\maximo\report\control\svgtools\batik-gvt.jar!\org\apache\batik\gvt\font\AWTGVTGlyphVector.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       1.0.7
 */