/*     */ package org.apache.batik.gvt.font;
/*     */ 
/*     */ import java.awt.Shape;
/*     */ import java.awt.geom.Rectangle2D;
/*     */ import java.lang.ref.ReferenceQueue;
/*     */ import java.lang.ref.SoftReference;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class AWTGlyphGeometryCache
/*     */ {
/*     */   protected static final int INITIAL_CAPACITY = 71;
/*     */   protected Entry[] table;
/*     */   protected int count;
/*  54 */   protected ReferenceQueue referenceQueue = new ReferenceQueue();
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*  60 */   public AWTGlyphGeometryCache() { this.table = new Entry[71]; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*  68 */   public AWTGlyphGeometryCache(int c) { this.table = new Entry[c]; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*  75 */   public int size() { return this.count; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public Value get(char c) {
/*  83 */     int hash = hashCode(c) & 0x7FFFFFFF;
/*  84 */     int index = hash % this.table.length;
/*     */     
/*  86 */     for (Entry e = this.table[index]; e != null; e = e.next) {
/*  87 */       if (e.hash == hash && e.match(c)) {
/*  88 */         return (Value)e.get();
/*     */       }
/*     */     } 
/*  91 */     return null;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public Value put(char c, Value value) {
/*  99 */     removeClearedEntries();
/*     */     
/* 101 */     int hash = hashCode(c) & 0x7FFFFFFF;
/* 102 */     int index = hash % this.table.length;
/*     */     
/* 104 */     Entry e = this.table[index];
/* 105 */     if (e != null) {
/* 106 */       if (e.hash == hash && e.match(c)) {
/* 107 */         Object old = e.get();
/* 108 */         this.table[index] = new Entry(this, hash, c, value, e.next);
/* 109 */         return (Value)old;
/*     */       } 
/* 111 */       Entry o = e;
/* 112 */       e = e.next;
/* 113 */       while (e != null) {
/* 114 */         if (e.hash == hash && e.match(c)) {
/* 115 */           Object old = e.get();
/* 116 */           e = new Entry(this, hash, c, value, e.next);
/* 117 */           o.next = e;
/* 118 */           return (Value)old;
/*     */         } 
/*     */         
/* 121 */         o = e;
/* 122 */         e = e.next;
/*     */       } 
/*     */     } 
/*     */ 
/*     */     
/* 127 */     int len = this.table.length;
/* 128 */     if (this.count++ >= len - (len >> 2)) {
/*     */       
/* 130 */       rehash();
/* 131 */       index = hash % this.table.length;
/*     */     } 
/*     */     
/* 134 */     this.table[index] = new Entry(this, hash, c, value, this.table[index]);
/* 135 */     return null;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void clear() {
/* 142 */     this.table = new Entry[71];
/* 143 */     this.count = 0;
/* 144 */     this.referenceQueue = new ReferenceQueue();
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected void rehash() {
/* 151 */     Entry[] arrayOfEntry = this.table;
/*     */     
/* 153 */     this.table = new Entry[arrayOfEntry.length * 2 + 1];
/*     */     
/* 155 */     for (int i = arrayOfEntry.length - 1; i >= 0; i--) {
/* 156 */       for (Entry old = arrayOfEntry[i]; old != null; ) {
/* 157 */         Entry e = old;
/* 158 */         old = old.next;
/*     */         
/* 160 */         int index = e.hash % this.table.length;
/* 161 */         e.next = this.table[index];
/* 162 */         this.table[index] = e;
/*     */       } 
/*     */     } 
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 171 */   protected int hashCode(char c) { return c; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected void removeClearedEntries() {
/*     */     Entry e;
/* 179 */     while ((e = (Entry)this.referenceQueue.poll()) != null) {
/* 180 */       int index = e.hash % this.table.length;
/* 181 */       Entry t = this.table[index];
/* 182 */       if (t == e) {
/* 183 */         this.table[index] = e.next;
/*     */       } else {
/* 185 */         while (t != null) {
/* 186 */           Entry c = t.next;
/* 187 */           if (c == e) {
/* 188 */             t.next = e.next;
/*     */             break;
/*     */           } 
/* 191 */           t = c;
/*     */         } 
/*     */       } 
/* 194 */       this.count--;
/*     */     } 
/*     */   }
/*     */ 
/*     */ 
/*     */   
/*     */   public static class Value
/*     */   {
/*     */     protected Shape outline;
/*     */ 
/*     */     
/*     */     protected Rectangle2D gmB;
/*     */     
/*     */     protected Rectangle2D outlineBounds;
/*     */ 
/*     */     
/*     */     public Value(Shape outline, Rectangle2D gmB) {
/* 211 */       this.outline = outline;
/* 212 */       this.outlineBounds = outline.getBounds2D();
/* 213 */       this.gmB = gmB;
/*     */     }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 220 */     public Shape getOutline() { return this.outline; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 227 */     public Rectangle2D getBounds2D() { return this.gmB; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 234 */     public Rectangle2D getOutlineBounds2D() { return this.outlineBounds; }
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected class Entry
/*     */     extends SoftReference
/*     */   {
/*     */     public int hash;
/*     */ 
/*     */ 
/*     */     
/*     */     public char c;
/*     */ 
/*     */ 
/*     */     
/*     */     public Entry next;
/*     */ 
/*     */ 
/*     */     
/*     */     private final AWTGlyphGeometryCache this$0;
/*     */ 
/*     */ 
/*     */ 
/*     */     
/*     */     public Entry(AWTGlyphGeometryCache this$0, int hash, char c, AWTGlyphGeometryCache.Value value, Entry next) {
/* 262 */       super(value, this$0.referenceQueue); this.this$0 = this$0;
/* 263 */       this.hash = hash;
/* 264 */       this.c = c;
/* 265 */       this.next = next;
/*     */     }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 272 */     public boolean match(char o2) { return (this.c == o2); }
/*     */   }
/*     */ }


/* Location:              C:\Program Files (x86)\IBM\WebSphere\AppServer\profiles\ctgAppSrv01\installedApps\ctgCell01\MAXIMO.ear\maximouiweb.war\WEB-INF\classes\!\com\ibm\tivoli\maximo\report\control\svgtools\batik-gvt.jar!\org\apache\batik\gvt\font\AWTGlyphGeometryCache.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       1.0.7
 */