/*     */ package org.apache.batik.gvt.font;
/*     */ 
/*     */ import java.awt.GraphicsEnvironment;
/*     */ import java.util.ArrayList;
/*     */ import java.util.Collection;
/*     */ import java.util.HashMap;
/*     */ import java.util.Iterator;
/*     */ import java.util.List;
/*     */ import java.util.Map;
/*     */ import java.util.StringTokenizer;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class FontFamilyResolver
/*     */ {
/*  42 */   public static final AWTFontFamily defaultFont = new AWTFontFamily("SansSerif");
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*  49 */   protected static final Map fonts = new HashMap();
/*     */   
/*  51 */   protected static final List awtFontFamilies = new ArrayList();
/*  52 */   protected static final List awtFonts = new ArrayList();
/*     */   
/*     */   protected static final Map resolvedFontFamilies;
/*     */ 
/*     */   
/*     */   static  {
/*  58 */     fonts.put("sans-serif", "SansSerif");
/*  59 */     fonts.put("serif", "Serif");
/*  60 */     fonts.put("times", "Serif");
/*  61 */     fonts.put("times new roman", "Serif");
/*  62 */     fonts.put("cursive", "Dialog");
/*  63 */     fonts.put("fantasy", "Symbol");
/*  64 */     fonts.put("monospace", "Monospaced");
/*  65 */     fonts.put("monospaced", "Monospaced");
/*  66 */     fonts.put("courier", "Monospaced");
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/*  73 */     env = GraphicsEnvironment.getLocalGraphicsEnvironment();
/*  74 */     String[] fontNames = env.getAvailableFontFamilyNames();
/*     */     
/*  76 */     int nFonts = (fontNames != null) ? fontNames.length : 0;
/*  77 */     for (int i = 0; i < nFonts; i++) {
/*  78 */       fonts.put(fontNames[i].toLowerCase(), fontNames[i]);
/*     */ 
/*     */       
/*  81 */       StringTokenizer st = new StringTokenizer(fontNames[i]);
/*  82 */       String fontNameWithoutSpaces = "";
/*  83 */       while (st.hasMoreTokens()) {
/*  84 */         fontNameWithoutSpaces = fontNameWithoutSpaces + st.nextToken();
/*     */       }
/*  86 */       fonts.put(fontNameWithoutSpaces.toLowerCase(), fontNames[i]);
/*     */ 
/*     */       
/*  89 */       String fontNameWithDashes = fontNames[i].replace(' ', '-');
/*  90 */       if (!fontNameWithDashes.equals(fontNames[i])) {
/*  91 */         fonts.put(fontNameWithDashes.toLowerCase(), fontNames[i]);
/*     */       }
/*     */     } 
/*     */ 
/*     */     
/*  96 */     awtFontFamilies.add(defaultFont);
/*  97 */     awtFonts.add(new AWTGVTFont(defaultFont.getFamilyName(), 0, 12));
/*     */     
/*  99 */     Collection fontValues = fonts.values();
/* 100 */     Iterator iter = fontValues.iterator();
/* 101 */     while (iter.hasNext()) {
/* 102 */       String fontFamily = (String)iter.next();
/* 103 */       AWTFontFamily awtFontFamily = new AWTFontFamily(fontFamily);
/* 104 */       awtFontFamilies.add(awtFontFamily);
/* 105 */       AWTGVTFont font = new AWTGVTFont(fontFamily, 0, 12);
/* 106 */       awtFonts.add(font);
/*     */     } 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 115 */     resolvedFontFamilies = new HashMap();
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 126 */   public static String lookup(String familyName) { return (String)fonts.get(familyName.toLowerCase()); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static GVTFontFamily resolve(String familyName) {
/* 140 */     familyName = familyName.toLowerCase();
/*     */ 
/*     */     
/* 143 */     GVTFontFamily resolvedFF = (GVTFontFamily)resolvedFontFamilies.get(familyName);
/*     */ 
/*     */     
/* 146 */     if (resolvedFF == null) {
/*     */ 
/*     */       
/* 149 */       String awtFamilyName = (String)fonts.get(familyName);
/* 150 */       if (awtFamilyName != null) {
/* 151 */         resolvedFF = new AWTFontFamily(awtFamilyName);
/*     */       }
/*     */       
/* 154 */       resolvedFontFamilies.put(familyName, resolvedFF);
/*     */     } 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 164 */     return resolvedFF;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 178 */   public static GVTFontFamily resolve(UnresolvedFontFamily fontFamily) { return resolve(fontFamily.getFamilyName()); }
/*     */ 
/*     */   
/*     */   public static GVTFontFamily getFamilyThatCanDisplay(char c) {
/* 182 */     for (int i = 0; i < awtFontFamilies.size(); i++) {
/* 183 */       AWTFontFamily fontFamily = (AWTFontFamily)awtFontFamilies.get(i);
/* 184 */       AWTGVTFont font = (AWTGVTFont)awtFonts.get(i);
/* 185 */       if (font.canDisplay(c) && fontFamily.getFamilyName().indexOf("Song") == -1)
/*     */       {
/* 187 */         return fontFamily;
/*     */       }
/*     */     } 
/*     */     
/* 191 */     return null;
/*     */   }
/*     */ }


/* Location:              C:\Program Files (x86)\IBM\WebSphere\AppServer\profiles\ctgAppSrv01\installedApps\ctgCell01\MAXIMO.ear\maximouiweb.war\WEB-INF\classes\!\com\ibm\tivoli\maximo\report\control\svgtools\batik-gvt.jar!\org\apache\batik\gvt\font\FontFamilyResolver.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       1.0.7
 */