/*     */ package org.apache.batik.gvt.renderer;
/*     */ 
/*     */ import java.awt.Color;
/*     */ import java.awt.Graphics2D;
/*     */ import java.awt.Point;
/*     */ import java.awt.Rectangle;
/*     */ import java.awt.RenderingHints;
/*     */ import java.awt.geom.AffineTransform;
/*     */ import java.awt.image.BufferedImage;
/*     */ import java.awt.image.Raster;
/*     */ import java.awt.image.SampleModel;
/*     */ import java.awt.image.WritableRaster;
/*     */ import java.util.Collection;
/*     */ import java.util.Iterator;
/*     */ import org.apache.batik.ext.awt.geom.RectListManager;
/*     */ import org.apache.batik.ext.awt.image.GraphicsUtil;
/*     */ import org.apache.batik.ext.awt.image.PadMode;
/*     */ import org.apache.batik.ext.awt.image.rendered.CachableRed;
/*     */ import org.apache.batik.ext.awt.image.rendered.PadRed;
/*     */ import org.apache.batik.util.HaltingThread;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class DynamicRenderer
/*     */   extends StaticRenderer
/*     */ {
/*     */   static final int COPY_OVERHEAD = 1000;
/*     */   static final int COPY_LINE_OVERHEAD = 10;
/*     */   RectListManager damagedAreas;
/*     */   
/*     */   public DynamicRenderer() {}
/*     */   
/*  60 */   public DynamicRenderer(RenderingHints rh, AffineTransform at) { super(rh, at); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*  67 */   protected CachableRed setupCache(CachableRed img) { return img; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void flush(Rectangle r) {}
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void flush(Collection areas) {}
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected void updateWorkingBuffers() {
/*  84 */     if (this.rootFilter == null) {
/*  85 */       this.rootFilter = this.rootGN.getGraphicsNodeRable(true);
/*  86 */       this.rootCR = null;
/*     */     } 
/*     */     
/*  89 */     this.rootCR = renderGNR();
/*  90 */     if (this.rootCR == null) {
/*     */       
/*  92 */       this.workingRaster = null;
/*  93 */       this.workingOffScreen = null;
/*  94 */       this.workingBaseRaster = null;
/*     */       
/*  96 */       this.currentOffScreen = null;
/*  97 */       this.currentBaseRaster = null;
/*  98 */       this.currentRaster = null;
/*     */       
/*     */       return;
/*     */     } 
/* 102 */     SampleModel sm = this.rootCR.getSampleModel();
/* 103 */     int w = this.offScreenWidth;
/* 104 */     int h = this.offScreenHeight;
/*     */     
/* 106 */     if (this.workingBaseRaster == null || this.workingBaseRaster.getWidth() < w || this.workingBaseRaster.getHeight() < h) {
/*     */ 
/*     */ 
/*     */       
/* 110 */       sm = sm.createCompatibleSampleModel(w, h);
/*     */       
/* 112 */       this.workingBaseRaster = Raster.createWritableRaster(sm, new Point(0, 0));
/*     */ 
/*     */       
/* 115 */       this.workingRaster = this.workingBaseRaster.createWritableChild(0, 0, w, h, 0, 0, null);
/*     */ 
/*     */       
/* 118 */       this.workingOffScreen = new BufferedImage(this.rootCR.getColorModel(), this.workingRaster, this.rootCR.getColorModel().isAlphaPremultiplied(), null);
/*     */     } 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 124 */     if (!this.isDoubleBuffered) {
/* 125 */       this.currentOffScreen = this.workingOffScreen;
/* 126 */       this.currentBaseRaster = this.workingBaseRaster;
/* 127 */       this.currentRaster = this.workingRaster;
/*     */     } 
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void repaint(RectListManager devRLM) {
/* 146 */     if (devRLM == null) {
/*     */       return;
/*     */     }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 159 */     updateWorkingBuffers();
/* 160 */     if (this.rootCR == null || this.workingBaseRaster == null) {
/*     */       return;
/*     */     }
/*     */ 
/*     */ 
/*     */     
/* 166 */     PadRed padRed = this.rootCR;
/* 167 */     WritableRaster syncRaster = this.workingBaseRaster;
/* 168 */     WritableRaster copyRaster = this.workingRaster;
/*     */     
/* 170 */     Rectangle srcR = this.rootCR.getBounds();
/*     */     
/* 172 */     Rectangle dstR = this.workingRaster.getBounds();
/* 173 */     if (dstR.x < srcR.x || dstR.y < srcR.y || dstR.x + dstR.width > srcR.x + srcR.width || dstR.y + dstR.height > srcR.y + srcR.height)
/*     */     {
/*     */ 
/*     */       
/* 177 */       padRed = new PadRed(padRed, dstR, PadMode.ZERO_PAD, null);
/*     */     }
/* 179 */     boolean repaintAll = false;
/*     */     
/* 181 */     Rectangle dr = copyRaster.getBounds();
/*     */ 
/*     */     
/* 184 */     synchronized (syncRaster) {
/*     */       
/* 186 */       if (repaintAll) {
/*     */         
/* 188 */         padRed.copyData(copyRaster);
/*     */       } else {
/* 190 */         Graphics2D g2d = null;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */         
/* 202 */         if (this.isDoubleBuffered && this.currentRaster != null && this.damagedAreas != null) {
/*     */ 
/*     */           
/* 205 */           this.damagedAreas.subtract(devRLM, 1000, 10);
/*     */           
/* 207 */           this.damagedAreas.mergeRects(1000, 10);
/*     */ 
/*     */ 
/*     */           
/* 211 */           Color fillColor = new Color(0, 0, 255, 50);
/* 212 */           Color borderColor = new Color(0, 0, 0, 50);
/*     */           
/* 214 */           Iterator iter = this.damagedAreas.iterator();
/* 215 */           while (iter.hasNext()) {
/* 216 */             Rectangle r = (Rectangle)iter.next();
/* 217 */             if (!dr.intersects(r))
/* 218 */               continue;  r = dr.intersection(r);
/*     */             
/* 220 */             Raster src = this.currentRaster.createWritableChild(r.x, r.y, r.width, r.height, r.x, r.y, null);
/*     */             
/* 222 */             GraphicsUtil.copyData(src, copyRaster);
/* 223 */             if (g2d != null) {
/* 224 */               g2d.setPaint(fillColor);
/* 225 */               g2d.fill(r);
/* 226 */               g2d.setPaint(borderColor);
/* 227 */               g2d.draw(r);
/*     */             } 
/*     */           } 
/*     */         } 
/*     */         
/* 232 */         Color fillColor = new Color(255, 0, 0, 50);
/* 233 */         Color borderColor = new Color(0, 0, 0, 50);
/*     */         
/* 235 */         Iterator iter = devRLM.iterator();
/* 236 */         while (iter.hasNext()) {
/* 237 */           Rectangle r = (Rectangle)iter.next();
/* 238 */           if (!dr.intersects(r))
/* 239 */             continue;  r = dr.intersection(r);
/*     */ 
/*     */           
/* 242 */           WritableRaster dst = copyRaster.createWritableChild(r.x, r.y, r.width, r.height, r.x, r.y, null);
/*     */           
/* 244 */           padRed.copyData(dst);
/* 245 */           if (g2d != null) {
/* 246 */             g2d.setPaint(fillColor);
/* 247 */             g2d.fill(r);
/* 248 */             g2d.setPaint(borderColor);
/* 249 */             g2d.draw(r);
/*     */           } 
/*     */         } 
/*     */       } 
/*     */     } 
/*     */     
/* 255 */     if (HaltingThread.hasBeenHalted()) {
/*     */       return;
/*     */     }
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 262 */     BufferedImage tmpBI = this.workingOffScreen;
/*     */     
/* 264 */     this.workingBaseRaster = this.currentBaseRaster;
/* 265 */     this.workingRaster = this.currentRaster;
/* 266 */     this.workingOffScreen = this.currentOffScreen;
/*     */     
/* 268 */     this.currentRaster = copyRaster;
/* 269 */     this.currentBaseRaster = syncRaster;
/* 270 */     this.currentOffScreen = tmpBI;
/*     */     
/* 272 */     this.damagedAreas = devRLM;
/*     */   }
/*     */ }


/* Location:              C:\Program Files (x86)\IBM\WebSphere\AppServer\profiles\ctgAppSrv01\installedApps\ctgCell01\MAXIMO.ear\maximouiweb.war\WEB-INF\classes\!\com\ibm\tivoli\maximo\report\control\svgtools\batik-gvt.jar!\org\apache\batik\gvt\renderer\DynamicRenderer.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       1.0.7
 */