/*     */ package org.apache.batik.gvt.renderer;
/*     */ 
/*     */ import java.awt.AlphaComposite;
/*     */ import java.awt.Graphics2D;
/*     */ import java.awt.Point;
/*     */ import java.awt.Rectangle;
/*     */ import java.awt.RenderingHints;
/*     */ import java.awt.Shape;
/*     */ import java.awt.geom.AffineTransform;
/*     */ import java.awt.image.BufferedImage;
/*     */ import java.awt.image.ColorModel;
/*     */ import java.awt.image.Raster;
/*     */ import java.awt.image.RenderedImage;
/*     */ import java.awt.image.SampleModel;
/*     */ import java.awt.image.WritableRaster;
/*     */ import java.awt.image.renderable.RenderContext;
/*     */ import java.lang.ref.SoftReference;
/*     */ import java.util.Collection;
/*     */ import java.util.Iterator;
/*     */ import org.apache.batik.ext.awt.geom.RectListManager;
/*     */ import org.apache.batik.ext.awt.image.GraphicsUtil;
/*     */ import org.apache.batik.ext.awt.image.PadMode;
/*     */ import org.apache.batik.ext.awt.image.renderable.Filter;
/*     */ import org.apache.batik.ext.awt.image.rendered.CachableRed;
/*     */ import org.apache.batik.ext.awt.image.rendered.PadRed;
/*     */ import org.apache.batik.ext.awt.image.rendered.TileCacheRed;
/*     */ import org.apache.batik.ext.awt.image.rendered.TranslateRed;
/*     */ import org.apache.batik.gvt.GraphicsNode;
/*     */ import org.apache.batik.util.HaltingThread;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class StaticRenderer
/*     */   implements ImageRenderer
/*     */ {
/*     */   protected GraphicsNode rootGN;
/*     */   protected Filter rootFilter;
/*     */   protected CachableRed rootCR;
/*     */   protected SoftReference lastCR;
/*     */   protected SoftReference lastCache;
/*     */   protected boolean isDoubleBuffered;
/*     */   protected WritableRaster currentBaseRaster;
/*     */   protected WritableRaster currentRaster;
/*     */   protected BufferedImage currentOffScreen;
/*     */   protected WritableRaster workingBaseRaster;
/*     */   protected WritableRaster workingRaster;
/*     */   protected BufferedImage workingOffScreen;
/*     */   protected int offScreenWidth;
/*     */   protected int offScreenHeight;
/*     */   protected RenderingHints renderingHints;
/*     */   protected AffineTransform usr2dev;
/*  95 */   protected static RenderingHints defaultRenderingHints = new RenderingHints(null); static  {
/*  96 */     defaultRenderingHints.put(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
/*     */ 
/*     */     
/*  99 */     defaultRenderingHints.put(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BILINEAR);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public StaticRenderer(RenderingHints rh, AffineTransform at) {
/*     */     this.isDoubleBuffered = false;
/* 109 */     this.renderingHints = new RenderingHints(null);
/* 110 */     this.renderingHints.add(rh);
/* 111 */     this.usr2dev = new AffineTransform(at);
/*     */   }
/*     */ 
/*     */ 
/*     */   
/*     */   public StaticRenderer() {
/*     */     this.isDoubleBuffered = false;
/* 118 */     this.renderingHints = new RenderingHints(null);
/* 119 */     this.renderingHints.add(defaultRenderingHints);
/* 120 */     this.usr2dev = new AffineTransform();
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void dispose() {
/* 128 */     this.rootGN = null;
/* 129 */     this.rootFilter = null;
/* 130 */     this.rootCR = null;
/*     */     
/* 132 */     this.workingOffScreen = null;
/* 133 */     this.workingBaseRaster = null;
/* 134 */     this.workingRaster = null;
/*     */     
/* 136 */     this.currentOffScreen = null;
/* 137 */     this.currentBaseRaster = null;
/* 138 */     this.currentRaster = null;
/*     */     
/* 140 */     this.renderingHints = null;
/* 141 */     this.lastCache = null;
/* 142 */     this.lastCR = null;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void setTree(GraphicsNode rootGN) {
/* 151 */     this.rootGN = rootGN;
/* 152 */     this.rootFilter = null;
/* 153 */     this.rootCR = null;
/*     */     
/* 155 */     this.workingOffScreen = null;
/* 156 */     this.workingRaster = null;
/*     */     
/* 158 */     this.currentOffScreen = null;
/* 159 */     this.currentRaster = null;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 168 */   public GraphicsNode getTree() { return this.rootGN; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void setRenderingHints(RenderingHints rh) {
/* 175 */     this.renderingHints = new RenderingHints(null);
/* 176 */     this.renderingHints.add(rh);
/*     */     
/* 178 */     this.rootFilter = null;
/* 179 */     this.rootCR = null;
/*     */     
/* 181 */     this.workingOffScreen = null;
/* 182 */     this.workingRaster = null;
/*     */     
/* 184 */     this.currentOffScreen = null;
/* 185 */     this.currentRaster = null;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 193 */   public RenderingHints getRenderingHints() { return this.renderingHints; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void setTransform(AffineTransform usr2dev) {
/* 204 */     if (this.usr2dev.equals(usr2dev)) {
/*     */       return;
/*     */     }
/* 207 */     if (usr2dev == null) {
/* 208 */       this.usr2dev = new AffineTransform();
/*     */     } else {
/* 210 */       this.usr2dev = new AffineTransform(usr2dev);
/*     */     } 
/* 212 */     this.rootCR = null;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 220 */   public AffineTransform getTransform() { return this.usr2dev; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 229 */   public boolean isDoubleBuffered() { return this.isDoubleBuffered; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void setDoubleBuffered(boolean isDoubleBuffered) {
/* 240 */     if (this.isDoubleBuffered == isDoubleBuffered) {
/*     */       return;
/*     */     }
/* 243 */     this.isDoubleBuffered = isDoubleBuffered;
/* 244 */     if (isDoubleBuffered) {
/*     */       
/* 246 */       this.currentOffScreen = null;
/* 247 */       this.currentBaseRaster = null;
/* 248 */       this.currentRaster = null;
/*     */     } else {
/*     */       
/* 251 */       this.currentOffScreen = this.workingOffScreen;
/* 252 */       this.currentBaseRaster = this.workingBaseRaster;
/* 253 */       this.currentRaster = this.workingRaster;
/*     */     } 
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void updateOffScreen(int width, int height) {
/* 267 */     this.offScreenWidth = width;
/* 268 */     this.offScreenHeight = height;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public BufferedImage getOffScreen() {
/* 283 */     if (this.rootGN == null) {
/* 284 */       return null;
/*     */     }
/* 286 */     return this.currentOffScreen;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void clearOffScreen() {
/* 304 */     if (this.isDoubleBuffered) {
/*     */       return;
/*     */     }
/* 307 */     updateWorkingBuffers();
/* 308 */     if (this.rootCR == null || this.workingBaseRaster == null) {
/*     */       return;
/*     */     }
/*     */     
/* 312 */     ColorModel cm = this.rootCR.getColorModel();
/* 313 */     WritableRaster syncRaster = this.workingBaseRaster;
/*     */ 
/*     */     
/* 316 */     synchronized (syncRaster) {
/* 317 */       BufferedImage bi = new BufferedImage(cm, this.workingBaseRaster, cm.isAlphaPremultiplied(), null);
/*     */       
/* 319 */       Graphics2D g2d = bi.createGraphics();
/* 320 */       g2d.setComposite(AlphaComposite.Clear);
/* 321 */       g2d.fillRect(0, 0, bi.getWidth(), bi.getHeight());
/* 322 */       g2d.dispose();
/*     */     } 
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void repaint(Shape area) {
/* 341 */     if (area == null)
/* 342 */       return;  RectListManager rlm = new RectListManager();
/* 343 */     rlm.add(this.usr2dev.createTransformedShape(area).getBounds());
/* 344 */     repaint(rlm);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void repaint(RectListManager areas) {
/* 362 */     if (areas == null) {
/*     */       return;
/*     */     }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 375 */     updateWorkingBuffers();
/* 376 */     if (this.rootCR == null || this.workingBaseRaster == null) {
/*     */       return;
/*     */     }
/*     */     
/* 380 */     PadRed padRed = this.rootCR;
/* 381 */     WritableRaster syncRaster = this.workingBaseRaster;
/* 382 */     WritableRaster copyRaster = this.workingRaster;
/*     */     
/* 384 */     Rectangle srcR = this.rootCR.getBounds();
/* 385 */     Rectangle dstR = this.workingRaster.getBounds();
/* 386 */     if (dstR.x < srcR.x || dstR.y < srcR.y || dstR.x + dstR.width > srcR.x + srcR.width || dstR.y + dstR.height > srcR.y + srcR.height)
/*     */     {
/*     */ 
/*     */       
/* 390 */       padRed = new PadRed(padRed, dstR, PadMode.ZERO_PAD, null);
/*     */     }
/*     */     
/* 393 */     synchronized (syncRaster) {
/* 394 */       padRed.copyData(copyRaster);
/*     */     } 
/*     */     
/* 397 */     if (!HaltingThread.hasBeenHalted()) {
/*     */       
/* 399 */       BufferedImage tmpBI = this.workingOffScreen;
/*     */       
/* 401 */       this.workingBaseRaster = this.currentBaseRaster;
/* 402 */       this.workingRaster = this.currentRaster;
/* 403 */       this.workingOffScreen = this.currentOffScreen;
/*     */       
/* 405 */       this.currentRaster = copyRaster;
/* 406 */       this.currentBaseRaster = syncRaster;
/* 407 */       this.currentOffScreen = tmpBI;
/*     */     } 
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void flush() {
/* 417 */     if (this.lastCache == null)
/* 418 */       return;  Object o = this.lastCache.get();
/* 419 */     if (o == null)
/*     */       return; 
/* 421 */     TileCacheRed tcr = (TileCacheRed)o;
/* 422 */     tcr.flushCache(tcr.getBounds());
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void flush(Collection areas) {
/* 429 */     AffineTransform at = getTransform();
/* 430 */     Iterator i = areas.iterator();
/* 431 */     while (i.hasNext()) {
/* 432 */       Shape s = (Shape)i.next();
/* 433 */       Rectangle r = at.createTransformedShape(s).getBounds();
/* 434 */       flush(r);
/*     */     } 
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void flush(Rectangle r) {
/* 442 */     if (this.lastCache == null)
/* 443 */       return;  Object o = this.lastCache.get();
/* 444 */     if (o == null)
/*     */       return; 
/* 446 */     TileCacheRed tcr = (TileCacheRed)o;
/* 447 */     r = (Rectangle)r.clone();
/* 448 */     r.x -= Math.round((float)this.usr2dev.getTranslateX());
/* 449 */     r.y -= Math.round((float)this.usr2dev.getTranslateY());
/*     */     
/* 451 */     tcr.flushCache(r);
/*     */   }
/*     */   
/*     */   protected CachableRed setupCache(CachableRed img) {
/* 455 */     if (this.lastCR == null || img != this.lastCR.get()) {
/*     */       
/* 457 */       this.lastCR = new SoftReference(img);
/* 458 */       this.lastCache = null;
/*     */     } 
/*     */     
/* 461 */     Object o = null;
/* 462 */     if (this.lastCache != null)
/* 463 */       o = this.lastCache.get(); 
/* 464 */     if (o != null) {
/* 465 */       return (CachableRed)o;
/*     */     }
/* 467 */     TileCacheRed tileCacheRed = new TileCacheRed(img);
/* 468 */     this.lastCache = new SoftReference(tileCacheRed);
/* 469 */     return tileCacheRed;
/*     */   }
/*     */ 
/*     */   
/*     */   protected CachableRed renderGNR() {
/* 474 */     AffineTransform at = this.usr2dev;
/* 475 */     AffineTransform rcAT = new AffineTransform(at.getScaleX(), at.getShearY(), at.getShearX(), at.getScaleY(), 0.0D, 0.0D);
/*     */ 
/*     */ 
/*     */     
/* 479 */     RenderContext rc = new RenderContext(rcAT, null, this.renderingHints);
/*     */     
/* 481 */     RenderedImage ri = this.rootFilter.createRendering(rc);
/* 482 */     if (ri == null) {
/* 483 */       return null;
/*     */     }
/*     */     
/* 486 */     CachableRed ret = GraphicsUtil.wrap(ri);
/* 487 */     ret = setupCache(ret);
/*     */     
/* 489 */     int dx = Math.round((float)at.getTranslateX());
/* 490 */     int dy = Math.round((float)at.getTranslateY());
/* 491 */     TranslateRed translateRed = new TranslateRed(ret, ret.getMinX() + dx, ret.getMinY() + dy);
/* 492 */     return GraphicsUtil.convertTosRGB(translateRed);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected void updateWorkingBuffers() {
/*     */     int j, i;
/* 503 */     if (this.rootFilter == null) {
/* 504 */       this.rootFilter = this.rootGN.getGraphicsNodeRable(true);
/* 505 */       this.rootCR = null;
/*     */     } 
/*     */     
/* 508 */     this.rootCR = renderGNR();
/* 509 */     if (this.rootCR == null) {
/*     */       
/* 511 */       this.workingRaster = null;
/* 512 */       this.workingOffScreen = null;
/* 513 */       this.workingBaseRaster = null;
/*     */       
/* 515 */       this.currentOffScreen = null;
/* 516 */       this.currentBaseRaster = null;
/* 517 */       this.currentRaster = null;
/*     */       
/*     */       return;
/*     */     } 
/* 521 */     SampleModel sm = this.rootCR.getSampleModel();
/* 522 */     int w = this.offScreenWidth;
/* 523 */     int h = this.offScreenHeight;
/*     */     
/* 525 */     int tw = sm.getWidth();
/* 526 */     int th = sm.getHeight();
/* 527 */     w = ((w + tw - 1) / tw + 1) * tw;
/* 528 */     h = ((h + th - 1) / th + 1) * th;
/*     */     
/* 530 */     if (this.workingBaseRaster == null || this.workingBaseRaster.getWidth() < w || this.workingBaseRaster.getHeight() < h) {
/*     */ 
/*     */ 
/*     */       
/* 534 */       sm = sm.createCompatibleSampleModel(w, h);
/*     */       
/* 536 */       this.workingBaseRaster = Raster.createWritableRaster(sm, new Point(0, 0));
/*     */     } 
/*     */ 
/*     */     
/* 540 */     int tgx = -this.rootCR.getTileGridXOffset();
/* 541 */     int tgy = -this.rootCR.getTileGridYOffset();
/*     */     
/* 543 */     if (tgx >= 0) { i = tgx / tw; }
/* 544 */     else { i = (tgx - tw + 1) / tw; }
/* 545 */      if (tgy >= 0) { j = tgy / th; }
/* 546 */     else { j = (tgy - th + 1) / th; }
/*     */     
/* 548 */     int xloc = i * tw - tgx;
/* 549 */     int yloc = j * th - tgy;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 557 */     this.workingRaster = this.workingBaseRaster.createWritableChild(0, 0, w, h, xloc, yloc, null);
/*     */ 
/*     */     
/* 560 */     this.workingOffScreen = new BufferedImage(this.rootCR.getColorModel(), this.workingRaster.createWritableChild(0, 0, this.offScreenWidth, this.offScreenHeight, 0, 0, null), this.rootCR.getColorModel().isAlphaPremultiplied(), null);
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 567 */     if (!this.isDoubleBuffered) {
/* 568 */       this.currentOffScreen = this.workingOffScreen;
/* 569 */       this.currentBaseRaster = this.workingBaseRaster;
/* 570 */       this.currentRaster = this.workingRaster;
/*     */     } 
/*     */   }
/*     */ }


/* Location:              C:\Program Files (x86)\IBM\WebSphere\AppServer\profiles\ctgAppSrv01\installedApps\ctgCell01\MAXIMO.ear\maximouiweb.war\WEB-INF\classes\!\com\ibm\tivoli\maximo\report\control\svgtools\batik-gvt.jar!\org\apache\batik\gvt\renderer\StaticRenderer.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       1.0.7
 */