/*      */ package org.apache.batik.gvt.renderer;
/*      */ 
/*      */ import java.awt.Graphics2D;
/*      */ import java.awt.Paint;
/*      */ import java.awt.RenderingHints;
/*      */ import java.awt.Shape;
/*      */ import java.awt.Stroke;
/*      */ import java.awt.font.FontRenderContext;
/*      */ import java.awt.font.TextAttribute;
/*      */ import java.awt.geom.GeneralPath;
/*      */ import java.awt.geom.Point2D;
/*      */ import java.awt.geom.Rectangle2D;
/*      */ import java.text.AttributedCharacterIterator;
/*      */ import java.text.AttributedString;
/*      */ import java.util.ArrayList;
/*      */ import java.util.HashSet;
/*      */ import java.util.Iterator;
/*      */ import java.util.List;
/*      */ import java.util.Set;
/*      */ import org.apache.batik.gvt.TextNode;
/*      */ import org.apache.batik.gvt.TextPainter;
/*      */ import org.apache.batik.gvt.font.FontFamilyResolver;
/*      */ import org.apache.batik.gvt.font.GVTFont;
/*      */ import org.apache.batik.gvt.font.GVTFontFamily;
/*      */ import org.apache.batik.gvt.font.GVTGlyphMetrics;
/*      */ import org.apache.batik.gvt.font.GVTLineMetrics;
/*      */ import org.apache.batik.gvt.text.AttributedCharacterSpanIterator;
/*      */ import org.apache.batik.gvt.text.BidiAttributedCharacterIterator;
/*      */ import org.apache.batik.gvt.text.GVTAttributedCharacterIterator;
/*      */ import org.apache.batik.gvt.text.Mark;
/*      */ import org.apache.batik.gvt.text.TextHit;
/*      */ import org.apache.batik.gvt.text.TextPaintInfo;
/*      */ import org.apache.batik.gvt.text.TextPath;
/*      */ import org.apache.batik.gvt.text.TextSpanLayout;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ public class StrokingTextPainter
/*      */   extends BasicTextPainter
/*      */ {
/*   73 */   public static final AttributedCharacterIterator.Attribute PAINT_INFO = GVTAttributedCharacterIterator.TextAttribute.PAINT_INFO;
/*      */ 
/*      */ 
/*      */   
/*   77 */   public static final AttributedCharacterIterator.Attribute FLOW_REGIONS = GVTAttributedCharacterIterator.TextAttribute.FLOW_REGIONS;
/*      */ 
/*      */ 
/*      */   
/*   81 */   public static final AttributedCharacterIterator.Attribute FLOW_PARAGRAPH = GVTAttributedCharacterIterator.TextAttribute.FLOW_PARAGRAPH;
/*      */ 
/*      */ 
/*      */   
/*   85 */   public static final AttributedCharacterIterator.Attribute TEXT_COMPOUND_ID = GVTAttributedCharacterIterator.TextAttribute.TEXT_COMPOUND_ID;
/*      */ 
/*      */ 
/*      */   
/*   89 */   public static final AttributedCharacterIterator.Attribute GVT_FONT = GVTAttributedCharacterIterator.TextAttribute.GVT_FONT;
/*      */ 
/*      */ 
/*      */   
/*   93 */   public static final AttributedCharacterIterator.Attribute GVT_FONTS = GVTAttributedCharacterIterator.TextAttribute.GVT_FONTS;
/*      */ 
/*      */ 
/*      */   
/*   97 */   public static final AttributedCharacterIterator.Attribute BIDI_LEVEL = GVTAttributedCharacterIterator.TextAttribute.BIDI_LEVEL;
/*      */ 
/*      */ 
/*      */   
/*  101 */   public static final AttributedCharacterIterator.Attribute XPOS = GVTAttributedCharacterIterator.TextAttribute.X;
/*      */ 
/*      */ 
/*      */   
/*  105 */   public static final AttributedCharacterIterator.Attribute YPOS = GVTAttributedCharacterIterator.TextAttribute.Y;
/*      */ 
/*      */ 
/*      */   
/*  109 */   public static final AttributedCharacterIterator.Attribute TEXTPATH = GVTAttributedCharacterIterator.TextAttribute.TEXTPATH;
/*      */ 
/*      */ 
/*      */   
/*  113 */   public static final AttributedCharacterIterator.Attribute WRITING_MODE = GVTAttributedCharacterIterator.TextAttribute.WRITING_MODE;
/*      */ 
/*      */   
/*  116 */   public static final Integer WRITING_MODE_TTB = GVTAttributedCharacterIterator.TextAttribute.WRITING_MODE_TTB;
/*      */ 
/*      */   
/*  119 */   public static final Integer WRITING_MODE_RTL = GVTAttributedCharacterIterator.TextAttribute.WRITING_MODE_RTL;
/*      */ 
/*      */ 
/*      */   
/*  123 */   public static final AttributedCharacterIterator.Attribute ANCHOR_TYPE = GVTAttributedCharacterIterator.TextAttribute.ANCHOR_TYPE;
/*      */ 
/*      */   
/*  126 */   public static final Integer ADJUST_SPACING = GVTAttributedCharacterIterator.TextAttribute.ADJUST_SPACING;
/*      */   
/*  128 */   public static final Integer ADJUST_ALL = GVTAttributedCharacterIterator.TextAttribute.ADJUST_ALL;
/*      */   
/*  130 */   public static final GVTAttributedCharacterIterator.TextAttribute ALT_GLYPH_HANDLER = GVTAttributedCharacterIterator.TextAttribute.ALT_GLYPH_HANDLER;
/*      */ 
/*      */   
/*  133 */   static Set extendedAtts = new HashSet();
/*      */   
/*      */   static  {
/*  136 */     extendedAtts.add(FLOW_PARAGRAPH);
/*  137 */     extendedAtts.add(TEXT_COMPOUND_ID);
/*  138 */     extendedAtts.add(GVT_FONT);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  145 */     singleton = new StrokingTextPainter();
/*      */   }
/*      */ 
/*      */   
/*      */   protected static TextPainter singleton;
/*      */   
/*  151 */   public static TextPainter getInstance() { return singleton; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public void paint(TextNode node, Graphics2D g2d) {
/*  162 */     AttributedCharacterIterator aci = node.getAttributedCharacterIterator();
/*  163 */     if (aci == null) {
/*      */       return;
/*      */     }
/*  166 */     List textRuns = getTextRuns(node, aci);
/*      */ 
/*      */ 
/*      */     
/*  170 */     paintDecorations(textRuns, g2d, 1);
/*  171 */     paintDecorations(textRuns, g2d, 4);
/*  172 */     paintTextRuns(textRuns, g2d);
/*  173 */     paintDecorations(textRuns, g2d, 2);
/*      */   }
/*      */ 
/*      */   
/*      */   protected void printAttrs(AttributedCharacterIterator aci) {
/*  178 */     aci.first();
/*  179 */     int start = aci.getBeginIndex();
/*  180 */     System.out.print("AttrRuns: ");
/*  181 */     while (aci.current() != Character.MAX_VALUE) {
/*  182 */       int end = aci.getRunLimit();
/*  183 */       System.out.print("" + (end - start) + ", ");
/*  184 */       aci.setIndex(end);
/*  185 */       start = end;
/*      */     } 
/*  187 */     System.out.println("");
/*      */   }
/*      */ 
/*      */   
/*      */   public List getTextRuns(TextNode node, AttributedCharacterIterator aci) {
/*  192 */     List textRuns = node.getTextRuns();
/*  193 */     if (textRuns != null) {
/*  194 */       return textRuns;
/*      */     }
/*      */     
/*  197 */     AttributedCharacterIterator[] chunkACIs = getTextChunkACIs(aci);
/*  198 */     textRuns = computeTextRuns(node, aci, chunkACIs);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  204 */     node.setTextRuns(textRuns);
/*  205 */     return node.getTextRuns();
/*      */   }
/*      */ 
/*      */   
/*      */   public List computeTextRuns(TextNode node, AttributedCharacterIterator aci, AttributedCharacterIterator[] chunkACIs) {
/*      */     TextChunk chunk;
/*  211 */     int[][] chunkCharMaps = new int[chunkACIs.length][];
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  216 */     int chunkStart = aci.getBeginIndex();
/*  217 */     for (int i = 0; i < chunkACIs.length; i++) {
/*      */       
/*  219 */       BidiAttributedCharacterIterator iter = new BidiAttributedCharacterIterator(chunkACIs[i], this.fontRenderContext, chunkStart);
/*      */       
/*  221 */       chunkACIs[i] = iter;
/*  222 */       chunkCharMaps[i] = iter.getCharMap();
/*      */ 
/*      */ 
/*      */       
/*  226 */       chunkACIs[i] = createModifiedACIForFontMatching(chunkACIs[i]);
/*      */ 
/*      */       
/*  229 */       chunkStart += chunkACIs[i].getEndIndex() - chunkACIs[i].getBeginIndex();
/*      */     } 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  237 */     List textRuns = new ArrayList();
/*  238 */     TextChunk prevChunk = null;
/*  239 */     int currentChunk = 0;
/*      */     
/*  241 */     Point2D location = node.getLocation();
/*      */ 
/*      */     
/*      */     do {
/*  245 */       chunkACIs[currentChunk].first();
/*      */       
/*  247 */       chunk = getTextChunk(node, chunkACIs[currentChunk], chunkCharMaps[currentChunk], textRuns, prevChunk);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */       
/*  254 */       chunkACIs[currentChunk].first();
/*  255 */       if (chunk != null) {
/*  256 */         location = adjustChunkOffsets(location, textRuns, chunk);
/*      */       }
/*  258 */       prevChunk = chunk;
/*  259 */       currentChunk++;
/*      */     }
/*  261 */     while (chunk != null && currentChunk < chunkACIs.length);
/*      */     
/*  263 */     return textRuns;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected AttributedCharacterIterator[] getTextChunkACIs(AttributedCharacterIterator aci) {
/*  273 */     List aciList = new ArrayList();
/*  274 */     int chunkStartIndex = aci.getBeginIndex();
/*  275 */     aci.first();
/*  276 */     Object writingMode = aci.getAttribute(WRITING_MODE);
/*  277 */     boolean vertical = (writingMode == WRITING_MODE_TTB);
/*      */     
/*  279 */     while (aci.setIndex(chunkStartIndex) != Character.MAX_VALUE) {
/*  280 */       TextPath prevTextPath = null;
/*  281 */       int start = chunkStartIndex, end = 0;
/*  282 */       for (; aci.setIndex(start) != Character.MAX_VALUE; start = end) {
/*      */         
/*  284 */         TextPath textPath = (TextPath)aci.getAttribute(TEXTPATH);
/*      */         
/*  286 */         if (start != chunkStartIndex) {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */           
/*  292 */           if (vertical) {
/*  293 */             Float runY = (Float)aci.getAttribute(YPOS);
/*      */             
/*  295 */             if (runY != null && !runY.isNaN())
/*      */               break; 
/*      */           } else {
/*  298 */             Float runX = (Float)aci.getAttribute(XPOS);
/*      */             
/*  300 */             if (runX != null && !runX.isNaN()) {
/*      */               break;
/*      */             }
/*      */           } 
/*      */           
/*  305 */           if (prevTextPath == null && textPath != null) {
/*      */             break;
/*      */           }
/*      */ 
/*      */ 
/*      */           
/*  311 */           if (prevTextPath != null && textPath == null) {
/*      */             break;
/*      */           }
/*      */         } 
/*  315 */         prevTextPath = textPath;
/*      */ 
/*      */ 
/*      */         
/*  319 */         if (aci.getAttribute(FLOW_PARAGRAPH) != null) {
/*  320 */           end = aci.getRunLimit(FLOW_PARAGRAPH);
/*      */           
/*  322 */           aci.setIndex(end);
/*      */           
/*      */           break;
/*      */         } 
/*      */         
/*  327 */         end = aci.getRunLimit(TEXT_COMPOUND_ID);
/*      */         
/*  329 */         if (start != chunkStartIndex) {
/*      */           continue;
/*      */         }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */         
/*  338 */         TextNode.Anchor anchor = (TextNode.Anchor)aci.getAttribute(ANCHOR_TYPE);
/*  339 */         if (anchor == TextNode.Anchor.START) {
/*      */           continue;
/*      */         }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */         
/*  347 */         if (vertical) {
/*  348 */           Float runY = (Float)aci.getAttribute(YPOS);
/*      */           
/*  350 */           if (runY == null || runY.isNaN()) {
/*      */             continue;
/*      */           }
/*      */         } else {
/*  354 */           Float runX = (Float)aci.getAttribute(XPOS);
/*      */           
/*  356 */           if (runX == null || runX.isNaN()) {
/*      */             continue;
/*      */           }
/*      */         } 
/*      */ 
/*      */ 
/*      */         
/*  363 */         for (int i = start + 1; i < end; i++) {
/*  364 */           aci.setIndex(i);
/*  365 */           if (vertical) {
/*  366 */             Float runY = (Float)aci.getAttribute(YPOS);
/*  367 */             if (runY == null || runY.isNaN())
/*      */               break; 
/*      */           } else {
/*  370 */             Float runX = (Float)aci.getAttribute(XPOS);
/*  371 */             if (runX == null || runX.isNaN())
/*      */               break; 
/*      */           } 
/*  374 */           aciList.add(new AttributedCharacterSpanIterator(aci, i - 1, i));
/*      */           
/*  376 */           chunkStartIndex = i;
/*      */         } 
/*      */         
/*      */         continue;
/*      */       } 
/*  381 */       int chunkEndIndex = aci.getIndex();
/*      */ 
/*      */       
/*  384 */       aciList.add(new AttributedCharacterSpanIterator(aci, chunkStartIndex, chunkEndIndex));
/*      */ 
/*      */       
/*  387 */       chunkStartIndex = chunkEndIndex;
/*      */     } 
/*      */ 
/*      */     
/*  391 */     AttributedCharacterIterator[] aciArray = new AttributedCharacterIterator[aciList.size()];
/*      */     
/*  393 */     Iterator iter = aciList.iterator();
/*  394 */     for (int i = 0; iter.hasNext(); i++) {
/*  395 */       aciArray[i] = (AttributedCharacterIterator)iter.next();
/*      */     }
/*  397 */     return aciArray;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected static AttributedCharacterIterator createModifiedACIForFontMatching(AttributedCharacterIterator aci) {
/*  413 */     aci.first();
/*  414 */     AttributedString as = null;
/*  415 */     int asOff = 0;
/*  416 */     int begin = aci.getBeginIndex();
/*  417 */     boolean moreChunks = true;
/*  418 */     int end = aci.getRunStart(TEXT_COMPOUND_ID);
/*  419 */     while (moreChunks) {
/*  420 */       int start = end;
/*  421 */       end = aci.getRunLimit(TEXT_COMPOUND_ID);
/*  422 */       int aciLength = end - start;
/*      */ 
/*      */       
/*  425 */       List fonts = (List)aci.getAttribute(GVT_FONTS);
/*      */ 
/*      */       
/*  428 */       float fontSize = 12.0F;
/*  429 */       Float fsFloat = (Float)aci.getAttribute(TextAttribute.SIZE);
/*  430 */       if (fsFloat != null) {
/*  431 */         fontSize = fsFloat.floatValue();
/*      */       }
/*      */ 
/*      */       
/*  435 */       if (fonts.size() == 0)
/*      */       {
/*  437 */         fonts.add(FontFamilyResolver.defaultFont.deriveFont(fontSize, aci));
/*      */       }
/*      */ 
/*      */ 
/*      */ 
/*      */       
/*  443 */       boolean[] fontAssigned = new boolean[aciLength];
/*      */       
/*  445 */       if (as == null) {
/*  446 */         as = new AttributedString(aci);
/*      */       }
/*  448 */       GVTFont defaultFont = null;
/*  449 */       int numSet = 0;
/*  450 */       int firstUnset = start;
/*      */       
/*  452 */       for (int i = 0; i < fonts.size(); i++) {
/*      */ 
/*      */         
/*  455 */         int currentIndex = firstUnset;
/*  456 */         boolean firstUnsetSet = false;
/*  457 */         aci.setIndex(currentIndex);
/*      */         
/*  459 */         GVTFont font = (GVTFont)fonts.get(i);
/*  460 */         if (defaultFont == null) {
/*  461 */           defaultFont = font;
/*      */         }
/*  463 */         while (currentIndex < end) {
/*  464 */           int displayUpToIndex = font.canDisplayUpTo(aci, currentIndex, end);
/*      */ 
/*      */ 
/*      */           
/*  468 */           Object altGlyphElement = aci.getAttribute(ALT_GLYPH_HANDLER);
/*  469 */           if (altGlyphElement != null)
/*      */           {
/*      */             
/*  472 */             displayUpToIndex = -1;
/*      */           }
/*      */           
/*  475 */           if (displayUpToIndex == -1)
/*      */           {
/*  477 */             displayUpToIndex = end;
/*      */           }
/*      */           
/*  480 */           if (displayUpToIndex <= currentIndex) {
/*  481 */             if (!firstUnsetSet) {
/*  482 */               firstUnset = currentIndex;
/*  483 */               firstUnsetSet = true;
/*      */             } 
/*      */             
/*  486 */             currentIndex++;
/*      */             
/*      */             continue;
/*      */           } 
/*      */           
/*  491 */           int runStart = -1;
/*  492 */           for (int j = currentIndex; j < displayUpToIndex; j++) {
/*  493 */             if (fontAssigned[j - start]) {
/*  494 */               if (runStart != -1)
/*      */               {
/*  496 */                 as.addAttribute(GVT_FONT, font, runStart - begin, j - begin);
/*      */                 
/*  498 */                 runStart = -1;
/*      */               }
/*      */             
/*  501 */             } else if (runStart == -1) {
/*  502 */               runStart = j;
/*      */             } 
/*  504 */             fontAssigned[j - start] = true;
/*  505 */             numSet++;
/*      */           } 
/*  507 */           if (runStart != -1)
/*      */           {
/*  509 */             as.addAttribute(GVT_FONT, font, runStart - begin, displayUpToIndex - begin);
/*      */           }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */           
/*  516 */           currentIndex = displayUpToIndex + 1;
/*      */         } 
/*      */ 
/*      */         
/*  520 */         if (numSet == aciLength) {
/*      */           break;
/*      */         }
/*      */       } 
/*      */       
/*  525 */       int runStart = -1;
/*  526 */       GVTFontFamily prevFF = null;
/*  527 */       GVTFont prevF = defaultFont;
/*  528 */       for (int i = 0; i < aciLength; i++) {
/*  529 */         if (fontAssigned[i]) {
/*  530 */           if (runStart != -1) {
/*      */             
/*  532 */             as.addAttribute(GVT_FONT, prevF, runStart + asOff, i + asOff);
/*      */             
/*  534 */             runStart = -1;
/*  535 */             prevF = null;
/*  536 */             prevFF = null;
/*      */           } 
/*      */         } else {
/*  539 */           char c = aci.setIndex(start + i);
/*      */           
/*  541 */           GVTFontFamily fontFamily = FontFamilyResolver.getFamilyThatCanDisplay(c);
/*      */ 
/*      */           
/*  544 */           if (runStart == -1) {
/*      */             
/*  546 */             runStart = i;
/*  547 */             prevFF = fontFamily;
/*  548 */             if (prevFF == null)
/*  549 */             { prevF = defaultFont; }
/*      */             else
/*  551 */             { prevF = fontFamily.deriveFont(fontSize, aci); } 
/*  552 */           } else if (prevFF != fontFamily) {
/*      */ 
/*      */             
/*  555 */             as.addAttribute(GVT_FONT, prevF, runStart + asOff, i + asOff);
/*      */ 
/*      */             
/*  558 */             runStart = i;
/*  559 */             prevFF = fontFamily;
/*  560 */             if (prevFF == null) {
/*  561 */               prevF = defaultFont;
/*      */             } else {
/*  563 */               prevF = fontFamily.deriveFont(fontSize, aci);
/*      */             } 
/*      */           } 
/*      */         } 
/*  567 */       }  if (runStart != -1)
/*      */       {
/*  569 */         as.addAttribute(GVT_FONT, prevF, runStart + asOff, aciLength + asOff);
/*      */       }
/*      */ 
/*      */       
/*  573 */       asOff += aciLength;
/*  574 */       if (aci.setIndex(end) == Character.MAX_VALUE) {
/*  575 */         moreChunks = false;
/*      */       }
/*  577 */       start = end;
/*      */     } 
/*  579 */     if (as != null) {
/*  580 */       return as.getIterator();
/*      */     }
/*      */     
/*  583 */     return aci;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected TextChunk getTextChunk(TextNode node, AttributedCharacterIterator aci, int[] charMap, List textRuns, TextChunk prevChunk) {
/*  592 */     int beginChunk = 0;
/*  593 */     if (prevChunk != null)
/*  594 */       beginChunk = prevChunk.end; 
/*  595 */     int endChunk = beginChunk;
/*  596 */     int begin = aci.getIndex();
/*      */     
/*  598 */     if (aci.current() == Character.MAX_VALUE) {
/*  599 */       return null;
/*      */     }
/*      */ 
/*      */     
/*  603 */     Point2D.Float offset = new Point2D.Float(0.0F, 0.0F);
/*  604 */     Point2D.Float advance = new Point2D.Float(0.0F, 0.0F);
/*  605 */     boolean isChunkStart = true;
/*  606 */     TextSpanLayout layout = null;
/*      */     while (true) {
/*  608 */       int start = aci.getRunStart(extendedAtts);
/*  609 */       int end = aci.getRunLimit(extendedAtts);
/*      */ 
/*      */       
/*  612 */       AttributedCharacterSpanIterator attributedCharacterSpanIterator = new AttributedCharacterSpanIterator(aci, start, end);
/*      */       
/*  614 */       int[] subCharMap = new int[end - start];
/*  615 */       System.arraycopy(charMap, start - begin, subCharMap, 0, subCharMap.length);
/*      */       
/*  617 */       FontRenderContext frc = this.fontRenderContext;
/*  618 */       RenderingHints rh = node.getRenderingHints();
/*      */ 
/*      */       
/*  621 */       if (rh != null && rh.get(RenderingHints.KEY_TEXT_ANTIALIASING) == RenderingHints.VALUE_TEXT_ANTIALIAS_OFF)
/*      */       {
/*      */ 
/*      */ 
/*      */         
/*  626 */         frc = this.aaOffFontRenderContext;
/*      */       }
/*      */       
/*  629 */       layout = getTextLayoutFactory().createTextLayout(attributedCharacterSpanIterator, subCharMap, offset, frc);
/*      */ 
/*      */       
/*  632 */       textRuns.add(new TextRun(this, layout, attributedCharacterSpanIterator, isChunkStart));
/*      */ 
/*      */ 
/*      */       
/*  636 */       Point2D layoutAdvance = layout.getAdvance2D();
/*      */       
/*  638 */       advance.x += (float)layoutAdvance.getX();
/*  639 */       advance.y += (float)layoutAdvance.getY();
/*      */       
/*  641 */       endChunk++;
/*  642 */       if (aci.setIndex(end) == Character.MAX_VALUE)
/*  643 */         break;  isChunkStart = false;
/*      */     } 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  649 */     return new TextChunk(this, beginChunk, endChunk, advance);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected Point2D adjustChunkOffsets(Point2D location, List textRuns, TextChunk chunk) {
/*      */     Point2D.Float float;
/*  661 */     TextRun r = (TextRun)textRuns.get(chunk.begin);
/*  662 */     int anchorType = r.getAnchorType();
/*  663 */     Float length = r.getLength();
/*  664 */     Integer lengthAdj = r.getLengthAdjust();
/*      */     
/*  666 */     boolean doAdjust = true;
/*  667 */     if (length == null || length.isNaN()) {
/*  668 */       doAdjust = false;
/*      */     }
/*  670 */     int numChars = 0;
/*  671 */     for (int n = chunk.begin; n < chunk.end; n++) {
/*  672 */       r = (TextRun)textRuns.get(n);
/*  673 */       AttributedCharacterIterator aci = r.getACI();
/*  674 */       numChars += aci.getEndIndex() - aci.getBeginIndex();
/*      */     } 
/*  676 */     if (lengthAdj == GVTAttributedCharacterIterator.TextAttribute.ADJUST_SPACING && numChars == 1)
/*      */     {
/*      */       
/*  679 */       doAdjust = false;
/*      */     }
/*  681 */     float xScale = 1.0F;
/*  682 */     float yScale = 1.0F;
/*      */     
/*  684 */     r = (TextRun)textRuns.get(chunk.end - 1);
/*  685 */     TextSpanLayout layout = r.getLayout();
/*  686 */     GVTGlyphMetrics lastMetrics = layout.getGlyphMetrics(layout.getGlyphCount() - 1);
/*      */     
/*  688 */     GVTLineMetrics lastLineMetrics = layout.getLineMetrics();
/*  689 */     Rectangle2D lastBounds = lastMetrics.getBounds2D();
/*  690 */     float halfLeading = (lastMetrics.getVerticalAdvance() - lastLineMetrics.getAscent() + lastLineMetrics.getDescent()) / 2.0F;
/*      */ 
/*      */     
/*  693 */     float lastW = (float)(lastBounds.getWidth() + lastBounds.getX());
/*  694 */     float lastH = (float)((halfLeading + lastLineMetrics.getAscent()) + lastBounds.getHeight() + lastBounds.getY());
/*      */ 
/*      */ 
/*      */     
/*  698 */     if (!doAdjust) {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */       
/*  707 */       float = new Point2D.Float((float)(chunk.advance.getX() + lastW - lastMetrics.getHorizontalAdvance()), (float)(chunk.advance.getY() - lastMetrics.getVerticalAdvance() + lastH));
/*      */     
/*      */     }
/*      */     else {
/*      */ 
/*      */       
/*  713 */       Point2D advance = chunk.advance;
/*      */ 
/*      */ 
/*      */ 
/*      */       
/*  718 */       if (layout.isVertical()) {
/*  719 */         if (lengthAdj == ADJUST_SPACING) {
/*  720 */           yScale = (float)((length.floatValue() - lastH) / (advance.getY() - lastMetrics.getVerticalAdvance()));
/*      */         }
/*      */         else {
/*      */           
/*  724 */           double adv = advance.getY() - lastMetrics.getVerticalAdvance() + lastH;
/*      */           
/*  726 */           yScale = (float)(length.floatValue() / adv);
/*      */         } 
/*  728 */         float = new Point2D.Float(0.0F, length.floatValue());
/*      */       } else {
/*  730 */         if (lengthAdj == ADJUST_SPACING) {
/*  731 */           xScale = (float)((length.floatValue() - lastW) / (advance.getX() - lastMetrics.getHorizontalAdvance()));
/*      */         }
/*      */         else {
/*      */           
/*  735 */           double adv = advance.getX() + lastW - lastMetrics.getHorizontalAdvance();
/*      */           
/*  737 */           xScale = (float)(length.floatValue() / adv);
/*      */         } 
/*  739 */         float = new Point2D.Float(length.floatValue(), 0.0F);
/*      */       } 
/*      */ 
/*      */ 
/*      */       
/*  744 */       Point2D.Float adv = new Point2D.Float(0.0F, 0.0F);
/*  745 */       for (int n = chunk.begin; n < chunk.end; n++) {
/*  746 */         r = (TextRun)textRuns.get(n);
/*  747 */         layout = r.getLayout();
/*  748 */         layout.setScale(xScale, yScale, (lengthAdj == ADJUST_SPACING));
/*  749 */         Point2D lAdv = layout.getAdvance2D();
/*  750 */         adv.x += (float)lAdv.getX();
/*  751 */         adv.y += (float)lAdv.getY();
/*      */       } 
/*  753 */       chunk.advance = adv;
/*      */     } 
/*      */     
/*  756 */     float dx = 0.0F;
/*  757 */     float dy = 0.0F;
/*  758 */     switch (anchorType) {
/*      */       case 1:
/*  760 */         dx = (float)(-float.getX() / 2.0D);
/*  761 */         dy = (float)(-float.getY() / 2.0D);
/*      */         break;
/*      */       case 2:
/*  764 */         dx = (float)-float.getX();
/*  765 */         dy = (float)-float.getY();
/*      */         break;
/*      */     } 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  774 */     r = (TextRun)textRuns.get(chunk.begin);
/*  775 */     layout = r.getLayout();
/*  776 */     AttributedCharacterIterator runaci = r.getACI();
/*  777 */     runaci.first();
/*  778 */     boolean vertical = layout.isVertical();
/*  779 */     Float runX = (Float)runaci.getAttribute(XPOS);
/*  780 */     Float runY = (Float)runaci.getAttribute(YPOS);
/*  781 */     TextPath textPath = (TextPath)runaci.getAttribute(TEXTPATH);
/*      */ 
/*      */ 
/*      */     
/*  785 */     float absX = (float)location.getX();
/*  786 */     float absY = (float)location.getY();
/*      */     
/*  788 */     float tpShiftX = 0.0F;
/*  789 */     float tpShiftY = 0.0F;
/*      */ 
/*      */ 
/*      */     
/*  793 */     if (runX != null && !runX.isNaN()) {
/*  794 */       absX = runX.floatValue();
/*  795 */       tpShiftX = absX;
/*      */     } 
/*      */     
/*  798 */     if (runY != null && !runY.isNaN()) {
/*  799 */       absY = runY.floatValue();
/*  800 */       tpShiftY = absY;
/*      */     } 
/*      */ 
/*      */ 
/*      */     
/*  805 */     if (vertical) {
/*  806 */       absY += dy;
/*  807 */       tpShiftY += dy;
/*  808 */       tpShiftX = 0.0F;
/*      */     } else {
/*  810 */       absX += dx;
/*  811 */       tpShiftX += dx;
/*  812 */       tpShiftY = 0.0F;
/*      */     } 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  818 */     for (int n = chunk.begin; n < chunk.end; n++) {
/*  819 */       r = (TextRun)textRuns.get(n);
/*  820 */       layout = r.getLayout();
/*  821 */       runaci = r.getACI();
/*  822 */       runaci.first();
/*  823 */       textPath = (TextPath)runaci.getAttribute(TEXTPATH);
/*  824 */       if (vertical) {
/*  825 */         runX = (Float)runaci.getAttribute(XPOS);
/*  826 */         if (runX != null && !runX.isNaN()) {
/*  827 */           absX = runX.floatValue();
/*      */         }
/*      */       } else {
/*  830 */         runY = (Float)runaci.getAttribute(YPOS);
/*  831 */         if (runY != null && !runY.isNaN()) {
/*  832 */           absY = runY.floatValue();
/*      */         }
/*      */       } 
/*      */       
/*  836 */       if (textPath == null) {
/*  837 */         layout.setOffset(new Point2D.Float(absX, absY));
/*      */         
/*  839 */         Point2D ladv = layout.getAdvance2D();
/*  840 */         absX = (float)(absX + ladv.getX());
/*  841 */         absY = (float)(absY + ladv.getY());
/*      */       } else {
/*  843 */         layout.setOffset(new Point2D.Float(tpShiftX, tpShiftY));
/*      */         
/*  845 */         Point2D ladv = layout.getAdvance2D();
/*  846 */         tpShiftX += (float)ladv.getX();
/*  847 */         tpShiftY += (float)ladv.getY();
/*      */         
/*  849 */         ladv = layout.getTextPathAdvance();
/*  850 */         absX = (float)ladv.getX();
/*  851 */         absY = (float)ladv.getY();
/*      */       } 
/*      */     } 
/*  854 */     return new Point2D.Float(absX, absY);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void paintDecorations(List textRuns, Graphics2D g2d, int decorationType) {
/*  863 */     Paint prevPaint = null;
/*  864 */     Paint prevStrokePaint = null;
/*  865 */     Stroke prevStroke = null;
/*  866 */     Rectangle2D decorationRect = null;
/*  867 */     double yLoc = 0.0D, height = 0.0D;
/*      */     
/*  869 */     for (int i = 0; i < textRuns.size(); i++) {
/*  870 */       TextRun textRun = (TextRun)textRuns.get(i);
/*  871 */       AttributedCharacterIterator runaci = textRun.getACI();
/*  872 */       runaci.first();
/*      */       
/*  874 */       TextPaintInfo tpi = (TextPaintInfo)runaci.getAttribute(PAINT_INFO);
/*  875 */       if (tpi != null && tpi.composite != null) {
/*  876 */         g2d.setComposite(tpi.composite);
/*      */       }
/*      */       
/*  879 */       Paint paint = null;
/*  880 */       Stroke stroke = null;
/*  881 */       Paint strokePaint = null;
/*  882 */       if (tpi != null) {
/*  883 */         switch (decorationType) {
/*      */           case 1:
/*  885 */             paint = tpi.underlinePaint;
/*  886 */             stroke = tpi.underlineStroke;
/*  887 */             strokePaint = tpi.underlineStrokePaint;
/*      */             break;
/*      */           case 4:
/*  890 */             paint = tpi.overlinePaint;
/*  891 */             stroke = tpi.overlineStroke;
/*  892 */             strokePaint = tpi.overlineStrokePaint;
/*      */             break;
/*      */           case 2:
/*  895 */             paint = tpi.strikethroughPaint;
/*  896 */             stroke = tpi.strikethroughStroke;
/*  897 */             strokePaint = tpi.strikethroughStrokePaint;
/*      */             break;
/*      */           
/*      */           default:
/*      */             return;
/*      */         } 
/*      */       
/*      */       }
/*  905 */       if (textRun.isFirstRunInChunk()) {
/*  906 */         Shape s = textRun.getLayout().getDecorationOutline(decorationType);
/*      */         
/*  908 */         Rectangle2D r2d = s.getBounds2D();
/*  909 */         yLoc = r2d.getY();
/*  910 */         height = r2d.getHeight();
/*      */       } 
/*      */       
/*  913 */       if (textRun.isFirstRunInChunk() || paint != prevPaint || stroke != prevStroke || strokePaint != prevStrokePaint)
/*      */       {
/*      */ 
/*      */ 
/*      */         
/*  918 */         if (decorationRect != null) {
/*      */           
/*  920 */           if (prevPaint != null) {
/*      */             
/*  922 */             g2d.setPaint(prevPaint);
/*  923 */             g2d.fill(decorationRect);
/*      */           } 
/*  925 */           if (prevStroke != null && prevStrokePaint != null) {
/*      */             
/*  927 */             g2d.setPaint(prevStrokePaint);
/*  928 */             g2d.setStroke(prevStroke);
/*  929 */             g2d.draw(decorationRect);
/*      */           } 
/*  931 */           decorationRect = null;
/*      */         } 
/*      */       }
/*      */       
/*  935 */       if ((paint != null || strokePaint != null) && !textRun.getLayout().isVertical() && !textRun.getLayout().isOnATextPath()) {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */         
/*  944 */         Shape decorationShape = textRun.getLayout().getDecorationOutline(decorationType);
/*      */         
/*  946 */         if (decorationRect == null) {
/*      */           
/*  948 */           Rectangle2D r2d = decorationShape.getBounds2D();
/*  949 */           decorationRect = new Rectangle2D.Double(r2d.getX(), yLoc, r2d.getWidth(), height);
/*      */         }
/*      */         else {
/*      */           
/*  953 */           Rectangle2D bounds = decorationShape.getBounds2D();
/*  954 */           double minX = Math.min(decorationRect.getX(), bounds.getX());
/*      */           
/*  956 */           double maxX = Math.max(decorationRect.getMaxX(), bounds.getMaxX());
/*      */           
/*  958 */           decorationRect.setRect(minX, yLoc, maxX - minX, height);
/*      */         } 
/*      */       } 
/*  961 */       prevPaint = paint;
/*  962 */       prevStroke = stroke;
/*  963 */       prevStrokePaint = strokePaint;
/*      */     } 
/*      */ 
/*      */ 
/*      */     
/*  968 */     if (decorationRect != null) {
/*      */       
/*  970 */       if (prevPaint != null) {
/*      */         
/*  972 */         g2d.setPaint(prevPaint);
/*  973 */         g2d.fill(decorationRect);
/*      */       } 
/*  975 */       if (prevStroke != null && prevStrokePaint != null) {
/*      */         
/*  977 */         g2d.setPaint(prevStrokePaint);
/*  978 */         g2d.setStroke(prevStroke);
/*  979 */         g2d.draw(decorationRect);
/*      */       } 
/*      */     } 
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void paintTextRuns(List textRuns, Graphics2D g2d) {
/*  990 */     for (int i = 0; i < textRuns.size(); i++) {
/*  991 */       TextRun textRun = (TextRun)textRuns.get(i);
/*  992 */       AttributedCharacterIterator runaci = textRun.getACI();
/*  993 */       runaci.first();
/*      */       
/*  995 */       TextPaintInfo tpi = (TextPaintInfo)runaci.getAttribute(PAINT_INFO);
/*  996 */       if (tpi != null && tpi.composite != null) {
/*  997 */         g2d.setComposite(tpi.composite);
/*      */       }
/*  999 */       textRun.getLayout().draw(g2d);
/*      */     } 
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public Shape getOutline(TextNode node) {
/* 1009 */     GeneralPath outline = null;
/*      */     
/* 1011 */     AttributedCharacterIterator aci = node.getAttributedCharacterIterator();
/* 1012 */     if (aci == null) {
/* 1013 */       return null;
/*      */     }
/*      */     
/* 1016 */     List textRuns = getTextRuns(node, aci);
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 1021 */     for (int i = 0; i < textRuns.size(); i++) {
/* 1022 */       TextRun textRun = (TextRun)textRuns.get(i);
/* 1023 */       TextSpanLayout textRunLayout = textRun.getLayout();
/* 1024 */       GeneralPath textRunOutline = new GeneralPath(textRunLayout.getOutline());
/*      */ 
/*      */       
/* 1027 */       if (outline == null) {
/* 1028 */         outline = textRunOutline;
/*      */       } else {
/* 1030 */         outline.setWindingRule(1);
/* 1031 */         outline.append(textRunOutline, false);
/*      */       } 
/*      */     } 
/*      */ 
/*      */     
/* 1036 */     Shape underline = getDecorationOutline(textRuns, 1);
/*      */ 
/*      */     
/* 1039 */     Shape strikeThrough = getDecorationOutline(textRuns, 2);
/*      */ 
/*      */     
/* 1042 */     Shape overline = getDecorationOutline(textRuns, 4);
/*      */ 
/*      */     
/* 1045 */     if (underline != null) {
/* 1046 */       if (outline == null) {
/* 1047 */         outline = new GeneralPath(underline);
/*      */       } else {
/* 1049 */         outline.setWindingRule(1);
/* 1050 */         outline.append(underline, false);
/*      */       } 
/*      */     }
/* 1053 */     if (strikeThrough != null) {
/* 1054 */       if (outline == null) {
/* 1055 */         outline = new GeneralPath(strikeThrough);
/*      */       } else {
/* 1057 */         outline.setWindingRule(1);
/* 1058 */         outline.append(strikeThrough, false);
/*      */       } 
/*      */     }
/* 1061 */     if (overline != null) {
/* 1062 */       if (outline == null) {
/* 1063 */         outline = new GeneralPath(overline);
/*      */       } else {
/* 1065 */         outline.setWindingRule(1);
/* 1066 */         outline.append(overline, false);
/*      */       } 
/*      */     }
/*      */     
/* 1070 */     return outline;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public Rectangle2D getBounds2D(TextNode node) {
/* 1080 */     AttributedCharacterIterator aci = node.getAttributedCharacterIterator();
/* 1081 */     if (aci == null) {
/* 1082 */       return null;
/*      */     }
/*      */     
/* 1085 */     List textRuns = getTextRuns(node, aci);
/*      */     
/* 1087 */     Rectangle2D bounds = null;
/*      */ 
/*      */     
/* 1090 */     for (int i = 0; i < textRuns.size(); i++) {
/* 1091 */       TextRun textRun = (TextRun)textRuns.get(i);
/* 1092 */       TextSpanLayout textRunLayout = textRun.getLayout();
/* 1093 */       Rectangle2D runBounds = textRunLayout.getBounds2D();
/* 1094 */       if (runBounds != null) {
/* 1095 */         if (bounds == null) {
/* 1096 */           bounds = runBounds;
/*      */         } else {
/*      */           
/* 1099 */           bounds.add(runBounds);
/*      */         } 
/*      */       }
/*      */     } 
/*      */ 
/*      */     
/* 1105 */     Shape underline = getDecorationStrokeOutline(textRuns, 1);
/*      */ 
/*      */     
/* 1108 */     if (underline != null) {
/* 1109 */       if (bounds == null) {
/* 1110 */         bounds = underline.getBounds2D();
/*      */       } else {
/*      */         
/* 1113 */         bounds.add(underline.getBounds2D());
/*      */       } 
/*      */     }
/* 1116 */     Shape strikeThrough = getDecorationStrokeOutline(textRuns, 2);
/*      */     
/* 1118 */     if (strikeThrough != null) {
/* 1119 */       if (bounds == null) {
/* 1120 */         bounds = strikeThrough.getBounds2D();
/*      */       } else {
/*      */         
/* 1123 */         bounds.add(strikeThrough.getBounds2D());
/*      */       } 
/*      */     }
/* 1126 */     Shape overline = getDecorationStrokeOutline(textRuns, 4);
/*      */     
/* 1128 */     if (overline != null)
/* 1129 */       if (bounds == null) {
/* 1130 */         bounds = overline.getBounds2D();
/*      */       } else {
/*      */         
/* 1133 */         bounds.add(overline.getBounds2D());
/*      */       }  
/* 1135 */     return bounds;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected Shape getDecorationOutline(List textRuns, int decorationType) {
/* 1150 */     GeneralPath outline = null;
/*      */     
/* 1152 */     Paint prevPaint = null;
/* 1153 */     Paint prevStrokePaint = null;
/* 1154 */     Stroke prevStroke = null;
/* 1155 */     Rectangle2D decorationRect = null;
/* 1156 */     double yLoc = 0.0D, height = 0.0D;
/*      */     
/* 1158 */     for (int i = 0; i < textRuns.size(); i++) {
/* 1159 */       TextRun textRun = (TextRun)textRuns.get(i);
/* 1160 */       AttributedCharacterIterator runaci = textRun.getACI();
/* 1161 */       runaci.first();
/*      */       
/* 1163 */       Paint paint = null;
/* 1164 */       Stroke stroke = null;
/* 1165 */       Paint strokePaint = null;
/* 1166 */       TextPaintInfo tpi = (TextPaintInfo)runaci.getAttribute(PAINT_INFO);
/* 1167 */       if (tpi != null) {
/* 1168 */         switch (decorationType) {
/*      */           case 1:
/* 1170 */             paint = tpi.underlinePaint;
/* 1171 */             stroke = tpi.underlineStroke;
/* 1172 */             strokePaint = tpi.underlineStrokePaint;
/*      */             break;
/*      */           case 4:
/* 1175 */             paint = tpi.overlinePaint;
/* 1176 */             stroke = tpi.overlineStroke;
/* 1177 */             strokePaint = tpi.overlineStrokePaint;
/*      */             break;
/*      */           case 2:
/* 1180 */             paint = tpi.strikethroughPaint;
/* 1181 */             stroke = tpi.strikethroughStroke;
/* 1182 */             strokePaint = tpi.strikethroughStrokePaint;
/*      */             break;
/*      */           
/*      */           default:
/* 1186 */             return null;
/*      */         } 
/*      */       
/*      */       }
/* 1190 */       if (textRun.isFirstRunInChunk()) {
/* 1191 */         Shape s = textRun.getLayout().getDecorationOutline(decorationType);
/*      */         
/* 1193 */         Rectangle2D r2d = s.getBounds2D();
/* 1194 */         yLoc = r2d.getY();
/* 1195 */         height = r2d.getHeight();
/*      */       } 
/*      */       
/* 1198 */       if (textRun.isFirstRunInChunk() || paint != prevPaint || stroke != prevStroke || strokePaint != prevStrokePaint)
/*      */       {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */         
/* 1205 */         if (decorationRect != null) {
/* 1206 */           if (outline == null) {
/* 1207 */             outline = new GeneralPath(decorationRect);
/*      */           } else {
/* 1209 */             outline.append(decorationRect, false);
/*      */           } 
/* 1211 */           decorationRect = null;
/*      */         } 
/*      */       }
/*      */       
/* 1215 */       if ((paint != null || strokePaint != null) && !textRun.getLayout().isVertical() && !textRun.getLayout().isOnATextPath()) {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */         
/* 1223 */         Shape decorationShape = textRun.getLayout().getDecorationOutline(decorationType);
/*      */         
/* 1225 */         if (decorationRect == null) {
/*      */           
/* 1227 */           Rectangle2D r2d = decorationShape.getBounds2D();
/* 1228 */           decorationRect = new Rectangle2D.Double(r2d.getX(), yLoc, r2d.getWidth(), height);
/*      */         }
/*      */         else {
/*      */           
/* 1232 */           Rectangle2D bounds = decorationShape.getBounds2D();
/* 1233 */           double minX = Math.min(decorationRect.getX(), bounds.getX());
/*      */           
/* 1235 */           double maxX = Math.max(decorationRect.getMaxX(), bounds.getMaxX());
/*      */           
/* 1237 */           decorationRect.setRect(minX, yLoc, maxX - minX, height);
/*      */         } 
/*      */       } 
/*      */       
/* 1241 */       prevPaint = paint;
/* 1242 */       prevStroke = stroke;
/* 1243 */       prevStrokePaint = strokePaint;
/*      */     } 
/*      */ 
/*      */     
/* 1247 */     if (decorationRect != null) {
/* 1248 */       if (outline == null) {
/* 1249 */         outline = new GeneralPath(decorationRect);
/*      */       } else {
/* 1251 */         outline.append(decorationRect, false);
/*      */       } 
/*      */     }
/*      */     
/* 1255 */     return outline;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected Shape getDecorationStrokeOutline(List textRuns, int decorationType) {
/* 1271 */     GeneralPath outline = null;
/*      */     
/* 1273 */     Paint prevPaint = null;
/* 1274 */     Paint prevStrokePaint = null;
/* 1275 */     Stroke prevStroke = null;
/* 1276 */     Rectangle2D decorationRect = null;
/* 1277 */     double yLoc = 0.0D, height = 0.0D;
/*      */     
/* 1279 */     for (int i = 0; i < textRuns.size(); i++) {
/*      */       
/* 1281 */       TextRun textRun = (TextRun)textRuns.get(i);
/* 1282 */       AttributedCharacterIterator runaci = textRun.getACI();
/* 1283 */       runaci.first();
/*      */       
/* 1285 */       Paint paint = null;
/* 1286 */       Stroke stroke = null;
/* 1287 */       Paint strokePaint = null;
/* 1288 */       TextPaintInfo tpi = (TextPaintInfo)runaci.getAttribute(PAINT_INFO);
/* 1289 */       if (tpi != null) {
/* 1290 */         switch (decorationType) {
/*      */           case 1:
/* 1292 */             paint = tpi.underlinePaint;
/* 1293 */             stroke = tpi.underlineStroke;
/* 1294 */             strokePaint = tpi.underlineStrokePaint;
/*      */             break;
/*      */           case 4:
/* 1297 */             paint = tpi.overlinePaint;
/* 1298 */             stroke = tpi.overlineStroke;
/* 1299 */             strokePaint = tpi.overlineStrokePaint;
/*      */             break;
/*      */           case 2:
/* 1302 */             paint = tpi.strikethroughPaint;
/* 1303 */             stroke = tpi.strikethroughStroke;
/* 1304 */             strokePaint = tpi.strikethroughStrokePaint;
/*      */             break;
/*      */           
/*      */           default:
/* 1308 */             return null;
/*      */         } 
/*      */       
/*      */       }
/* 1312 */       if (textRun.isFirstRunInChunk()) {
/* 1313 */         Shape s = textRun.getLayout().getDecorationOutline(decorationType);
/*      */         
/* 1315 */         Rectangle2D r2d = s.getBounds2D();
/* 1316 */         yLoc = r2d.getY();
/* 1317 */         height = r2d.getHeight();
/*      */       } 
/*      */       
/* 1320 */       if (textRun.isFirstRunInChunk() || paint != prevPaint || stroke != prevStroke || strokePaint != prevStrokePaint)
/*      */       {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */         
/* 1327 */         if (decorationRect != null) {
/*      */           
/* 1329 */           Shape s = null;
/* 1330 */           if (prevStroke != null && prevStrokePaint != null) {
/*      */             
/* 1332 */             s = prevStroke.createStrokedShape(decorationRect);
/* 1333 */           } else if (prevPaint != null) {
/* 1334 */             s = decorationRect;
/* 1335 */           }  if (s != null)
/* 1336 */             if (outline == null) {
/* 1337 */               outline = new GeneralPath(s);
/*      */             } else {
/* 1339 */               outline.append(s, false);
/*      */             }  
/* 1341 */           decorationRect = null;
/*      */         } 
/*      */       }
/*      */       
/* 1345 */       if ((paint != null || strokePaint != null) && !textRun.getLayout().isVertical() && !textRun.getLayout().isOnATextPath()) {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */         
/* 1353 */         Shape decorationShape = textRun.getLayout().getDecorationOutline(decorationType);
/*      */ 
/*      */         
/* 1356 */         if (decorationRect == null) {
/*      */           
/* 1358 */           Rectangle2D r2d = decorationShape.getBounds2D();
/* 1359 */           decorationRect = new Rectangle2D.Double(r2d.getX(), yLoc, r2d.getWidth(), height);
/*      */         }
/*      */         else {
/*      */           
/* 1363 */           Rectangle2D bounds = decorationShape.getBounds2D();
/* 1364 */           double minX = Math.min(decorationRect.getX(), bounds.getX());
/*      */           
/* 1366 */           double maxX = Math.max(decorationRect.getMaxX(), bounds.getMaxX());
/*      */           
/* 1368 */           decorationRect.setRect(minX, yLoc, maxX - minX, height);
/*      */         } 
/*      */       } 
/*      */       
/* 1372 */       prevPaint = paint;
/* 1373 */       prevStroke = stroke;
/* 1374 */       prevStrokePaint = strokePaint;
/*      */     } 
/*      */ 
/*      */ 
/*      */     
/* 1379 */     if (decorationRect != null) {
/* 1380 */       Shape s = null;
/* 1381 */       if (prevStroke != null && prevStrokePaint != null) {
/*      */         
/* 1383 */         s = prevStroke.createStrokedShape(decorationRect);
/* 1384 */       } else if (prevPaint != null) {
/* 1385 */         s = decorationRect;
/* 1386 */       }  if (s != null) {
/* 1387 */         if (outline == null) {
/* 1388 */           outline = new GeneralPath(s);
/*      */         } else {
/* 1390 */           outline.append(s, false);
/*      */         } 
/*      */       }
/*      */     } 
/* 1394 */     return outline;
/*      */   }
/*      */ 
/*      */ 
/*      */   
/*      */   public Mark getMark(TextNode node, int index, boolean leadingEdge) {
/* 1400 */     AttributedCharacterIterator aci = node.getAttributedCharacterIterator();
/* 1401 */     if (aci == null) {
/* 1402 */       return null;
/*      */     }
/* 1404 */     if (index < aci.getBeginIndex() || index > aci.getEndIndex())
/*      */     {
/* 1406 */       return null;
/*      */     }
/* 1408 */     TextHit textHit = new TextHit(index, leadingEdge);
/* 1409 */     return new BasicTextPainter.BasicMark(node, textHit);
/*      */   }
/*      */ 
/*      */   
/*      */   protected Mark hitTest(double x, double y, TextNode node) {
/* 1414 */     AttributedCharacterIterator aci = node.getAttributedCharacterIterator();
/* 1415 */     if (aci == null) {
/* 1416 */       return null;
/*      */     }
/*      */     
/* 1419 */     List textRuns = getTextRuns(node, aci);
/*      */ 
/*      */     
/* 1422 */     for (int i = 0; i < textRuns.size(); i++) {
/* 1423 */       TextRun textRun = (TextRun)textRuns.get(i);
/* 1424 */       TextSpanLayout layout = textRun.getLayout();
/* 1425 */       TextHit textHit = layout.hitTestChar((float)x, (float)y);
/* 1426 */       if (textHit != null && layout.getBounds2D().contains(x, y)) {
/* 1427 */         return new BasicTextPainter.BasicMark(node, textHit);
/*      */       }
/*      */     } 
/*      */     
/* 1431 */     return null;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public Mark selectFirst(TextNode node) {
/* 1439 */     AttributedCharacterIterator aci = node.getAttributedCharacterIterator();
/* 1440 */     if (aci == null) {
/* 1441 */       return null;
/*      */     }
/* 1443 */     TextHit textHit = new TextHit(aci.getBeginIndex(), false);
/* 1444 */     return new BasicTextPainter.BasicMark(node, textHit);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public Mark selectLast(TextNode node) {
/* 1452 */     AttributedCharacterIterator aci = node.getAttributedCharacterIterator();
/* 1453 */     if (aci == null) {
/* 1454 */       return null;
/*      */     }
/* 1456 */     TextHit textHit = new TextHit(aci.getEndIndex() - 1, false);
/* 1457 */     return new BasicTextPainter.BasicMark(node, textHit);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public int[] getSelected(Mark startMark, Mark finishMark) {
/*      */     BasicTextPainter.BasicMark finish, start;
/* 1470 */     if (startMark == null || finishMark == null) {
/* 1471 */       return null;
/*      */     }
/*      */ 
/*      */     
/*      */     try {
/* 1476 */       start = (BasicTextPainter.BasicMark)startMark;
/* 1477 */       finish = (BasicTextPainter.BasicMark)finishMark;
/*      */     } catch (ClassCastException cce) {
/* 1479 */       throw new Error("This Mark was not instantiated by this TextPainter class!");
/*      */     } 
/*      */ 
/*      */     
/* 1483 */     TextNode textNode = start.getTextNode();
/* 1484 */     if (textNode == null)
/* 1485 */       return null; 
/* 1486 */     if (textNode != finish.getTextNode()) {
/* 1487 */       throw new Error("Markers are from different TextNodes!");
/*      */     }
/*      */     
/* 1490 */     AttributedCharacterIterator aci = textNode.getAttributedCharacterIterator();
/* 1491 */     if (aci == null) {
/* 1492 */       return null;
/*      */     }
/* 1494 */     int[] result = new int[2];
/* 1495 */     result[0] = start.getHit().getCharIndex();
/* 1496 */     result[1] = finish.getHit().getCharIndex();
/*      */ 
/*      */     
/* 1499 */     List textRuns = getTextRuns(textNode, aci);
/* 1500 */     Iterator trI = textRuns.iterator();
/* 1501 */     int startGlyphIndex = -1;
/* 1502 */     int endGlyphIndex = -1;
/* 1503 */     TextSpanLayout startLayout = null, endLayout = null;
/* 1504 */     while (trI.hasNext()) {
/* 1505 */       TextRun tr = (TextRun)trI.next();
/* 1506 */       TextSpanLayout tsl = tr.getLayout();
/* 1507 */       if (startGlyphIndex == -1) {
/* 1508 */         startGlyphIndex = tsl.getGlyphIndex(result[0]);
/* 1509 */         if (startGlyphIndex != -1) {
/* 1510 */           startLayout = tsl;
/*      */         }
/*      */       } 
/* 1513 */       if (endGlyphIndex == -1) {
/* 1514 */         endGlyphIndex = tsl.getGlyphIndex(result[1]);
/* 1515 */         if (endGlyphIndex != -1)
/* 1516 */           endLayout = tsl; 
/*      */       } 
/* 1518 */       if (startGlyphIndex != -1 && endGlyphIndex != -1)
/*      */         break; 
/*      */     } 
/* 1521 */     if (startLayout == null || endLayout == null) {
/* 1522 */       return null;
/*      */     }
/* 1524 */     int startCharCount = startLayout.getCharacterCount(startGlyphIndex, startGlyphIndex);
/*      */     
/* 1526 */     int endCharCount = endLayout.getCharacterCount(endGlyphIndex, endGlyphIndex);
/*      */     
/* 1528 */     if (startCharCount > 1) {
/* 1529 */       if (result[0] > result[1] && startLayout.isLeftToRight()) {
/* 1530 */         result[0] = result[0] + startCharCount - 1;
/* 1531 */       } else if (result[1] > result[0] && !startLayout.isLeftToRight()) {
/* 1532 */         result[0] = result[0] - startCharCount - 1;
/*      */       } 
/*      */     }
/* 1535 */     if (endCharCount > 1) {
/* 1536 */       if (result[1] > result[0] && endLayout.isLeftToRight()) {
/* 1537 */         result[1] = result[1] + endCharCount - 1;
/* 1538 */       } else if (result[0] > result[1] && !endLayout.isLeftToRight()) {
/* 1539 */         result[1] = result[1] - endCharCount - 1;
/*      */       } 
/*      */     }
/*      */     
/* 1543 */     return result;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public Shape getHighlightShape(Mark beginMark, Mark endMark) {
/*      */     BasicTextPainter.BasicMark end, begin;
/* 1554 */     if (beginMark == null || endMark == null) {
/* 1555 */       return null;
/*      */     }
/*      */ 
/*      */ 
/*      */     
/*      */     try {
/* 1561 */       begin = (BasicTextPainter.BasicMark)beginMark;
/* 1562 */       end = (BasicTextPainter.BasicMark)endMark;
/*      */     } catch (ClassCastException cce) {
/* 1564 */       throw new Error("This Mark was not instantiated by this TextPainter class!");
/*      */     } 
/*      */ 
/*      */     
/* 1568 */     TextNode textNode = begin.getTextNode();
/* 1569 */     if (textNode == null)
/* 1570 */       return null; 
/* 1571 */     if (textNode != end.getTextNode()) {
/* 1572 */       throw new Error("Markers are from different TextNodes!");
/*      */     }
/*      */     
/* 1575 */     AttributedCharacterIterator aci = textNode.getAttributedCharacterIterator();
/* 1576 */     if (aci == null) {
/* 1577 */       return null;
/*      */     }
/* 1579 */     int beginIndex = begin.getHit().getCharIndex();
/* 1580 */     int endIndex = end.getHit().getCharIndex();
/* 1581 */     if (beginIndex > endIndex) {
/*      */       
/* 1583 */       BasicTextPainter.BasicMark tmpMark = begin;
/* 1584 */       begin = end; end = tmpMark;
/*      */       
/* 1586 */       int tmpIndex = beginIndex;
/* 1587 */       beginIndex = endIndex; endIndex = tmpIndex;
/*      */     } 
/*      */ 
/*      */     
/* 1591 */     List textRuns = getTextRuns(textNode, aci);
/*      */     
/* 1593 */     GeneralPath highlightedShape = new GeneralPath();
/*      */ 
/*      */ 
/*      */     
/* 1597 */     for (int i = 0; i < textRuns.size(); i++) {
/* 1598 */       TextRun textRun = (TextRun)textRuns.get(i);
/* 1599 */       TextSpanLayout layout = textRun.getLayout();
/*      */       
/* 1601 */       Shape layoutHighlightedShape = layout.getHighlightShape(beginIndex, endIndex);
/*      */ 
/*      */ 
/*      */ 
/*      */       
/* 1606 */       if (layoutHighlightedShape != null && !layoutHighlightedShape.getBounds().isEmpty())
/*      */       {
/* 1608 */         highlightedShape.append(layoutHighlightedShape, false);
/*      */       }
/*      */     } 
/* 1611 */     return highlightedShape;
/*      */   }
/*      */   
/*      */   class TextChunk
/*      */   {
/*      */     public int begin;
/*      */     public int end;
/*      */     public Point2D advance;
/*      */     private final StrokingTextPainter this$0;
/*      */     
/*      */     public TextChunk(StrokingTextPainter this$0, int begin, int end, Point2D advance) {
/* 1622 */       this.this$0 = this$0;
/* 1623 */       this.begin = begin;
/* 1624 */       this.end = end;
/* 1625 */       this.advance = new Point2D.Float((float)advance.getX(), (float)advance.getY());
/*      */     }
/*      */   }
/*      */ 
/*      */   
/*      */   public class TextRun
/*      */   {
/*      */     protected AttributedCharacterIterator aci;
/*      */     
/*      */     protected TextSpanLayout layout;
/*      */     
/*      */     protected int anchorType;
/*      */     
/*      */     protected boolean firstRunInChunk;
/*      */     
/*      */     protected Float length;
/*      */     
/*      */     protected Integer lengthAdjust;
/*      */     private final StrokingTextPainter this$0;
/*      */     
/*      */     public TextRun(StrokingTextPainter this$0, TextSpanLayout layout, AttributedCharacterIterator aci, boolean firstRunInChunk) {
/* 1646 */       this.this$0 = this$0;
/*      */       
/* 1648 */       this.layout = layout;
/* 1649 */       this.aci = aci;
/* 1650 */       this.aci.first();
/* 1651 */       this.firstRunInChunk = firstRunInChunk;
/* 1652 */       this.anchorType = 0;
/*      */       
/* 1654 */       TextNode.Anchor anchor = (TextNode.Anchor)aci.getAttribute(GVTAttributedCharacterIterator.TextAttribute.ANCHOR_TYPE);
/*      */       
/* 1656 */       if (anchor != null) {
/* 1657 */         this.anchorType = anchor.getType();
/*      */       }
/*      */ 
/*      */ 
/*      */       
/* 1662 */       if (aci.getAttribute(StrokingTextPainter.WRITING_MODE) == StrokingTextPainter.WRITING_MODE_RTL) {
/* 1663 */         if (this.anchorType == 0) {
/* 1664 */           this.anchorType = 2;
/* 1665 */         } else if (this.anchorType == 2) {
/* 1666 */           this.anchorType = 0;
/*      */         } 
/*      */       }
/*      */ 
/*      */       
/* 1671 */       this.length = (Float)aci.getAttribute(GVTAttributedCharacterIterator.TextAttribute.BBOX_WIDTH);
/*      */       
/* 1673 */       this.lengthAdjust = (Integer)aci.getAttribute(GVTAttributedCharacterIterator.TextAttribute.LENGTH_ADJUST);
/*      */     }
/*      */ 
/*      */ 
/*      */     
/* 1678 */     public AttributedCharacterIterator getACI() { return this.aci; }
/*      */ 
/*      */ 
/*      */     
/* 1682 */     public TextSpanLayout getLayout() { return this.layout; }
/*      */ 
/*      */ 
/*      */     
/* 1686 */     public int getAnchorType() { return this.anchorType; }
/*      */ 
/*      */ 
/*      */     
/* 1690 */     public Float getLength() { return this.length; }
/*      */ 
/*      */ 
/*      */     
/* 1694 */     public Integer getLengthAdjust() { return this.lengthAdjust; }
/*      */ 
/*      */ 
/*      */     
/* 1698 */     public boolean isFirstRunInChunk() { return this.firstRunInChunk; }
/*      */   }
/*      */ }


/* Location:              C:\Program Files (x86)\IBM\WebSphere\AppServer\profiles\ctgAppSrv01\installedApps\ctgCell01\MAXIMO.ear\maximouiweb.war\WEB-INF\classes\!\com\ibm\tivoli\maximo\report\control\svgtools\batik-gvt.jar!\org\apache\batik\gvt\renderer\StrokingTextPainter.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       1.0.7
 */