/*     */ package org.apache.batik.gvt.svg12;
/*     */ 
/*     */ import java.awt.Dimension;
/*     */ import java.awt.Graphics2D;
/*     */ import java.awt.Shape;
/*     */ import java.awt.geom.AffineTransform;
/*     */ import java.awt.geom.Rectangle2D;
/*     */ import java.lang.ref.SoftReference;
/*     */ import org.apache.batik.bridge.BridgeContext;
/*     */ import org.apache.batik.bridge.GVTBuilder;
/*     */ import org.apache.batik.gvt.AbstractGraphicsNode;
/*     */ import org.apache.batik.gvt.GraphicsNode;
/*     */ import org.apache.batik.util.SVGConstants;
/*     */ import org.w3c.dom.Element;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class MultiResGraphicsNode
/*     */   extends AbstractGraphicsNode
/*     */   implements SVGConstants
/*     */ {
/*     */   SoftReference[] srcs;
/*     */   Element[] srcElems;
/*     */   Dimension[] minSz;
/*     */   Dimension[] maxSz;
/*     */   Rectangle2D bounds;
/*     */   BridgeContext ctx;
/*     */   Element multiImgElem;
/*     */   
/*     */   public MultiResGraphicsNode(Element multiImgElem, Rectangle2D bounds, Element[] srcElems, Dimension[] minSz, Dimension[] maxSz, BridgeContext ctx) {
/*  62 */     this.multiImgElem = multiImgElem;
/*  63 */     this.srcElems = new Element[srcElems.length];
/*  64 */     this.minSz = new Dimension[srcElems.length];
/*  65 */     this.maxSz = new Dimension[srcElems.length];
/*  66 */     this.ctx = ctx;
/*     */     
/*  68 */     for (int i = 0; i < srcElems.length; i++) {
/*  69 */       this.srcElems[i] = srcElems[i];
/*  70 */       this.minSz[i] = minSz[i];
/*  71 */       this.maxSz[i] = maxSz[i];
/*     */     } 
/*     */     
/*  74 */     this.srcs = new SoftReference[srcElems.length];
/*  75 */     this.bounds = bounds;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void primitivePaint(Graphics2D g2d) {
/*     */     double d4, d3, d2, d1;
/*  85 */     AffineTransform at = g2d.getTransform();
/*     */     
/*  87 */     double scx = Math.sqrt(at.getShearY() * at.getShearY() + at.getScaleX() * at.getScaleX());
/*     */     
/*  89 */     double scy = Math.sqrt(at.getShearX() * at.getShearX() + at.getScaleY() * at.getScaleY());
/*     */ 
/*     */     
/*  92 */     GraphicsNode gn = null;
/*  93 */     int idx = -1;
/*  94 */     double w = this.bounds.getWidth() * scx;
/*  95 */     double minDist = calcDist(w, this.minSz[0], this.maxSz[0]);
/*  96 */     int minIdx = 0;
/*     */     
/*  98 */     for (int i = 0; i < this.minSz.length; i++) {
/*  99 */       double dist = calcDist(w, this.minSz[i], this.maxSz[i]);
/*     */       
/* 101 */       if (dist < minDist) {
/* 102 */         minDist = dist;
/* 103 */         minIdx = i;
/*     */       } 
/*     */       
/* 106 */       if ((this.minSz[i] == null || w >= (this.minSz[i]).width) && (this.maxSz[i] == null || w <= (this.maxSz[i]).width))
/*     */       {
/*     */ 
/*     */ 
/*     */         
/* 111 */         if (idx == -1 || minIdx == i) {
/* 112 */           idx = i;
/*     */         }
/*     */       }
/*     */     } 
/*     */     
/* 117 */     if (idx == -1)
/* 118 */       idx = minIdx; 
/* 119 */     gn = getGraphicsNode(idx);
/* 120 */     if (gn == null) {
/*     */       return;
/*     */     }
/*     */     
/* 124 */     Rectangle2D gnBounds = gn.getBounds();
/* 125 */     if (gnBounds == null)
/*     */       return; 
/* 127 */     double gnDevW = gnBounds.getWidth() * scx;
/* 128 */     double gnDevH = gnBounds.getHeight() * scy;
/* 129 */     double gnDevX = gnBounds.getX() * scx;
/* 130 */     double gnDevY = gnBounds.getY() * scy;
/*     */     
/* 132 */     if (gnDevW < 0.0D) {
/* 133 */       d1 = gnDevX + gnDevW;
/* 134 */       d2 = gnDevX;
/*     */     } else {
/* 136 */       d1 = gnDevX;
/* 137 */       d2 = gnDevX + gnDevW;
/*     */     } 
/* 139 */     if (gnDevH < 0.0D) {
/* 140 */       d3 = gnDevY + gnDevH;
/* 141 */       d4 = gnDevY;
/*     */     } else {
/* 143 */       d3 = gnDevY;
/* 144 */       d4 = gnDevY + gnDevH;
/*     */     } 
/*     */ 
/*     */     
/* 148 */     gnDevW = (int)(Math.ceil(d2) - Math.floor(d1));
/* 149 */     gnDevH = (int)(Math.ceil(d4) - Math.floor(d3));
/* 150 */     scx = gnDevW / gnBounds.getWidth() / scx;
/* 151 */     scy = gnDevH / gnBounds.getHeight() / scy;
/*     */ 
/*     */ 
/*     */     
/* 155 */     AffineTransform nat = g2d.getTransform();
/* 156 */     nat = new AffineTransform(nat.getScaleX() * scx, nat.getShearY() * scx, nat.getShearX() * scy, nat.getScaleY() * scy, nat.getTranslateX(), nat.getTranslateY());
/*     */ 
/*     */     
/* 159 */     g2d.setTransform(nat);
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 165 */     gn.paint(g2d);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public double calcDist(double loc, Dimension min, Dimension max) {
/* 180 */     if (min == null) {
/* 181 */       if (max == null) {
/* 182 */         return 1.0E11D;
/*     */       }
/* 184 */       return Math.abs(loc - max.width);
/*     */     } 
/* 186 */     if (max == null) {
/* 187 */       return Math.abs(loc - min.width);
/*     */     }
/* 189 */     double mid = (max.width + min.width) / 2.0D;
/* 190 */     return Math.abs(loc - mid);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 199 */   public Rectangle2D getPrimitiveBounds() { return this.bounds; }
/*     */ 
/*     */ 
/*     */   
/* 203 */   public Rectangle2D getGeometryBounds() { return this.bounds; }
/*     */ 
/*     */ 
/*     */   
/* 207 */   public Rectangle2D getSensitiveBounds() { return this.bounds; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 214 */   public Shape getOutline() { return this.bounds; }
/*     */ 
/*     */   
/*     */   public GraphicsNode getGraphicsNode(int idx) {
/* 218 */     if (this.srcs[idx] != null) {
/* 219 */       Object o = this.srcs[idx].get();
/* 220 */       if (o != null) {
/* 221 */         return (GraphicsNode)o;
/*     */       }
/*     */     } 
/*     */     
/* 225 */     try { GVTBuilder builder = this.ctx.getGVTBuilder();
/*     */       
/* 227 */       GraphicsNode gn = builder.build(this.ctx, this.srcElems[idx]);
/* 228 */       this.srcs[idx] = new SoftReference(gn);
/* 229 */       return gn; }
/* 230 */     catch (Exception ex) { ex.printStackTrace();
/*     */       
/* 232 */       return null; }
/*     */   
/*     */   }
/*     */ }


/* Location:              C:\Program Files (x86)\IBM\WebSphere\AppServer\profiles\ctgAppSrv01\installedApps\ctgCell01\MAXIMO.ear\maximouiweb.war\WEB-INF\classes\!\com\ibm\tivoli\maximo\report\control\svgtools\batik-gvt.jar!\org\apache\batik\gvt\svg12\MultiResGraphicsNode.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       1.0.7
 */