/*     */ package org.apache.batik.gvt.text;
/*     */ 
/*     */ import java.text.AttributedCharacterIterator;
/*     */ import java.text.AttributedString;
/*     */ import java.util.Map;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class ArabicTextHandler
/*     */ {
/*     */   private static final int arabicStart = 1536;
/*     */   private static final int arabicEnd = 1791;
/*  40 */   private static final AttributedCharacterIterator.Attribute ARABIC_FORM = GVTAttributedCharacterIterator.TextAttribute.ARABIC_FORM;
/*     */   
/*  42 */   private static final Integer ARABIC_NONE = GVTAttributedCharacterIterator.TextAttribute.ARABIC_NONE;
/*     */   
/*  44 */   private static final Integer ARABIC_ISOLATED = GVTAttributedCharacterIterator.TextAttribute.ARABIC_ISOLATED;
/*     */   
/*  46 */   private static final Integer ARABIC_TERMINAL = GVTAttributedCharacterIterator.TextAttribute.ARABIC_TERMINAL;
/*     */   
/*  48 */   private static final Integer ARABIC_INITIAL = GVTAttributedCharacterIterator.TextAttribute.ARABIC_INITIAL;
/*     */   
/*  50 */   private static final Integer ARABIC_MEDIAL = GVTAttributedCharacterIterator.TextAttribute.ARABIC_MEDIAL;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static AttributedString assignArabicForms(AttributedString as) {
/*  72 */     if (!containsArabic(as)) {
/*  73 */       return as;
/*     */     }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/*  80 */     AttributedCharacterIterator aci = as.getIterator();
/*  81 */     int numChars = aci.getEndIndex() - aci.getBeginIndex();
/*  82 */     int[] charOrder = null;
/*  83 */     if (numChars >= 3) {
/*  84 */       char prevChar = aci.first();
/*  85 */       char c = aci.next();
/*  86 */       int i = 1;
/*  87 */       char nextChar = aci.next();
/*  88 */       for (; nextChar != Character.MAX_VALUE; 
/*  89 */         prevChar = c, c = nextChar, nextChar = aci.next(), i++) {
/*  90 */         if (arabicCharTransparent(c) && 
/*  91 */           hasSubstitute(prevChar, nextChar)) {
/*     */           
/*  93 */           if (charOrder == null) {
/*  94 */             charOrder = new int[numChars];
/*  95 */             for (int j = 0; j < numChars; j++) {
/*  96 */               charOrder[j] = j + aci.getBeginIndex();
/*     */             }
/*     */           } 
/*  99 */           int temp = charOrder[i];
/* 100 */           charOrder[i] = charOrder[i - 1];
/* 101 */           charOrder[i - 1] = temp;
/*     */         } 
/*     */       } 
/*     */     } 
/*     */ 
/*     */     
/* 107 */     if (charOrder != null) {
/*     */       
/* 109 */       StringBuffer reorderedString = new StringBuffer(numChars);
/*     */       
/* 111 */       for (int i = 0; i < numChars; i++) {
/* 112 */         char c = aci.setIndex(charOrder[i]);
/* 113 */         reorderedString.append(c);
/*     */       } 
/*     */       
/* 116 */       AttributedString reorderedAS = new AttributedString(reorderedString.toString());
/*     */       
/* 118 */       for (int i = 0; i < numChars; i++) {
/* 119 */         aci.setIndex(charOrder[i]);
/* 120 */         Map attributes = aci.getAttributes();
/* 121 */         reorderedAS.addAttributes(attributes, i, i + 1);
/*     */       } 
/*     */       
/* 124 */       if (charOrder[0] != aci.getBeginIndex()) {
/*     */ 
/*     */ 
/*     */         
/* 128 */         aci.setIndex(charOrder[0]);
/* 129 */         Float x = (Float)aci.getAttribute(GVTAttributedCharacterIterator.TextAttribute.X);
/*     */         
/* 131 */         Float y = (Float)aci.getAttribute(GVTAttributedCharacterIterator.TextAttribute.Y);
/*     */ 
/*     */         
/* 134 */         if (x != null && !x.isNaN()) {
/* 135 */           reorderedAS.addAttribute(GVTAttributedCharacterIterator.TextAttribute.X, new Float(NaNF), charOrder[0], charOrder[0] + 1);
/*     */ 
/*     */           
/* 138 */           reorderedAS.addAttribute(GVTAttributedCharacterIterator.TextAttribute.X, x, 0, 1);
/*     */         } 
/*     */ 
/*     */         
/* 142 */         if (y != null && !y.isNaN()) {
/* 143 */           reorderedAS.addAttribute(GVTAttributedCharacterIterator.TextAttribute.Y, new Float(NaNF), charOrder[0], charOrder[0] + 1);
/*     */ 
/*     */           
/* 146 */           reorderedAS.addAttribute(GVTAttributedCharacterIterator.TextAttribute.Y, y, 0, 1);
/*     */         } 
/*     */       } 
/*     */ 
/*     */       
/* 151 */       as = reorderedAS;
/*     */     } 
/*     */ 
/*     */     
/* 155 */     aci = as.getIterator();
/* 156 */     int runStart = -1;
/* 157 */     int idx = aci.getBeginIndex();
/* 158 */     int c = aci.first();
/* 159 */     for (; c != 65535; 
/* 160 */       c = aci.next(), idx++) {
/* 161 */       if (c >= 1536 && c <= 1791) {
/* 162 */         if (runStart == -1)
/* 163 */           runStart = idx; 
/* 164 */       } else if (runStart != -1) {
/* 165 */         as.addAttribute(ARABIC_FORM, ARABIC_NONE, runStart, idx);
/* 166 */         runStart = -1;
/*     */       } 
/*     */     } 
/* 169 */     if (runStart != -1) {
/* 170 */       as.addAttribute(ARABIC_FORM, ARABIC_NONE, runStart, idx);
/*     */     }
/* 172 */     aci = as.getIterator();
/* 173 */     int end = aci.getBeginIndex();
/*     */     
/* 175 */     Integer currentForm = ARABIC_NONE;
/*     */     
/* 177 */     while (aci.setIndex(end) != Character.MAX_VALUE) {
/* 178 */       int start = aci.getRunStart(ARABIC_FORM);
/* 179 */       end = aci.getRunLimit(ARABIC_FORM);
/* 180 */       char currentChar = aci.setIndex(start);
/* 181 */       currentForm = (Integer)aci.getAttribute(ARABIC_FORM);
/*     */       
/* 183 */       if (currentForm == null) {
/*     */         continue;
/*     */       }
/*     */ 
/*     */ 
/*     */       
/* 189 */       int currentIndex = start;
/* 190 */       int prevCharIndex = start - 1;
/* 191 */       while (currentIndex < end) {
/* 192 */         char prevChar = currentChar;
/* 193 */         currentChar = aci.setIndex(currentIndex);
/* 194 */         while (arabicCharTransparent(currentChar) && currentIndex < end) {
/*     */           
/* 196 */           currentIndex++;
/* 197 */           currentChar = aci.setIndex(currentIndex);
/*     */         } 
/* 199 */         if (currentIndex >= end) {
/*     */           break;
/*     */         }
/*     */         
/* 203 */         Integer prevForm = currentForm;
/* 204 */         currentForm = ARABIC_NONE;
/* 205 */         if (prevCharIndex >= start) {
/*     */           
/* 207 */           if (arabicCharShapesRight(prevChar) && arabicCharShapesLeft(currentChar)) {
/*     */ 
/*     */             
/* 210 */             prevForm = new Integer(prevForm.intValue() + 1);
/* 211 */             as.addAttribute(ARABIC_FORM, prevForm, prevCharIndex, prevCharIndex + 1);
/*     */ 
/*     */ 
/*     */             
/* 215 */             currentForm = ARABIC_INITIAL;
/* 216 */           } else if (arabicCharShaped(currentChar)) {
/*     */             
/* 218 */             currentForm = ARABIC_ISOLATED;
/*     */           
/*     */           }
/*     */         
/*     */         }
/* 223 */         else if (arabicCharShaped(currentChar)) {
/*     */           
/* 225 */           currentForm = ARABIC_ISOLATED;
/*     */         } 
/* 227 */         if (currentForm != ARABIC_NONE) {
/* 228 */           as.addAttribute(ARABIC_FORM, currentForm, currentIndex, currentIndex + 1);
/*     */         }
/* 230 */         prevCharIndex = currentIndex;
/* 231 */         currentIndex++;
/*     */       } 
/*     */     } 
/* 234 */     return as;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static boolean arabicChar(char c) {
/* 245 */     if (c >= '؀' && c <= 'ۿ') {
/* 246 */       return true;
/*     */     }
/* 248 */     return false;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 258 */   public static boolean containsArabic(AttributedString as) { return containsArabic(as.getIterator()); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static boolean containsArabic(AttributedCharacterIterator aci) {
/* 268 */     char c = aci.first();
/* 269 */     for (; c != Character.MAX_VALUE; 
/* 270 */       c = aci.next()) {
/* 271 */       if (arabicChar(c)) {
/* 272 */         return true;
/*     */       }
/*     */     } 
/* 275 */     return false;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static boolean arabicCharTransparent(char c) {
/* 285 */     char c1 = c;
/* 286 */     if (c1 < 'ً' || c1 > 'ۭ') {
/* 287 */       return false;
/*     */     }
/* 289 */     if (c1 <= 'ٕ' || c1 == 'ٰ' || (c1 >= 'ۖ' && c1 <= 'ۤ') || (c1 >= 'ۧ' && c1 <= 'ۨ') || c1 >= '۪')
/*     */     {
/*     */ 
/*     */ 
/*     */       
/* 294 */       return true;
/*     */     }
/* 296 */     return false;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   private static boolean arabicCharShapesRight(char c) {
/* 307 */     char c1 = c;
/* 308 */     if ((c1 >= 'آ' && c1 <= 'إ') || c1 == 'ا' || c1 == 'ة' || (c1 >= 'د' && c1 <= 'ز') || c1 == 'و' || (c1 >= 'ٱ' && c1 <= 'ٳ') || (c1 >= 'ٵ' && c1 <= 'ٷ') || (c1 >= 'ڈ' && c1 <= 'ڙ') || c1 == 'ۀ' || (c1 >= 'ۂ' && c1 <= 'ۋ') || c1 == 'ۍ' || c1 == 'ۏ' || (c1 >= 'ے' && c1 <= 'ۓ') || arabicCharShapesDuel(c))
/*     */     {
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */       
/* 323 */       return true;
/*     */     }
/* 325 */     return false;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   private static boolean arabicCharShapesDuel(char c) {
/* 335 */     char c1 = c;
/*     */     
/* 337 */     if (c1 == 'ئ' || c1 == 'ب' || (c1 >= 'ت' && c1 <= 'خ') || (c1 >= 'س' && c1 <= 'غ') || (c1 >= 'ف' && c1 <= 'ه') || (c1 >= 'ى' && c1 <= 'ي') || (c1 >= 'ٸ' && c1 <= 'ڇ') || (c1 >= 'ښ' && c1 <= 'ڿ') || c1 == 'ہ' || c1 == 'ی' || c1 == 'ێ' || (c1 >= 'ې' && c1 <= 'ۑ') || (c1 >= 'ۺ' && c1 <= 'ۼ'))
/*     */     {
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */       
/* 350 */       return true;
/*     */     }
/* 352 */     return false;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 363 */   private static boolean arabicCharShapesLeft(char c) { return arabicCharShapesDuel(c); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 373 */   private static boolean arabicCharShaped(char c) { return arabicCharShapesRight(c); }
/*     */ 
/*     */   
/*     */   public static boolean hasSubstitute(char ch1, char ch2) {
/* 377 */     if (ch1 < doubleCharFirst || ch1 > doubleCharLast) return false;
/*     */     
/* 379 */     int[][] remaps = doubleCharRemappings[ch1 - doubleCharFirst];
/* 380 */     if (remaps == null) return false; 
/* 381 */     for (int i = 0; i < remaps.length; i++) {
/* 382 */       if (remaps[i][0] == ch2)
/* 383 */         return true; 
/*     */     } 
/* 385 */     return false;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static int getSubstituteChar(char ch1, char ch2, int form) {
/* 400 */     if (form == 0) return -1; 
/* 401 */     if (ch1 < doubleCharFirst || ch1 > doubleCharLast) return -1;
/*     */     
/* 403 */     int[][] remaps = doubleCharRemappings[ch1 - doubleCharFirst];
/* 404 */     if (remaps == null) return -1; 
/* 405 */     for (int i = 0; i < remaps.length; i++) {
/* 406 */       if (remaps[i][0] == ch2)
/* 407 */         return remaps[i][form]; 
/*     */     } 
/* 409 */     return -1;
/*     */   }
/*     */   
/*     */   public static int getSubstituteChar(char ch, int form) {
/* 413 */     if (form == 0) return -1; 
/* 414 */     if (ch < singleCharFirst || ch > singleCharLast) return -1;
/*     */     
/* 416 */     int[] chars = singleCharRemappings[ch - singleCharFirst];
/* 417 */     if (chars == null) return -1;
/*     */     
/* 419 */     return chars[form - 1];
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static String createSubstituteString(AttributedCharacterIterator aci) {
/* 435 */     int start = aci.getBeginIndex();
/* 436 */     int end = aci.getEndIndex();
/* 437 */     int numChar = end - start;
/* 438 */     StringBuffer substString = new StringBuffer(numChar);
/* 439 */     for (int i = start; i < end; i++) {
/* 440 */       char c = aci.setIndex(i);
/* 441 */       if (!arabicChar(c)) {
/* 442 */         substString.append(c);
/*     */         
/*     */         continue;
/*     */       } 
/* 446 */       Integer form = (Integer)aci.getAttribute(ARABIC_FORM);
/*     */       
/* 448 */       if (charStartsLigature(c) && i + 1 < end) {
/* 449 */         char nextChar = aci.setIndex(i + 1);
/* 450 */         Integer nextForm = (Integer)aci.getAttribute(ARABIC_FORM);
/* 451 */         if (form != null && nextForm != null) {
/* 452 */           if (form.equals(ARABIC_TERMINAL) && nextForm.equals(ARABIC_INITIAL)) {
/*     */ 
/*     */             
/* 455 */             int substChar = getSubstituteChar(c, nextChar, ARABIC_ISOLATED.intValue());
/*     */             
/* 457 */             if (substChar > -1) {
/* 458 */               substString.append((char)substChar);
/* 459 */               i++;
/*     */               continue;
/*     */             } 
/* 462 */           } else if (form.equals(ARABIC_TERMINAL)) {
/*     */             
/* 464 */             int substChar = getSubstituteChar(c, nextChar, ARABIC_TERMINAL.intValue());
/*     */             
/* 466 */             if (substChar > -1) {
/* 467 */               substString.append((char)substChar);
/* 468 */               i++;
/*     */               continue;
/*     */             } 
/* 471 */           } else if (form.equals(ARABIC_MEDIAL) && nextForm.equals(ARABIC_MEDIAL)) {
/*     */ 
/*     */             
/* 474 */             int substChar = getSubstituteChar(c, nextChar, ARABIC_MEDIAL.intValue());
/*     */             
/* 476 */             if (substChar > -1) {
/* 477 */               substString.append((char)substChar);
/* 478 */               i++;
/*     */ 
/*     */               
/*     */               continue;
/*     */             } 
/*     */           } 
/*     */         }
/*     */       } 
/*     */       
/* 487 */       if (form != null && form.intValue() > 0) {
/* 488 */         int substChar = getSubstituteChar(c, form.intValue());
/* 489 */         if (substChar > -1) {
/* 490 */           c = (char)substChar;
/*     */         }
/*     */       } 
/* 493 */       substString.append(c);
/*     */       continue;
/*     */     } 
/* 496 */     return substString.toString();
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static boolean charStartsLigature(char c) {
/* 507 */     char c1 = c;
/* 508 */     if (c1 == 'ً' || c1 == 'ٌ' || c1 == 'ٍ' || c1 == 'َ' || c1 == 'ُ' || c1 == 'ِ' || c1 == 'ّ' || c1 == 'ْ' || c1 == 'آ' || c1 == 'أ' || c1 == 'إ' || c1 == 'ا')
/*     */     {
/*     */ 
/*     */       
/* 512 */       return true;
/*     */     }
/* 514 */     return false;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static int getNumChars(char c) {
/* 527 */     if (isLigature(c))
/*     */     {
/* 529 */       return 2; } 
/* 530 */     return 1;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static boolean isLigature(char c) {
/* 541 */     char c1 = c;
/* 542 */     if (c1 < 'ﹰ' || c1 > 'ﻼ') {
/* 543 */       return false;
/*     */     }
/* 545 */     if (c1 <= 'ﹲ' || c1 == 'ﹴ' || (c1 >= 'ﹶ' && c1 <= 'ﹿ') || c1 >= 'ﻵ')
/*     */     {
/*     */ 
/*     */       
/* 549 */       return true;
/*     */     }
/* 551 */     return false;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 561 */   static int singleCharFirst = 1569;
/* 562 */   static int singleCharLast = 1610; static int[][] singleCharRemappings = { 
/* 563 */       { 65152, -1, -1, -1 }, { 65153, 65154, -1, -1 }, { 65155, 65156, -1, -1 }, { 65157, 65158, -1, -1 }, { 65159, 65160, -1, -1 }, { 65161, 65162, 65163, 65164 }, { 65165, 65166, -1, -1 }, { 65167, 65168, 65169, 65170 }, { 65171, 65172, -1, -1 }, { 65173, 65174, 65175, 65176 }, { 65177, 65178, 65179, 65180 }, { 65181, 65182, 65183, 65184 }, { 65185, 65186, 65187, 65188 }, { 65189, 65190, 65191, 65192 }, { 65193, 65194, -1, -1 }, { 65195, 65196, -1, -1 }, { 65197, 65198, -1, -1 }, { 65199, 65200, -1, -1 }, { 65201, 65202, 65203, 65204 }, { 65205, 65206, 65207, 65208 }, { 65209, 65210, 65211, 65212 }, { 65213, 65214, 65215, 65216 }, { 65217, 65218, 65219, 65220 }, { 65221, 65222, 65223, 65224 }, { 65225, 65226, 65227, 65228 }, { 65229, 65230, 65231, 65232 }, null, null, null, null, null, null, { 65233, 65234, 65235, 65236 }, { 65237, 65238, 65239, 65240 }, { 65241, 65242, 65243, 65244 }, { 65245, 65246, 65247, 65248 }, { 65249, 65250, 65251, 65252 }, { 65253, 65254, 65255, 65256 }, { 65257, 65258, 65259, 65260 }, { 65261, 65262, -1, -1 }, { 65263, 65264, -1, -1 }, { 65265, 65266, 65267, 65268 } };
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 610 */   static int doubleCharFirst = 1570;
/* 611 */   static int doubleCharLast = 1618; static int[][][] doubleCharRemappings = { 
/* 612 */       { { 1604, 65269, 65270, -1, -1 } }, { { 1604, 65271, 65272, -1, -1 } }, null, { { 1604, 65273, 65274, -1, -1 } }, null, { { 1604, 65275, 65276, -1, -1 } }, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, { { 32, 65136, -1, -1, -1 }, { 1600, -1, -1, -1, 65137 } }, { { 32, 65138, -1, -1, -1 } }, { { 32, 65140, -1, -1, -1 } }, { { 32, 65142, -1, -1, -1 }, { 1600, -1, -1, -1, 65143 } }, { { 32, 65144, -1, -1, -1 }, { 1600, -1, -1, -1, 65145 } }, { { 32, 65146, -1, -1, -1 }, { 1600, -1, -1, -1, 65147 } }, { { 32, 65148, -1, -1, -1 }, { 1600, -1, -1, -1, 65149 } }, { { 32, 65150, -1, -1, -1 }, { 1600, -1, -1, -1, 65151 } } };
/*     */ }


/* Location:              C:\Program Files (x86)\IBM\WebSphere\AppServer\profiles\ctgAppSrv01\installedApps\ctgCell01\MAXIMO.ear\maximouiweb.war\WEB-INF\classes\!\com\ibm\tivoli\maximo\report\control\svgtools\batik-gvt.jar!\org\apache\batik\gvt\text\ArabicTextHandler.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       1.0.7
 */