/*     */ package org.apache.batik.gvt.text;
/*     */ 
/*     */ import java.awt.font.FontRenderContext;
/*     */ import java.awt.font.TextLayout;
/*     */ import java.text.AttributedCharacterIterator;
/*     */ import java.text.AttributedString;
/*     */ import java.util.HashMap;
/*     */ import java.util.Iterator;
/*     */ import java.util.Map;
/*     */ import java.util.Set;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class BidiAttributedCharacterIterator
/*     */   implements AttributedCharacterIterator
/*     */ {
/*     */   private AttributedCharacterIterator reorderedACI;
/*     */   private FontRenderContext frc;
/*     */   private int chunkStart;
/*     */   private int[] newCharOrder;
/*  46 */   private static final Float FLOAT_NAN = new Float(NaNF);
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected BidiAttributedCharacterIterator(AttributedCharacterIterator reorderedACI, FontRenderContext frc, int chunkStart, int[] newCharOrder) {
/*  54 */     this.reorderedACI = reorderedACI;
/*  55 */     this.frc = frc;
/*  56 */     this.chunkStart = chunkStart;
/*  57 */     this.newCharOrder = newCharOrder;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public BidiAttributedCharacterIterator(AttributedCharacterIterator aci, FontRenderContext frc, int chunkStart) {
/*  73 */     this.frc = frc;
/*  74 */     this.chunkStart = chunkStart;
/*  75 */     aci.first();
/*  76 */     int numChars = aci.getEndIndex() - aci.getBeginIndex();
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/*  87 */     StringBuffer strB = new StringBuffer(numChars);
/*  88 */     char c = aci.first();
/*  89 */     for (int i = 0; i < numChars; i++) {
/*  90 */       strB.append(c);
/*  91 */       c = aci.next();
/*     */     } 
/*  93 */     AttributedString as = new AttributedString(strB.toString());
/*  94 */     int start = aci.getBeginIndex();
/*  95 */     int end = aci.getEndIndex();
/*  96 */     int index = start;
/*  97 */     while (index < end) {
/*  98 */       aci.setIndex(index);
/*  99 */       Map attrMap = aci.getAttributes();
/* 100 */       int extent = aci.getRunLimit();
/* 101 */       Map destMap = new HashMap(attrMap.size());
/* 102 */       Iterator it = attrMap.entrySet().iterator();
/* 103 */       while (it.hasNext()) {
/*     */ 
/*     */         
/* 106 */         Map.Entry e = (Map.Entry)it.next();
/* 107 */         Object key = e.getKey();
/* 108 */         if (key == null)
/* 109 */           continue;  Object value = e.getValue();
/* 110 */         if (value == null)
/* 111 */           continue;  destMap.put(key, value);
/*     */       } 
/*     */ 
/*     */       
/* 115 */       as.addAttributes(destMap, index - start, extent - start);
/* 116 */       index = extent;
/*     */     } 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 122 */     TextLayout tl = new TextLayout(as.getIterator(), frc);
/*     */     
/* 124 */     int[] charIndices = new int[numChars];
/* 125 */     int[] charLevels = new int[numChars];
/*     */     
/* 127 */     int runStart = 0;
/* 128 */     int currBiDi = tl.getCharacterLevel(0);
/* 129 */     charIndices[0] = 0;
/* 130 */     charLevels[0] = currBiDi;
/* 131 */     int maxBiDi = currBiDi;
/*     */     
/* 133 */     for (int i = 1; i < numChars; i++) {
/* 134 */       int newBiDi = tl.getCharacterLevel(i);
/* 135 */       charIndices[i] = i;
/* 136 */       charLevels[i] = newBiDi;
/*     */       
/* 138 */       if (newBiDi != currBiDi) {
/* 139 */         as.addAttribute(GVTAttributedCharacterIterator.TextAttribute.BIDI_LEVEL, new Integer(currBiDi), runStart, i);
/*     */ 
/*     */         
/* 142 */         runStart = i;
/* 143 */         currBiDi = newBiDi;
/* 144 */         if (newBiDi > maxBiDi) maxBiDi = newBiDi; 
/*     */       } 
/*     */     } 
/* 147 */     as.addAttribute(GVTAttributedCharacterIterator.TextAttribute.BIDI_LEVEL, new Integer(currBiDi), runStart, numChars);
/*     */ 
/*     */ 
/*     */     
/* 151 */     aci = as.getIterator();
/*     */     
/* 153 */     if (runStart == 0 && currBiDi == 0) {
/*     */ 
/*     */ 
/*     */       
/* 157 */       this.reorderedACI = aci;
/* 158 */       this.newCharOrder = new int[numChars];
/* 159 */       for (int i = 0; i < numChars; i++) {
/* 160 */         this.newCharOrder[i] = chunkStart + i;
/*     */       }
/*     */       
/*     */       return;
/*     */     } 
/* 165 */     this.newCharOrder = doBidiReorder(charIndices, charLevels, numChars, maxBiDi);
/*     */ 
/*     */ 
/*     */     
/* 169 */     StringBuffer reorderedString = new StringBuffer(numChars);
/* 170 */     int reorderedFirstChar = 0;
/* 171 */     for (int i = 0; i < numChars; i++) {
/* 172 */       int srcIdx = this.newCharOrder[i];
/* 173 */       char c = aci.setIndex(srcIdx);
/* 174 */       if (srcIdx == 0) reorderedFirstChar = i;
/*     */ 
/*     */       
/* 177 */       int bidiLevel = tl.getCharacterLevel(srcIdx);
/* 178 */       if ((bidiLevel & true) != 0)
/*     */       {
/*     */ 
/*     */         
/* 182 */         c = (char)mirrorChar(c);
/*     */       }
/*     */       
/* 185 */       reorderedString.append(c);
/*     */     } 
/*     */ 
/*     */     
/* 189 */     AttributedString reorderedAS = new AttributedString(reorderedString.toString());
/*     */     
/* 191 */     Map[] attrs = new Map[numChars];
/* 192 */     int start = aci.getBeginIndex();
/* 193 */     int end = aci.getEndIndex();
/* 194 */     int index = start;
/* 195 */     while (index < end) {
/* 196 */       aci.setIndex(index);
/* 197 */       Map attrMap = aci.getAttributes();
/* 198 */       int extent = aci.getRunLimit();
/* 199 */       for (int i = index; i < extent; i++)
/* 200 */         attrs[i - start] = attrMap; 
/* 201 */       index = extent;
/*     */     } 
/*     */     
/* 204 */     runStart = 0;
/* 205 */     Map prevAttrMap = attrs[this.newCharOrder[0]];
/* 206 */     for (int i = 1; i < numChars; i++) {
/* 207 */       Map attrMap = attrs[this.newCharOrder[i]];
/* 208 */       if (attrMap != prevAttrMap) {
/*     */         
/* 210 */         reorderedAS.addAttributes(prevAttrMap, runStart, i);
/* 211 */         prevAttrMap = attrMap;
/* 212 */         runStart = i;
/*     */       } 
/*     */     } 
/* 215 */     reorderedAS.addAttributes(prevAttrMap, runStart, numChars);
/*     */ 
/*     */     
/* 218 */     aci.first();
/* 219 */     Float x = (Float)aci.getAttribute(GVTAttributedCharacterIterator.TextAttribute.X);
/*     */     
/* 221 */     if (x != null && !x.isNaN()) {
/* 222 */       reorderedAS.addAttribute(GVTAttributedCharacterIterator.TextAttribute.X, FLOAT_NAN, reorderedFirstChar, reorderedFirstChar + 1);
/*     */ 
/*     */       
/* 225 */       reorderedAS.addAttribute(GVTAttributedCharacterIterator.TextAttribute.X, x, 0, 1);
/*     */     } 
/*     */ 
/*     */     
/* 229 */     Float y = (Float)aci.getAttribute(GVTAttributedCharacterIterator.TextAttribute.Y);
/*     */     
/* 231 */     if (y != null && !y.isNaN()) {
/* 232 */       reorderedAS.addAttribute(GVTAttributedCharacterIterator.TextAttribute.Y, FLOAT_NAN, reorderedFirstChar, reorderedFirstChar + 1);
/*     */ 
/*     */       
/* 235 */       reorderedAS.addAttribute(GVTAttributedCharacterIterator.TextAttribute.Y, y, 0, 1);
/*     */     } 
/*     */ 
/*     */ 
/*     */     
/* 240 */     Float dx = (Float)aci.getAttribute(GVTAttributedCharacterIterator.TextAttribute.DX);
/*     */     
/* 242 */     if (dx != null && !dx.isNaN()) {
/* 243 */       reorderedAS.addAttribute(GVTAttributedCharacterIterator.TextAttribute.DX, FLOAT_NAN, reorderedFirstChar, reorderedFirstChar + 1);
/*     */ 
/*     */       
/* 246 */       reorderedAS.addAttribute(GVTAttributedCharacterIterator.TextAttribute.DX, dx, 0, 1);
/*     */     } 
/*     */     
/* 249 */     Float dy = (Float)aci.getAttribute(GVTAttributedCharacterIterator.TextAttribute.DY);
/*     */     
/* 251 */     if (dy != null && !dy.isNaN()) {
/* 252 */       reorderedAS.addAttribute(GVTAttributedCharacterIterator.TextAttribute.DY, FLOAT_NAN, reorderedFirstChar, reorderedFirstChar + 1);
/*     */ 
/*     */       
/* 255 */       reorderedAS.addAttribute(GVTAttributedCharacterIterator.TextAttribute.DY, dy, 0, 1);
/*     */     } 
/*     */ 
/*     */ 
/*     */     
/* 260 */     reorderedAS = ArabicTextHandler.assignArabicForms(reorderedAS);
/*     */ 
/*     */     
/* 263 */     for (int i = 0; i < this.newCharOrder.length; i++) {
/* 264 */       this.newCharOrder[i] = this.newCharOrder[i] + chunkStart;
/*     */     }
/* 266 */     this.reorderedACI = reorderedAS.getIterator();
/*     */   }
/*     */ 
/*     */ 
/*     */   
/* 271 */   public int[] getCharMap() { return this.newCharOrder; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   private int[] doBidiReorder(int[] charIndices, int[] charLevels, int numChars, int highestLevel) {
/* 285 */     if (highestLevel == 0) return charIndices;
/*     */ 
/*     */ 
/*     */     
/* 289 */     int currentIndex = 0;
/* 290 */     while (currentIndex < numChars) {
/*     */ 
/*     */       
/* 293 */       while (currentIndex < numChars && charLevels[currentIndex] < highestLevel)
/*     */       {
/* 295 */         currentIndex++;
/*     */       }
/* 297 */       if (currentIndex == numChars) {
/*     */         break;
/*     */       }
/*     */       
/* 301 */       int startIndex = currentIndex;
/*     */       
/* 303 */       currentIndex++;
/*     */       
/* 305 */       while (currentIndex < numChars && charLevels[currentIndex] == highestLevel)
/*     */       {
/* 307 */         currentIndex++;
/*     */       }
/* 309 */       int endIndex = currentIndex - 1;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */       
/* 316 */       int middle = (endIndex - startIndex >> 1) + 1;
/* 317 */       for (int i = 0; i < middle; i++) {
/* 318 */         int tmp = charIndices[startIndex + i];
/* 319 */         charIndices[startIndex + i] = charIndices[endIndex - i];
/* 320 */         charIndices[endIndex - i] = tmp;
/*     */         
/* 322 */         charLevels[startIndex + i] = highestLevel - 1;
/* 323 */         charLevels[endIndex - i] = highestLevel - 1;
/*     */       } 
/*     */     } 
/* 326 */     return doBidiReorder(charIndices, charLevels, numChars, highestLevel - 1);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 335 */   public Set getAllAttributeKeys() { return this.reorderedACI.getAllAttributeKeys(); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 343 */   public Object getAttribute(AttributedCharacterIterator.Attribute attribute) { return this.reorderedACI.getAttribute(attribute); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 351 */   public Map getAttributes() { return this.reorderedACI.getAttributes(); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 360 */   public int getRunLimit() { return this.reorderedACI.getRunLimit(); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 369 */   public int getRunLimit(AttributedCharacterIterator.Attribute attribute) { return this.reorderedACI.getRunLimit(attribute); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 378 */   public int getRunLimit(Set attributes) { return this.reorderedACI.getRunLimit(attributes); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 386 */   public int getRunStart() { return this.reorderedACI.getRunStart(); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 396 */   public int getRunStart(AttributedCharacterIterator.Attribute attribute) { return this.reorderedACI.getRunStart(attribute); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 405 */   public int getRunStart(Set attributes) { return this.reorderedACI.getRunStart(attributes); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 412 */   public Object clone() { return new BidiAttributedCharacterIterator((AttributedCharacterIterator)this.reorderedACI.clone(), this.frc, this.chunkStart, (int[])this.newCharOrder.clone()); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 421 */   public char current() { return this.reorderedACI.current(); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 429 */   public char first() { return this.reorderedACI.first(); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 436 */   public int getBeginIndex() { return this.reorderedACI.getBeginIndex(); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 443 */   public int getEndIndex() { return this.reorderedACI.getEndIndex(); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 450 */   public int getIndex() { return this.reorderedACI.getIndex(); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 458 */   public char last() { return this.reorderedACI.last(); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 466 */   public char next() { return this.reorderedACI.next(); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 473 */   public char previous() { return this.reorderedACI.previous(); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 480 */   public char setIndex(int position) { return this.reorderedACI.setIndex(position); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static int mirrorChar(int c) {
/* 493 */     switch (c) {
/*     */       case 40:
/* 495 */         return 41;
/* 496 */       case 41: return 40;
/* 497 */       case 60: return 62;
/* 498 */       case 62: return 60;
/* 499 */       case 91: return 93;
/* 500 */       case 93: return 91;
/* 501 */       case 123: return 125;
/* 502 */       case 125: return 123;
/* 503 */       case 171: return 187;
/* 504 */       case 187: return 171;
/* 505 */       case 8249: return 8250;
/* 506 */       case 8250: return 8249;
/* 507 */       case 8261: return 8262;
/* 508 */       case 8262: return 8261;
/* 509 */       case 8317: return 8318;
/* 510 */       case 8318: return 8317;
/* 511 */       case 8333: return 8334;
/* 512 */       case 8334: return 8333;
/* 513 */       case 8712: return 8715;
/* 514 */       case 8713: return 8716;
/* 515 */       case 8714: return 8717;
/* 516 */       case 8715: return 8712;
/* 517 */       case 8716: return 8713;
/* 518 */       case 8717: return 8714;
/* 519 */       case 8764: return 8765;
/* 520 */       case 8765: return 8764;
/* 521 */       case 8771: return 8909;
/* 522 */       case 8786: return 8787;
/* 523 */       case 8787: return 8786;
/* 524 */       case 8788: return 8789;
/* 525 */       case 8789: return 8788;
/* 526 */       case 8804: return 8805;
/* 527 */       case 8805: return 8804;
/* 528 */       case 8806: return 8807;
/* 529 */       case 8807: return 8806;
/* 530 */       case 8808: return 8809;
/* 531 */       case 8809: return 8808;
/* 532 */       case 8810: return 8811;
/* 533 */       case 8811: return 8810;
/* 534 */       case 8814: return 8815;
/* 535 */       case 8815: return 8814;
/* 536 */       case 8816: return 8817;
/* 537 */       case 8817: return 8816;
/* 538 */       case 8818: return 8819;
/* 539 */       case 8819: return 8818;
/* 540 */       case 8820: return 8821;
/* 541 */       case 8821: return 8820;
/* 542 */       case 8822: return 8823;
/* 543 */       case 8823: return 8822;
/* 544 */       case 8824: return 8825;
/* 545 */       case 8825: return 8824;
/* 546 */       case 8826: return 8827;
/* 547 */       case 8827: return 8826;
/* 548 */       case 8828: return 8829;
/* 549 */       case 8829: return 8828;
/* 550 */       case 8830: return 8831;
/* 551 */       case 8831: return 8830;
/* 552 */       case 8832: return 8833;
/* 553 */       case 8833: return 8832;
/* 554 */       case 8834: return 8835;
/* 555 */       case 8835: return 8834;
/* 556 */       case 8836: return 8837;
/* 557 */       case 8837: return 8836;
/* 558 */       case 8838: return 8839;
/* 559 */       case 8839: return 8838;
/* 560 */       case 8840: return 8841;
/* 561 */       case 8841: return 8840;
/* 562 */       case 8842: return 8843;
/* 563 */       case 8843: return 8842;
/* 564 */       case 8847: return 8848;
/* 565 */       case 8848: return 8847;
/* 566 */       case 8849: return 8850;
/* 567 */       case 8850: return 8849;
/* 568 */       case 8866: return 8867;
/* 569 */       case 8867: return 8866;
/* 570 */       case 8880: return 8881;
/* 571 */       case 8881: return 8880;
/* 572 */       case 8882: return 8883;
/* 573 */       case 8883: return 8882;
/* 574 */       case 8884: return 8885;
/* 575 */       case 8885: return 8884;
/* 576 */       case 8886: return 8887;
/* 577 */       case 8887: return 8886;
/* 578 */       case 8905: return 8906;
/* 579 */       case 8906: return 8905;
/* 580 */       case 8907: return 8908;
/* 581 */       case 8908: return 8907;
/* 582 */       case 8909: return 8771;
/* 583 */       case 8912: return 8913;
/* 584 */       case 8913: return 8912;
/* 585 */       case 8918: return 8919;
/* 586 */       case 8919: return 8918;
/* 587 */       case 8920: return 8921;
/* 588 */       case 8921: return 8920;
/* 589 */       case 8922: return 8923;
/* 590 */       case 8923: return 8922;
/* 591 */       case 8924: return 8925;
/* 592 */       case 8925: return 8924;
/* 593 */       case 8926: return 8927;
/* 594 */       case 8927: return 8926;
/* 595 */       case 8928: return 8929;
/* 596 */       case 8929: return 8928;
/* 597 */       case 8930: return 8931;
/* 598 */       case 8931: return 8930;
/* 599 */       case 8932: return 8933;
/* 600 */       case 8933: return 8932;
/* 601 */       case 8934: return 8935;
/* 602 */       case 8935: return 8934;
/* 603 */       case 8936: return 8937;
/* 604 */       case 8937: return 8936;
/* 605 */       case 8938: return 8939;
/* 606 */       case 8939: return 8938;
/* 607 */       case 8940: return 8941;
/* 608 */       case 8941: return 8940;
/* 609 */       case 8944: return 8945;
/* 610 */       case 8945: return 8944;
/* 611 */       case 8968: return 8969;
/* 612 */       case 8969: return 8968;
/* 613 */       case 8970: return 8971;
/* 614 */       case 8971: return 8970;
/* 615 */       case 9001: return 9002;
/* 616 */       case 9002: return 9001;
/* 617 */       case 12296: return 12297;
/* 618 */       case 12297: return 12296;
/* 619 */       case 12298: return 12299;
/* 620 */       case 12299: return 12298;
/* 621 */       case 12300: return 12301;
/* 622 */       case 12301: return 12300;
/* 623 */       case 12302: return 12303;
/* 624 */       case 12303: return 12302;
/* 625 */       case 12304: return 12305;
/* 626 */       case 12305: return 12304;
/* 627 */       case 12308: return 12309;
/* 628 */       case 12309: return 12308;
/* 629 */       case 12310: return 12311;
/* 630 */       case 12311: return 12310;
/* 631 */       case 12312: return 12313;
/* 632 */       case 12313: return 12312;
/* 633 */       case 12314: return 12315;
/* 634 */       case 12315: return 12314;
/*     */     } 
/*     */     
/* 637 */     return c;
/*     */   }
/*     */ }


/* Location:              C:\Program Files (x86)\IBM\WebSphere\AppServer\profiles\ctgAppSrv01\installedApps\ctgCell01\MAXIMO.ear\maximouiweb.war\WEB-INF\classes\!\com\ibm\tivoli\maximo\report\control\svgtools\batik-gvt.jar!\org\apache\batik\gvt\text\BidiAttributedCharacterIterator.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       1.0.7
 */