/*     */ package org.apache.batik.gvt.text;
/*     */ 
/*     */ import java.awt.Shape;
/*     */ import java.awt.geom.AffineTransform;
/*     */ import java.awt.geom.NoninvertibleTransformException;
/*     */ import java.awt.geom.Point2D;
/*     */ import java.text.CharacterIterator;
/*     */ import java.util.ArrayList;
/*     */ import java.util.Iterator;
/*     */ import org.apache.batik.gvt.GraphicsNode;
/*     */ import org.apache.batik.gvt.RootGraphicsNode;
/*     */ import org.apache.batik.gvt.Selectable;
/*     */ import org.apache.batik.gvt.Selector;
/*     */ import org.apache.batik.gvt.TextNode;
/*     */ import org.apache.batik.gvt.event.GraphicsNodeChangeEvent;
/*     */ import org.apache.batik.gvt.event.GraphicsNodeEvent;
/*     */ import org.apache.batik.gvt.event.GraphicsNodeKeyEvent;
/*     */ import org.apache.batik.gvt.event.GraphicsNodeMouseEvent;
/*     */ import org.apache.batik.gvt.event.SelectionEvent;
/*     */ import org.apache.batik.gvt.event.SelectionListener;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class ConcreteTextSelector
/*     */   implements Selector
/*     */ {
/*     */   private ArrayList listeners;
/*     */   private GraphicsNode selectionNode;
/*     */   private RootGraphicsNode selectionNodeRoot;
/*     */   
/*  57 */   public void mouseClicked(GraphicsNodeMouseEvent evt) { checkSelectGesture(evt); }
/*     */ 
/*     */ 
/*     */   
/*  61 */   public void mouseDragged(GraphicsNodeMouseEvent evt) { checkSelectGesture(evt); }
/*     */ 
/*     */ 
/*     */   
/*  65 */   public void mouseEntered(GraphicsNodeMouseEvent evt) { checkSelectGesture(evt); }
/*     */ 
/*     */ 
/*     */   
/*  69 */   public void mouseExited(GraphicsNodeMouseEvent evt) { checkSelectGesture(evt); }
/*     */ 
/*     */ 
/*     */   
/*     */   public void mouseMoved(GraphicsNodeMouseEvent evt) {}
/*     */ 
/*     */   
/*  76 */   public void mousePressed(GraphicsNodeMouseEvent evt) { checkSelectGesture(evt); }
/*     */ 
/*     */ 
/*     */   
/*  80 */   public void mouseReleased(GraphicsNodeMouseEvent evt) { checkSelectGesture(evt); }
/*     */ 
/*     */ 
/*     */   
/*  84 */   public void keyPressed(GraphicsNodeKeyEvent evt) { report(evt, "keyPressed"); }
/*     */ 
/*     */ 
/*     */   
/*  88 */   public void keyReleased(GraphicsNodeKeyEvent evt) { report(evt, "keyReleased"); }
/*     */ 
/*     */ 
/*     */   
/*  92 */   public void keyTyped(GraphicsNodeKeyEvent evt) { report(evt, "keyTyped"); }
/*     */ 
/*     */   
/*     */   public void changeStarted(GraphicsNodeChangeEvent gnce) {}
/*     */ 
/*     */   
/*     */   public void changeCompleted(GraphicsNodeChangeEvent gnce) {
/*  99 */     if (this.selectionNode == null) {
/*     */       return;
/*     */     }
/* 102 */     Shape newShape = ((Selectable)this.selectionNode).getHighlightShape();
/*     */     
/* 104 */     dispatchSelectionEvent(new SelectionEvent(getSelection(), 1, newShape));
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void setSelection(Mark begin, Mark end) {
/* 111 */     TextNode node = begin.getTextNode();
/* 112 */     if (node != end.getTextNode()) {
/* 113 */       throw new Error("Markers not from same TextNode");
/*     */     }
/* 115 */     node.setSelection(begin, end);
/* 116 */     this.selectionNode = node;
/* 117 */     this.selectionNodeRoot = node.getRoot();
/* 118 */     Object selection = getSelection();
/* 119 */     Shape shape = node.getHighlightShape();
/* 120 */     dispatchSelectionEvent(new SelectionEvent(selection, 2, shape));
/*     */   }
/*     */ 
/*     */   
/*     */   public void clearSelection() {
/* 125 */     if (this.selectionNode == null) {
/*     */       return;
/*     */     }
/* 128 */     dispatchSelectionEvent(new SelectionEvent(null, 3, null));
/*     */     
/* 130 */     this.selectionNode = null;
/* 131 */     this.selectionNodeRoot = null;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected void checkSelectGesture(GraphicsNodeEvent evt) {
/* 143 */     GraphicsNodeMouseEvent mevt = null;
/* 144 */     if (evt instanceof GraphicsNodeMouseEvent) {
/* 145 */       mevt = (GraphicsNodeMouseEvent)evt;
/*     */     }
/*     */     
/* 148 */     GraphicsNode source = evt.getGraphicsNode();
/* 149 */     if (isDeselectGesture(evt)) {
/* 150 */       if (this.selectionNode != null) {
/* 151 */         this.selectionNodeRoot.removeTreeGraphicsNodeChangeListener(this);
/*     */       }
/* 153 */       clearSelection();
/* 154 */     } else if (mevt != null) {
/*     */       
/* 156 */       Point2D p = new Point2D.Double(mevt.getX(), mevt.getY());
/* 157 */       AffineTransform t = source.getGlobalTransform();
/* 158 */       if (t == null) {
/* 159 */         t = new AffineTransform();
/*     */       } else {
/*     */         
/*     */         try {
/* 163 */           t = t.createInverse();
/* 164 */         } catch (NoninvertibleTransformException ni) {}
/*     */       } 
/*     */       
/* 167 */       p = t.transform(p, null);
/*     */       
/* 169 */       if (source instanceof Selectable && isSelectStartGesture(evt)) {
/*     */         
/* 171 */         if (this.selectionNode != source) {
/* 172 */           if (this.selectionNode != null) {
/* 173 */             this.selectionNodeRoot.removeTreeGraphicsNodeChangeListener(this);
/*     */           }
/*     */           
/* 176 */           this.selectionNode = source;
/* 177 */           if (source != null) {
/* 178 */             this.selectionNodeRoot = source.getRoot();
/* 179 */             this.selectionNodeRoot.addTreeGraphicsNodeChangeListener(this);
/*     */           } 
/*     */         } 
/*     */ 
/*     */         
/* 184 */         ((Selectable)source).selectAt(p.getX(), p.getY());
/* 185 */         dispatchSelectionEvent(new SelectionEvent(null, 4, null));
/*     */ 
/*     */ 
/*     */       
/*     */       }
/* 190 */       else if (isSelectEndGesture(evt)) {
/* 191 */         if (this.selectionNode == source) {
/* 192 */           ((Selectable)source).selectTo(p.getX(), p.getY());
/*     */         }
/* 194 */         Object oldSelection = getSelection();
/* 195 */         if (this.selectionNode != null)
/*     */         {
/* 197 */           Shape newShape = ((Selectable)this.selectionNode).getHighlightShape();
/* 198 */           dispatchSelectionEvent(new SelectionEvent(oldSelection, 2, newShape));
/*     */         
/*     */         }
/*     */       
/*     */       }
/* 203 */       else if (isSelectContinueGesture(evt)) {
/*     */         
/* 205 */         if (this.selectionNode == source) {
/* 206 */           boolean result = ((Selectable)source).selectTo(p.getX(), p.getY());
/*     */           
/* 208 */           if (result) {
/* 209 */             Shape newShape = ((Selectable)this.selectionNode).getHighlightShape();
/*     */ 
/*     */             
/* 212 */             dispatchSelectionEvent(new SelectionEvent(null, 1, newShape));
/*     */           }
/*     */         
/*     */         }
/*     */       
/*     */       }
/* 218 */       else if (source instanceof Selectable && isSelectAllGesture(evt)) {
/*     */         
/* 220 */         if (this.selectionNode != source) {
/* 221 */           if (this.selectionNode != null) {
/* 222 */             this.selectionNodeRoot.removeTreeGraphicsNodeChangeListener(this);
/*     */           }
/*     */           
/* 225 */           this.selectionNode = source;
/* 226 */           if (source != null) {
/* 227 */             this.selectionNodeRoot = source.getRoot();
/* 228 */             this.selectionNodeRoot.addTreeGraphicsNodeChangeListener(this);
/*     */           } 
/*     */         } 
/*     */         
/* 232 */         ((Selectable)source).selectAll(p.getX(), p.getY());
/* 233 */         Object oldSelection = getSelection();
/* 234 */         Shape newShape = ((Selectable)source).getHighlightShape();
/*     */         
/* 236 */         dispatchSelectionEvent(new SelectionEvent(oldSelection, 2, newShape));
/*     */       } 
/*     */     } 
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 245 */   private boolean isDeselectGesture(GraphicsNodeEvent evt) { return (evt.getID() == 500 && ((GraphicsNodeMouseEvent)evt).getClickCount() == 1); }
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 250 */   private boolean isSelectStartGesture(GraphicsNodeEvent evt) { return (evt.getID() == 501); }
/*     */ 
/*     */ 
/*     */   
/* 254 */   private boolean isSelectEndGesture(GraphicsNodeEvent evt) { return (evt.getID() == 502); }
/*     */ 
/*     */ 
/*     */   
/* 258 */   private boolean isSelectContinueGesture(GraphicsNodeEvent evt) { return (evt.getID() == 506); }
/*     */ 
/*     */ 
/*     */   
/* 262 */   private boolean isSelectAllGesture(GraphicsNodeEvent evt) { return (evt.getID() == 500 && ((GraphicsNodeMouseEvent)evt).getClickCount() == 2); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public Object getSelection() {
/* 270 */     Object value = null;
/* 271 */     if (this.selectionNode instanceof Selectable) {
/* 272 */       value = ((Selectable)this.selectionNode).getSelection();
/*     */     }
/* 274 */     return value;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 281 */   public boolean isEmpty() { return (getSelection() == null); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void dispatchSelectionEvent(SelectionEvent e) {
/* 288 */     if (this.listeners != null) {
/* 289 */       Iterator iter = this.listeners.iterator();
/* 290 */       switch (e.getID()) {
/*     */         case 2:
/* 292 */           while (iter.hasNext()) {
/* 293 */             ((SelectionListener)iter.next()).selectionDone(e);
/*     */           }
/*     */           break;
/*     */         case 1:
/* 297 */           while (iter.hasNext()) {
/* 298 */             ((SelectionListener)iter.next()).selectionChanged(e);
/*     */           }
/*     */           break;
/*     */         case 3:
/* 302 */           while (iter.hasNext()) {
/* 303 */             ((SelectionListener)iter.next()).selectionCleared(e);
/*     */           }
/*     */           break;
/*     */         case 4:
/* 307 */           while (iter.hasNext()) {
/* 308 */             ((SelectionListener)iter.next()).selectionStarted(e);
/*     */           }
/*     */           break;
/*     */       } 
/*     */     } 
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void addSelectionListener(SelectionListener l) {
/* 320 */     if (this.listeners == null) {
/* 321 */       this.listeners = new ArrayList();
/*     */     }
/* 323 */     this.listeners.add(l);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void removeSelectionListener(SelectionListener l) {
/* 331 */     if (this.listeners != null) {
/* 332 */       this.listeners.remove(l);
/*     */     }
/*     */   }
/*     */   
/*     */   private void report(GraphicsNodeEvent evt, String message) {
/* 337 */     GraphicsNode source = evt.getGraphicsNode();
/* 338 */     String label = "(non-text node)";
/* 339 */     if (source instanceof TextNode) {
/*     */       
/* 341 */       CharacterIterator iter = ((TextNode)source).getAttributedCharacterIterator();
/*     */       
/* 343 */       char[] cbuff = new char[iter.getEndIndex()];
/* 344 */       if (cbuff.length > 0) cbuff[0] = iter.first(); 
/* 345 */       for (int i = 1; i < cbuff.length; i++) {
/* 346 */         cbuff[i] = iter.next();
/*     */       }
/* 348 */       label = new String(cbuff);
/*     */     } 
/* 350 */     System.out.println("Mouse " + message + " in " + label);
/*     */   }
/*     */ }


/* Location:              C:\Program Files (x86)\IBM\WebSphere\AppServer\profiles\ctgAppSrv01\installedApps\ctgCell01\MAXIMO.ear\maximouiweb.war\WEB-INF\classes\!\com\ibm\tivoli\maximo\report\control\svgtools\batik-gvt.jar!\org\apache\batik\gvt\text\ConcreteTextSelector.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       1.0.7
 */