/*      */ package org.apache.batik.gvt.text;
/*      */ 
/*      */ import java.awt.BasicStroke;
/*      */ import java.awt.Graphics2D;
/*      */ import java.awt.Shape;
/*      */ import java.awt.Stroke;
/*      */ import java.awt.font.FontRenderContext;
/*      */ import java.awt.font.TextAttribute;
/*      */ import java.awt.geom.AffineTransform;
/*      */ import java.awt.geom.Area;
/*      */ import java.awt.geom.GeneralPath;
/*      */ import java.awt.geom.Line2D;
/*      */ import java.awt.geom.PathIterator;
/*      */ import java.awt.geom.Point2D;
/*      */ import java.awt.geom.Point2D.Float;
/*      */ import java.awt.geom.Rectangle2D;
/*      */ import java.text.AttributedCharacterIterator;
/*      */ import java.util.HashSet;
/*      */ import java.util.Set;
/*      */ import org.apache.batik.gvt.font.AWTGVTFont;
/*      */ import org.apache.batik.gvt.font.AltGlyphHandler;
/*      */ import org.apache.batik.gvt.font.GVTFont;
/*      */ import org.apache.batik.gvt.font.GVTGlyphMetrics;
/*      */ import org.apache.batik.gvt.font.GVTGlyphVector;
/*      */ import org.apache.batik.gvt.font.GVTLineMetrics;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ public class GlyphLayout
/*      */   implements TextSpanLayout
/*      */ {
/*      */   private GVTGlyphVector gv;
/*      */   private GVTFont font;
/*      */   private GVTLineMetrics metrics;
/*      */   private AttributedCharacterIterator aci;
/*      */   private Point2D advance;
/*      */   private Point2D offset;
/*      */   private float xScale;
/*      */   private float yScale;
/*      */   private TextPath textPath;
/*      */   private Point2D textPathAdvance;
/*      */   private int[] charMap;
/*      */   private boolean vertical;
/*      */   private boolean adjSpacing;
/*      */   private float[] glyphAdvances;
/*      */   private boolean isAltGlyph;
/*      */   private boolean layoutApplied;
/*      */   private boolean spacingApplied;
/*      */   private boolean pathApplied;
/*   89 */   public static final AttributedCharacterIterator.Attribute FLOW_LINE_BREAK = GVTAttributedCharacterIterator.TextAttribute.FLOW_LINE_BREAK;
/*      */ 
/*      */   
/*   92 */   public static final AttributedCharacterIterator.Attribute FLOW_PARAGRAPH = GVTAttributedCharacterIterator.TextAttribute.FLOW_PARAGRAPH;
/*      */ 
/*      */ 
/*      */   
/*   96 */   public static final AttributedCharacterIterator.Attribute FLOW_EMPTY_PARAGRAPH = GVTAttributedCharacterIterator.TextAttribute.FLOW_EMPTY_PARAGRAPH;
/*      */ 
/*      */   
/*   99 */   public static final AttributedCharacterIterator.Attribute LINE_HEIGHT = GVTAttributedCharacterIterator.TextAttribute.LINE_HEIGHT;
/*      */ 
/*      */ 
/*      */   
/*  103 */   public static final AttributedCharacterIterator.Attribute VERTICAL_ORIENTATION = GVTAttributedCharacterIterator.TextAttribute.VERTICAL_ORIENTATION;
/*      */ 
/*      */ 
/*      */   
/*  107 */   public static final AttributedCharacterIterator.Attribute VERTICAL_ORIENTATION_ANGLE = GVTAttributedCharacterIterator.TextAttribute.VERTICAL_ORIENTATION_ANGLE;
/*      */ 
/*      */ 
/*      */   
/*  111 */   public static final AttributedCharacterIterator.Attribute HORIZONTAL_ORIENTATION_ANGLE = GVTAttributedCharacterIterator.TextAttribute.HORIZONTAL_ORIENTATION_ANGLE;
/*      */ 
/*      */   
/*  114 */   private static final AttributedCharacterIterator.Attribute X = GVTAttributedCharacterIterator.TextAttribute.X;
/*      */ 
/*      */   
/*  117 */   private static final AttributedCharacterIterator.Attribute Y = GVTAttributedCharacterIterator.TextAttribute.Y;
/*      */ 
/*      */   
/*  120 */   private static final AttributedCharacterIterator.Attribute DX = GVTAttributedCharacterIterator.TextAttribute.DX;
/*      */ 
/*      */   
/*  123 */   private static final AttributedCharacterIterator.Attribute DY = GVTAttributedCharacterIterator.TextAttribute.DY;
/*      */ 
/*      */   
/*  126 */   private static final AttributedCharacterIterator.Attribute ROTATION = GVTAttributedCharacterIterator.TextAttribute.ROTATION;
/*      */ 
/*      */   
/*  129 */   private static final AttributedCharacterIterator.Attribute BASELINE_SHIFT = GVTAttributedCharacterIterator.TextAttribute.BASELINE_SHIFT;
/*      */ 
/*      */   
/*  132 */   private static final AttributedCharacterIterator.Attribute WRITING_MODE = GVTAttributedCharacterIterator.TextAttribute.WRITING_MODE;
/*      */ 
/*      */   
/*  135 */   private static final Integer WRITING_MODE_TTB = GVTAttributedCharacterIterator.TextAttribute.WRITING_MODE_TTB;
/*      */ 
/*      */   
/*  138 */   private static final Integer ORIENTATION_AUTO = GVTAttributedCharacterIterator.TextAttribute.ORIENTATION_AUTO;
/*      */ 
/*      */   
/*  141 */   public static final AttributedCharacterIterator.Attribute GVT_FONT = GVTAttributedCharacterIterator.TextAttribute.GVT_FONT;
/*      */ 
/*      */   
/*  144 */   protected static Set runAtts = new HashSet(); protected static Set szAtts;
/*      */   
/*      */   static  {
/*  147 */     runAtts.add(X);
/*  148 */     runAtts.add(Y);
/*  149 */     runAtts.add(DX);
/*  150 */     runAtts.add(DY);
/*  151 */     runAtts.add(ROTATION);
/*  152 */     runAtts.add(BASELINE_SHIFT);
/*      */ 
/*      */     
/*  155 */     szAtts = new HashSet();
/*      */ 
/*      */     
/*  158 */     szAtts.add(TextAttribute.SIZE);
/*  159 */     szAtts.add(GVT_FONT);
/*  160 */     szAtts.add(LINE_HEIGHT);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public static final double eps = 1.0E-5D;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public GlyphLayout(AttributedCharacterIterator aci, int[] charMap, Point2D offset, FontRenderContext frc) {
/*      */     this.xScale = 1.0F;
/*      */     this.yScale = 1.0F;
/*      */     this.adjSpacing = true;
/*      */     this.layoutApplied = false;
/*      */     this.spacingApplied = false;
/*      */     this.pathApplied = false;
/*  180 */     this.aci = aci;
/*  181 */     this.offset = offset;
/*  182 */     this.font = getFont();
/*  183 */     this.charMap = charMap;
/*      */     
/*  185 */     this.metrics = this.font.getLineMetrics(aci, aci.getBeginIndex(), aci.getEndIndex(), frc);
/*      */ 
/*      */ 
/*      */     
/*  189 */     this.gv = null;
/*  190 */     this.aci.first();
/*  191 */     this.vertical = (aci.getAttribute(WRITING_MODE) == WRITING_MODE_TTB);
/*  192 */     this.textPath = (TextPath)aci.getAttribute(GVTAttributedCharacterIterator.TextAttribute.TEXTPATH);
/*      */ 
/*      */     
/*  195 */     AltGlyphHandler altGlyphHandler = (AltGlyphHandler)this.aci.getAttribute(GVTAttributedCharacterIterator.TextAttribute.ALT_GLYPH_HANDLER);
/*      */ 
/*      */     
/*  198 */     if (altGlyphHandler != null) {
/*      */ 
/*      */       
/*  201 */       this.gv = altGlyphHandler.createGlyphVector(frc, this.font.getSize(), this.aci);
/*      */       
/*  203 */       if (this.gv != null) {
/*  204 */         this.isAltGlyph = true;
/*      */       }
/*      */     } 
/*  207 */     if (this.gv == null)
/*      */     {
/*      */       
/*  210 */       this.gv = this.font.createGlyphVector(frc, this.aci);
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */   
/*  216 */   public GVTGlyphVector getGlyphVector() { return this.gv; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  226 */   public Point2D getOffset() { return this.offset; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public void setScale(float xScale, float yScale, boolean adjSpacing) {
/*  242 */     if (this.vertical) { xScale = 1.0F; }
/*  243 */     else { yScale = 1.0F; }
/*      */     
/*  245 */     if (xScale != this.xScale || yScale != this.yScale || adjSpacing != this.adjSpacing) {
/*      */ 
/*      */       
/*  248 */       this.xScale = xScale;
/*  249 */       this.yScale = yScale;
/*  250 */       this.adjSpacing = adjSpacing;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */       
/*  256 */       this.spacingApplied = false;
/*  257 */       this.glyphAdvances = null;
/*  258 */       this.pathApplied = false;
/*      */     } 
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public void setOffset(Point2D offset) {
/*  270 */     if (offset.getX() != this.offset.getX() || offset.getY() != this.offset.getY()) {
/*      */       
/*  272 */       if (this.layoutApplied || this.spacingApplied) {
/*      */ 
/*      */         
/*  275 */         float dx = (float)(offset.getX() - this.offset.getX());
/*  276 */         float dy = (float)(offset.getY() - this.offset.getY());
/*  277 */         int numGlyphs = this.gv.getNumGlyphs();
/*      */ 
/*      */         
/*  280 */         float[] gp = this.gv.getGlyphPositions(0, numGlyphs + 1, null);
/*  281 */         Point2D.Float pos = new Point2D.Float();
/*  282 */         for (int i = 0; i <= numGlyphs; i++) {
/*  283 */           pos.x = gp[2 * i] + dx;
/*  284 */           pos.y = gp[2 * i + 1] + dy;
/*  285 */           this.gv.setGlyphPosition(i, pos);
/*      */         } 
/*      */       } 
/*      */ 
/*      */ 
/*      */ 
/*      */       
/*  292 */       this.offset = offset;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */       
/*  298 */       this.pathApplied = false;
/*      */     } 
/*      */   }
/*      */ 
/*      */   
/*  303 */   public GVTGlyphMetrics getGlyphMetrics(int glyphIndex) { return this.gv.getGlyphMetrics(glyphIndex); }
/*      */ 
/*      */ 
/*      */   
/*  307 */   public GVTLineMetrics getLineMetrics() { return this.metrics; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  314 */   public boolean isVertical() { return this.vertical; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  321 */   public boolean isOnATextPath() { return (this.textPath != null); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  329 */   public int getGlyphCount() { return this.gv.getNumGlyphs(); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  343 */   public int getCharacterCount(int startGlyphIndex, int endGlyphIndex) { return this.gv.getCharacterCount(startGlyphIndex, endGlyphIndex); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public boolean isLeftToRight() {
/*  350 */     this.aci.first();
/*  351 */     int bidiLevel = ((Integer)this.aci.getAttribute(GVTAttributedCharacterIterator.TextAttribute.BIDI_LEVEL)).intValue();
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  358 */     return ((bidiLevel & true) == 0);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   private final void syncLayout() {
/*  367 */     if (!this.pathApplied)
/*      */     {
/*  369 */       doPathLayout();
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public void draw(Graphics2D g2d) {
/*  379 */     syncLayout();
/*  380 */     this.gv.draw(g2d, this.aci);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public Point2D getAdvance2D() {
/*  388 */     adjustTextSpacing();
/*  389 */     return this.advance;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public Shape getOutline() {
/*  397 */     syncLayout();
/*      */     
/*  399 */     return this.gv.getOutline();
/*      */   }
/*      */   
/*      */   public float[] getGlyphAdvances() {
/*  403 */     if (this.glyphAdvances != null) {
/*  404 */       return this.glyphAdvances;
/*      */     }
/*  406 */     if (!this.spacingApplied)
/*      */     {
/*  408 */       adjustTextSpacing();
/*      */     }
/*  410 */     int numGlyphs = this.gv.getNumGlyphs();
/*  411 */     float[] glyphPos = this.gv.getGlyphPositions(0, numGlyphs + 1, null);
/*  412 */     this.glyphAdvances = new float[numGlyphs + 1];
/*  413 */     int off = 0;
/*  414 */     if (isVertical()) {
/*  415 */       off = 1;
/*      */     }
/*  417 */     float start = glyphPos[off];
/*  418 */     for (int i = 0; i < numGlyphs + 1; i++) {
/*  419 */       this.glyphAdvances[i] = glyphPos[2 * i + off] - start;
/*      */     }
/*  421 */     return this.glyphAdvances;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public Shape getDecorationOutline(int decorationType) {
/*  432 */     syncLayout();
/*      */     
/*  434 */     Shape g = new GeneralPath();
/*  435 */     if ((decorationType & true) != 0) {
/*  436 */       ((GeneralPath)g).append(getUnderlineShape(), false);
/*      */     }
/*  438 */     if ((decorationType & 0x2) != 0) {
/*  439 */       ((GeneralPath)g).append(getStrikethroughShape(), false);
/*      */     }
/*  441 */     if ((decorationType & 0x4) != 0) {
/*  442 */       ((GeneralPath)g).append(getOverlineShape(), false);
/*      */     }
/*  444 */     return g;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public Rectangle2D getBounds2D() {
/*  451 */     syncLayout();
/*  452 */     return this.gv.getBounds2D(this.aci);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public Rectangle2D getGeometricBounds() {
/*  460 */     syncLayout();
/*      */     
/*  462 */     Rectangle2D gvB = this.gv.getGeometricBounds();
/*  463 */     Rectangle2D decB = getDecorationOutline(7).getBounds2D();
/*  464 */     return gvB.createUnion(decB);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public Point2D getTextPathAdvance() {
/*  472 */     syncLayout();
/*  473 */     if (this.textPath != null) {
/*  474 */       return this.textPathAdvance;
/*      */     }
/*  476 */     return getAdvance2D();
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public int getGlyphIndex(int charIndex) {
/*  490 */     int numGlyphs = getGlyphCount();
/*  491 */     int j = 0;
/*  492 */     for (int i = 0; i < numGlyphs; i++) {
/*  493 */       int count = getCharacterCount(i, i);
/*  494 */       for (int n = 0; n < count; n++) {
/*  495 */         int glyphCharIndex = this.charMap[j++];
/*  496 */         if (charIndex == glyphCharIndex)
/*  497 */           return i; 
/*  498 */         if (j >= this.charMap.length)
/*  499 */           return -1; 
/*      */       } 
/*      */     } 
/*  502 */     return -1;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public int getLastGlyphIndex(int charIndex) {
/*  514 */     int numGlyphs = getGlyphCount();
/*  515 */     int j = this.charMap.length - 1;
/*  516 */     for (int i = numGlyphs - 1; i >= 0; i--) {
/*  517 */       int count = getCharacterCount(i, i);
/*  518 */       for (int n = 0; n < count; n++) {
/*  519 */         int glyphCharIndex = this.charMap[j--];
/*  520 */         if (charIndex == glyphCharIndex) return i; 
/*  521 */         if (j < 0) return -1; 
/*      */       } 
/*      */     } 
/*  524 */     return -1;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public double getComputedOrientationAngle(int index) {
/*  534 */     if (isGlyphOrientationAuto()) {
/*  535 */       if (isVertical()) {
/*  536 */         char ch = this.aci.setIndex(index);
/*  537 */         if (isLatinChar(ch)) {
/*  538 */           return 90.0D;
/*      */         }
/*  540 */         return 0.0D;
/*      */       } 
/*  542 */       return 0.0D;
/*      */     } 
/*      */     
/*  545 */     return getGlyphOrientationAngle();
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public Shape getHighlightShape(int beginCharIndex, int endCharIndex) {
/*  560 */     syncLayout();
/*      */     
/*  562 */     if (beginCharIndex > endCharIndex) {
/*  563 */       int temp = beginCharIndex;
/*  564 */       beginCharIndex = endCharIndex;
/*  565 */       endCharIndex = temp;
/*      */     } 
/*  567 */     GeneralPath shape = null;
/*  568 */     int numGlyphs = getGlyphCount();
/*      */     
/*  570 */     Point2D.Float[] arrayOfFloat1 = new Point2D.Float[2 * numGlyphs];
/*  571 */     Point2D.Float[] arrayOfFloat2 = new Point2D.Float[2 * numGlyphs];
/*      */     
/*  573 */     int ptIdx = 0;
/*      */     
/*  575 */     int currentChar = 0;
/*  576 */     for (int i = 0; i < numGlyphs; i++) {
/*  577 */       int glyphCharIndex = this.charMap[currentChar];
/*  578 */       if (glyphCharIndex >= beginCharIndex && glyphCharIndex <= endCharIndex && this.gv.isGlyphVisible(i)) {
/*      */ 
/*      */ 
/*      */         
/*  582 */         Shape gbounds = this.gv.getGlyphLogicalBounds(i);
/*  583 */         if (gbounds != null) {
/*      */           
/*  585 */           if (shape == null) {
/*  586 */             shape = new GeneralPath();
/*      */           }
/*      */ 
/*      */ 
/*      */           
/*  591 */           float[] pts = new float[6];
/*  592 */           int count = 0;
/*  593 */           int type = -1;
/*      */           
/*  595 */           PathIterator pi = gbounds.getPathIterator(null);
/*  596 */           Point2D.Float firstPt = null;
/*      */           
/*  598 */           while (!pi.isDone()) {
/*  599 */             type = pi.currentSegment(pts);
/*  600 */             if (type == 0 || type == 1) {
/*      */ 
/*      */               
/*  603 */               if (count > 4)
/*  604 */                 break;  if (count == 4) {
/*      */                 
/*  606 */                 if (firstPt == null || firstPt.x != pts[0] || firstPt.y != pts[1]) {
/*      */                   break;
/*      */                 }
/*      */               }
/*      */               else {
/*      */                 
/*  612 */                 Point2D.Float pt = new Point2D.Float(pts[0], pts[1]);
/*  613 */                 if (count == 0) firstPt = pt;
/*      */                 
/*  615 */                 switch (count) { case 0:
/*  616 */                     arrayOfFloat2[ptIdx] = pt; break;
/*  617 */                   case 1: arrayOfFloat1[ptIdx] = pt; break;
/*  618 */                   case 2: arrayOfFloat1[ptIdx + 1] = pt; break;
/*  619 */                   case 3: arrayOfFloat2[ptIdx + 1] = pt; break; }
/*      */               
/*      */               } 
/*  622 */             } else if (type == 4) {
/*      */               
/*  624 */               if (count < 4 || count > 5) {
/*      */                 break;
/*      */               }
/*      */             } else {
/*      */               break;
/*      */             } 
/*  630 */             count++;
/*  631 */             pi.next();
/*      */           } 
/*  633 */           if (pi.isDone()) {
/*      */             
/*  635 */             if (arrayOfFloat2[ptIdx] != null && ((arrayOfFloat1[ptIdx]).x != (arrayOfFloat1[ptIdx + 1]).x || (arrayOfFloat1[ptIdx]).y != (arrayOfFloat1[ptIdx + 1]).y))
/*      */             {
/*      */ 
/*      */               
/*  639 */               ptIdx += 2;
/*      */             
/*      */             }
/*      */           }
/*      */           else {
/*      */             
/*  645 */             addPtsToPath(shape, arrayOfFloat1, arrayOfFloat2, ptIdx);
/*  646 */             ptIdx = 0;
/*  647 */             shape.append(gbounds, false);
/*      */           } 
/*      */         } 
/*      */       } 
/*  651 */       currentChar += getCharacterCount(i, i);
/*  652 */       if (currentChar >= this.charMap.length)
/*  653 */         currentChar = this.charMap.length - 1; 
/*      */     } 
/*  655 */     addPtsToPath(shape, arrayOfFloat1, arrayOfFloat2, ptIdx);
/*      */     
/*  657 */     return shape;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  663 */   public static boolean epsEQ(double a, double b) { return (a + 1.0E-5D > b && a - 1.0E-5D < b); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public static int makeConvexHull(Float[] pts, int numPts) {
/*  670 */     for (int i = 1; i < numPts; i++) {
/*      */ 
/*      */       
/*  673 */       if ((pts[i]).x < (pts[i - 1]).x || ((pts[i]).x == (pts[i - 1]).x && (pts[i]).y < (pts[i - 1]).y)) {
/*      */         
/*  675 */         Point2D.Float tmp = pts[i];
/*  676 */         pts[i] = pts[i - 1];
/*  677 */         pts[i - 1] = tmp;
/*  678 */         i = 0;
/*      */       } 
/*      */     } 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  685 */     Point2D.Float pt0 = pts[0];
/*  686 */     Point2D.Float pt1 = pts[numPts - 1];
/*  687 */     Point2D.Float dxdy = new Point2D.Float(pt1.x - pt0.x, pt1.y - pt0.y);
/*  688 */     float c = dxdy.y * pt0.x - dxdy.x * pt0.y;
/*      */     
/*  690 */     Point2D.Float[] arrayOfFloat1 = new Point2D.Float[numPts];
/*  691 */     Point2D.Float[] arrayOfFloat2 = new Point2D.Float[numPts];
/*  692 */     arrayOfFloat1[0] = pts[0]; arrayOfFloat2[0] = pts[0];
/*  693 */     int nTopPts = 1;
/*  694 */     int nBotPts = 1;
/*  695 */     for (int i = 1; i < numPts - 1; i++) {
/*  696 */       Point2D.Float pt = pts[i];
/*  697 */       float soln = dxdy.x * pt.y - dxdy.y * pt.x + c;
/*  698 */       if (soln < 0.0F) {
/*      */         
/*  700 */         while (nBotPts >= 2) {
/*  701 */           pt0 = arrayOfFloat2[nBotPts - 2];
/*  702 */           pt1 = arrayOfFloat2[nBotPts - 1];
/*  703 */           float dx = pt1.x - pt0.x;
/*  704 */           float dy = pt1.y - pt0.y;
/*  705 */           float c0 = dy * pt0.x - dx * pt0.y;
/*  706 */           soln = dx * pt.y - dy * pt.x + c0;
/*  707 */           if (soln > 1.0E-5D)
/*      */             break; 
/*  709 */           if (soln > -1.0E-5D) {
/*      */             
/*  711 */             if (pt1.y < pt.y) pt = pt1; 
/*  712 */             nBotPts--;
/*      */             
/*      */             break;
/*      */           } 
/*  716 */           nBotPts--;
/*      */         } 
/*  718 */         arrayOfFloat2[nBotPts++] = pt;
/*      */       } else {
/*      */         
/*  721 */         while (nTopPts >= 2) {
/*  722 */           pt0 = arrayOfFloat1[nTopPts - 2];
/*  723 */           pt1 = arrayOfFloat1[nTopPts - 1];
/*  724 */           float dx = pt1.x - pt0.x;
/*  725 */           float dy = pt1.y - pt0.y;
/*  726 */           float c0 = dy * pt0.x - dx * pt0.y;
/*  727 */           soln = dx * pt.y - dy * pt.x + c0;
/*  728 */           if (soln < -1.0E-5D)
/*      */             break; 
/*  730 */           if (soln < 1.0E-5D) {
/*      */             
/*  732 */             if (pt1.y > pt.y) pt = pt1; 
/*  733 */             nTopPts--;
/*      */             
/*      */             break;
/*      */           } 
/*  737 */           nTopPts--;
/*      */         } 
/*  739 */         arrayOfFloat1[nTopPts++] = pt;
/*      */       } 
/*      */     } 
/*      */ 
/*      */     
/*  744 */     Point2D.Float pt = pts[numPts - 1];
/*  745 */     while (nBotPts >= 2) {
/*  746 */       pt0 = arrayOfFloat2[nBotPts - 2];
/*  747 */       pt1 = arrayOfFloat2[nBotPts - 1];
/*  748 */       float dx = pt1.x - pt0.x;
/*  749 */       float dy = pt1.y - pt0.y;
/*  750 */       float c0 = dy * pt0.x - dx * pt0.y;
/*  751 */       float f = dx * pt.y - dy * pt.x + c0;
/*  752 */       if (f > 1.0E-5D) {
/*      */         break;
/*      */       }
/*  755 */       if (f > -1.0E-5D) {
/*      */         
/*  757 */         if (pt1.y >= pt.y) nBotPts--;
/*      */         
/*      */         break;
/*      */       } 
/*  761 */       nBotPts--;
/*      */     } 
/*      */     
/*  764 */     while (nTopPts >= 2) {
/*  765 */       pt0 = arrayOfFloat1[nTopPts - 2];
/*  766 */       pt1 = arrayOfFloat1[nTopPts - 1];
/*  767 */       float dx = pt1.x - pt0.x;
/*  768 */       float dy = pt1.y - pt0.y;
/*  769 */       float c0 = dy * pt0.x - dx * pt0.y;
/*  770 */       float f = dx * pt.y - dy * pt.x + c0;
/*  771 */       if (f < -1.0E-5D) {
/*      */         break;
/*      */       }
/*  774 */       if (f < 1.0E-5D) {
/*      */         
/*  776 */         if (pt1.y <= pt.y) nTopPts--;
/*      */         
/*      */         break;
/*      */       } 
/*  780 */       nTopPts--;
/*      */     } 
/*      */     
/*  783 */     System.arraycopy(arrayOfFloat1, 0, pts, 0, nTopPts);
/*  784 */     int i = nTopPts;
/*      */ 
/*      */     
/*  787 */     pts[i++] = pts[numPts - 1];
/*      */ 
/*      */     
/*  790 */     for (int n = nBotPts - 1; n > 0; n--, i++) {
/*  791 */       pts[i] = arrayOfFloat2[n];
/*      */     }
/*      */     
/*  794 */     return i;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public static void addPtsToPath(GeneralPath shape, Float[] topPts, Float[] botPts, int numPts) {
/*  801 */     if (numPts < 2)
/*  802 */       return;  if (numPts == 2) {
/*  803 */       shape.moveTo((topPts[0]).x, (topPts[0]).y);
/*  804 */       shape.lineTo((topPts[1]).x, (topPts[1]).y);
/*  805 */       shape.lineTo((botPts[1]).x, (botPts[1]).y);
/*  806 */       shape.lineTo((botPts[0]).x, (botPts[0]).y);
/*  807 */       shape.lineTo((topPts[0]).x, (topPts[0]).y);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */       
/*      */       return;
/*      */     } 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  819 */     Point2D.Float[] arrayOfFloat1 = new Point2D.Float[8];
/*  820 */     Point2D.Float[] arrayOfFloat2 = new Point2D.Float[8];
/*  821 */     arrayOfFloat1[4] = topPts[0];
/*  822 */     arrayOfFloat1[5] = topPts[1];
/*  823 */     arrayOfFloat1[6] = botPts[1];
/*  824 */     arrayOfFloat1[7] = botPts[0];
/*  825 */     Area[] areas = new Area[numPts / 2];
/*  826 */     int nAreas = 0;
/*  827 */     for (int i = 2; i < numPts; i += 2) {
/*  828 */       arrayOfFloat1[0] = arrayOfFloat1[4];
/*  829 */       arrayOfFloat1[1] = arrayOfFloat1[5];
/*  830 */       arrayOfFloat1[2] = arrayOfFloat1[6];
/*  831 */       arrayOfFloat1[3] = arrayOfFloat1[7];
/*  832 */       arrayOfFloat1[4] = topPts[i];
/*  833 */       arrayOfFloat1[5] = topPts[i + 1];
/*  834 */       arrayOfFloat1[6] = botPts[i + 1];
/*  835 */       arrayOfFloat1[7] = botPts[i];
/*      */ 
/*      */       
/*  838 */       float delta = (arrayOfFloat1[2]).x - (arrayOfFloat1[0]).x;
/*  839 */       float dist = delta * delta;
/*  840 */       delta = (arrayOfFloat1[2]).y - (arrayOfFloat1[0]).y;
/*  841 */       dist += delta * delta;
/*  842 */       float sz = (float)Math.sqrt(dist);
/*      */       
/*  844 */       delta = (arrayOfFloat1[6]).x - (arrayOfFloat1[4]).x;
/*  845 */       dist = delta * delta;
/*  846 */       delta = (arrayOfFloat1[6]).y - (arrayOfFloat1[4]).y;
/*  847 */       dist += delta * delta;
/*  848 */       sz += (float)Math.sqrt(dist);
/*      */       
/*  850 */       delta = ((arrayOfFloat1[0]).x + (arrayOfFloat1[1]).x + (arrayOfFloat1[2]).x + (arrayOfFloat1[3]).x - (arrayOfFloat1[4]).x + (arrayOfFloat1[5]).x + (arrayOfFloat1[6]).x + (arrayOfFloat1[7]).x) / 4.0F;
/*      */       
/*  852 */       dist = delta * delta;
/*  853 */       delta = ((arrayOfFloat1[0]).y + (arrayOfFloat1[1]).y + (arrayOfFloat1[2]).y + (arrayOfFloat1[3]).y - (arrayOfFloat1[4]).y + (arrayOfFloat1[5]).y + (arrayOfFloat1[6]).y + (arrayOfFloat1[7]).y) / 4.0F;
/*      */       
/*  855 */       dist += delta * delta;
/*  856 */       dist = (float)Math.sqrt(dist);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */       
/*  867 */       GeneralPath gp = new GeneralPath();
/*  868 */       if (dist < sz) {
/*      */         
/*  870 */         System.arraycopy(arrayOfFloat1, 0, arrayOfFloat2, 0, 8);
/*  871 */         int npts = makeConvexHull(arrayOfFloat2, 8);
/*  872 */         gp.moveTo((arrayOfFloat2[0]).x, (arrayOfFloat2[0]).y);
/*  873 */         for (int n = 1; n < npts; n++)
/*  874 */           gp.lineTo((arrayOfFloat2[n]).x, (arrayOfFloat2[n]).y); 
/*  875 */         gp.closePath();
/*      */       } else {
/*      */         
/*  878 */         mergeAreas(shape, areas, nAreas);
/*  879 */         nAreas = 0;
/*      */ 
/*      */         
/*  882 */         if (i == 2) {
/*  883 */           gp.moveTo((arrayOfFloat1[0]).x, (arrayOfFloat1[0]).y);
/*  884 */           gp.lineTo((arrayOfFloat1[1]).x, (arrayOfFloat1[1]).y);
/*  885 */           gp.lineTo((arrayOfFloat1[2]).x, (arrayOfFloat1[2]).y);
/*  886 */           gp.lineTo((arrayOfFloat1[3]).x, (arrayOfFloat1[3]).y);
/*  887 */           gp.closePath();
/*  888 */           shape.append(gp, false);
/*  889 */           gp.reset();
/*      */         } 
/*  891 */         gp.moveTo((arrayOfFloat1[4]).x, (arrayOfFloat1[4]).y);
/*  892 */         gp.lineTo((arrayOfFloat1[5]).x, (arrayOfFloat1[5]).y);
/*  893 */         gp.lineTo((arrayOfFloat1[6]).x, (arrayOfFloat1[6]).y);
/*  894 */         gp.lineTo((arrayOfFloat1[7]).x, (arrayOfFloat1[7]).y);
/*  895 */         gp.closePath();
/*      */       } 
/*  897 */       areas[nAreas++] = new Area(gp);
/*      */     } 
/*      */     
/*  900 */     mergeAreas(shape, areas, nAreas);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public static void mergeAreas(GeneralPath shape, Area[] shapes, int nShapes) {
/*  909 */     while (nShapes > 1) {
/*  910 */       int n = 0;
/*  911 */       for (int i = 1; i < nShapes; i += 2) {
/*  912 */         shapes[i - 1].add(shapes[i]);
/*  913 */         shapes[n++] = shapes[i - 1];
/*  914 */         shapes[i] = null;
/*      */       } 
/*      */ 
/*      */       
/*  918 */       if ((nShapes & true) == 1)
/*  919 */         shapes[n - 1].add(shapes[nShapes - 1]); 
/*  920 */       nShapes /= 2;
/*      */     } 
/*  922 */     if (nShapes == 1) {
/*  923 */       shape.append(shapes[0], false);
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public TextHit hitTestChar(float x, float y) {
/*  937 */     syncLayout();
/*      */     
/*  939 */     textHit = null;
/*      */     
/*  941 */     int currentChar = 0;
/*  942 */     for (int i = 0; i < this.gv.getNumGlyphs(); i++) {
/*  943 */       Shape gbounds = this.gv.getGlyphLogicalBounds(i);
/*  944 */       if (gbounds != null) {
/*  945 */         Rectangle2D gbounds2d = gbounds.getBounds2D();
/*      */ 
/*      */         
/*  948 */         if (gbounds.contains(x, y)) {
/*  949 */           boolean isRightHalf = (x > gbounds2d.getX() + gbounds2d.getWidth() / 2.0D);
/*      */           
/*  951 */           boolean isLeadingEdge = !isRightHalf;
/*  952 */           int charIndex = this.charMap[currentChar];
/*  953 */           return new TextHit(charIndex, isLeadingEdge);
/*      */         } 
/*      */       } 
/*      */       
/*  957 */       currentChar += getCharacterCount(i, i);
/*  958 */       if (currentChar >= this.charMap.length)
/*  959 */         currentChar = this.charMap.length - 1; 
/*      */     } 
/*  961 */     return textHit;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected GVTFont getFont() {
/*  974 */     this.aci.first();
/*  975 */     GVTFont gvtFont = (GVTFont)this.aci.getAttribute(GVT_FONT);
/*      */     
/*  977 */     if (gvtFont != null) {
/*  978 */       return gvtFont;
/*      */     }
/*      */     
/*  981 */     return new AWTGVTFont(this.aci.getAttributes());
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected Shape getOverlineShape() {
/*  988 */     double y = this.metrics.getOverlineOffset();
/*  989 */     float overlineThickness = this.metrics.getOverlineThickness();
/*      */ 
/*      */ 
/*      */     
/*  993 */     y += overlineThickness;
/*      */ 
/*      */     
/*  996 */     this.aci.first();
/*  997 */     Float dy = (Float)this.aci.getAttribute(DY);
/*  998 */     if (dy != null) {
/*  999 */       y += dy.floatValue();
/*      */     }
/* 1001 */     Stroke overlineStroke = new BasicStroke(overlineThickness);
/*      */     
/* 1003 */     Rectangle2D logicalBounds = this.gv.getLogicalBounds();
/*      */     
/* 1005 */     return overlineStroke.createStrokedShape(new Line2D.Double(logicalBounds.getMinX() + overlineThickness / 2.0D, this.offset.getY() + y, logicalBounds.getMaxX() - overlineThickness / 2.0D, this.offset.getY() + y));
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected Shape getUnderlineShape() {
/* 1016 */     double y = this.metrics.getUnderlineOffset();
/* 1017 */     float underlineThickness = this.metrics.getUnderlineThickness();
/*      */ 
/*      */ 
/*      */     
/* 1021 */     y += underlineThickness * 1.5D;
/*      */     
/* 1023 */     BasicStroke underlineStroke = new BasicStroke(underlineThickness);
/*      */ 
/*      */ 
/*      */     
/* 1027 */     this.aci.first();
/* 1028 */     Float dy = (Float)this.aci.getAttribute(DY);
/* 1029 */     if (dy != null) {
/* 1030 */       y += dy.floatValue();
/*      */     }
/* 1032 */     Rectangle2D logicalBounds = this.gv.getLogicalBounds();
/*      */     
/* 1034 */     return underlineStroke.createStrokedShape(new Line2D.Double(logicalBounds.getMinX() + underlineThickness / 2.0D, this.offset.getY() + y, logicalBounds.getMaxX() - underlineThickness / 2.0D, this.offset.getY() + y));
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected Shape getStrikethroughShape() {
/* 1044 */     double y = this.metrics.getStrikethroughOffset();
/* 1045 */     float strikethroughThickness = this.metrics.getStrikethroughThickness();
/*      */     
/* 1047 */     Stroke strikethroughStroke = new BasicStroke(strikethroughThickness);
/*      */ 
/*      */ 
/*      */     
/* 1051 */     this.aci.first();
/* 1052 */     Float dy = (Float)this.aci.getAttribute(DY);
/* 1053 */     if (dy != null) {
/* 1054 */       y += dy.floatValue();
/*      */     }
/* 1056 */     Rectangle2D logicalBounds = this.gv.getLogicalBounds();
/* 1057 */     return strikethroughStroke.createStrokedShape(new Line2D.Double(logicalBounds.getMinX() + strikethroughThickness / 2.0D, this.offset.getY() + y, logicalBounds.getMaxX() - strikethroughThickness / 2.0D, this.offset.getY() + y));
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void doExplicitGlyphLayout() {
/* 1071 */     this.gv.performDefaultLayout();
/*      */     
/* 1073 */     float baselineAscent = this.vertical ? (float)this.gv.getLogicalBounds().getWidth() : (this.metrics.getAscent() + Math.abs(this.metrics.getDescent()));
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 1078 */     int numGlyphs = this.gv.getNumGlyphs();
/*      */ 
/*      */     
/* 1081 */     float[] gp = this.gv.getGlyphPositions(0, numGlyphs + 1, null);
/* 1082 */     float verticalFirstOffset = 0.0F;
/* 1083 */     float horizontalFirstOffset = 0.0F;
/*      */     
/* 1085 */     boolean glyphOrientationAuto = isGlyphOrientationAuto();
/* 1086 */     int glyphOrientationAngle = 0;
/* 1087 */     if (!glyphOrientationAuto) {
/* 1088 */       glyphOrientationAngle = getGlyphOrientationAngle();
/*      */     }
/* 1090 */     int i = 0;
/* 1091 */     int aciStart = this.aci.getBeginIndex();
/* 1092 */     int aciIndex = 0;
/* 1093 */     char ch = this.aci.first();
/* 1094 */     int runLimit = aciIndex + aciStart;
/*      */     
/* 1096 */     Float x = null, y = null, dx = null, dy = null, rotation = null;
/* 1097 */     Object baseline = null;
/*      */     
/* 1099 */     float shift_x_pos = 0.0F;
/* 1100 */     float shift_y_pos = 0.0F;
/* 1101 */     float curr_x_pos = (float)this.offset.getX();
/* 1102 */     float curr_y_pos = (float)this.offset.getY();
/*      */     
/* 1104 */     Point2D.Float pos = new Point2D.Float();
/* 1105 */     boolean hasArabicTransparent = false;
/*      */     
/* 1107 */     while (i < numGlyphs) {
/*      */       
/* 1109 */       if (aciIndex + aciStart >= runLimit) {
/* 1110 */         runLimit = this.aci.getRunLimit(runAtts);
/* 1111 */         x = (Float)this.aci.getAttribute(X);
/* 1112 */         y = (Float)this.aci.getAttribute(Y);
/* 1113 */         dx = (Float)this.aci.getAttribute(DX);
/* 1114 */         dy = (Float)this.aci.getAttribute(DY);
/* 1115 */         rotation = (Float)this.aci.getAttribute(ROTATION);
/* 1116 */         baseline = this.aci.getAttribute(BASELINE_SHIFT);
/*      */       } 
/*      */       
/* 1119 */       GVTGlyphMetrics gm = this.gv.getGlyphMetrics(i);
/*      */       
/* 1121 */       if (i == 0) {
/* 1122 */         if (isVertical()) {
/* 1123 */           if (glyphOrientationAuto) {
/* 1124 */             if (isLatinChar(ch)) {
/*      */               
/* 1126 */               verticalFirstOffset = 0.0F;
/*      */             } else {
/*      */               
/* 1129 */               float advY = gm.getVerticalAdvance();
/* 1130 */               float asc = this.metrics.getAscent();
/* 1131 */               float dsc = this.metrics.getDescent();
/* 1132 */               verticalFirstOffset = asc + (advY - asc + dsc) / 2.0F;
/*      */             }
/*      */           
/* 1135 */           } else if (glyphOrientationAngle == 0) {
/* 1136 */             float advY = gm.getVerticalAdvance();
/* 1137 */             float asc = this.metrics.getAscent();
/* 1138 */             float dsc = this.metrics.getDescent();
/* 1139 */             verticalFirstOffset = asc + (advY - asc + dsc) / 2.0F;
/*      */           } else {
/*      */             
/* 1142 */             verticalFirstOffset = 0.0F;
/*      */           }
/*      */         
/*      */         }
/* 1146 */         else if (glyphOrientationAngle == 270) {
/* 1147 */           horizontalFirstOffset = (float)gm.getBounds2D().getHeight();
/*      */         }
/*      */         else {
/*      */           
/* 1151 */           horizontalFirstOffset = 0.0F;
/*      */         }
/*      */       
/*      */       }
/* 1155 */       else if (glyphOrientationAuto && verticalFirstOffset == 0.0F && !isLatinChar(ch)) {
/*      */         
/* 1157 */         float advY = gm.getVerticalAdvance();
/* 1158 */         float asc = this.metrics.getAscent();
/* 1159 */         float dsc = this.metrics.getDescent();
/* 1160 */         verticalFirstOffset = asc + (advY - asc + dsc) / 2.0F;
/*      */       } 
/*      */ 
/*      */ 
/*      */ 
/*      */       
/* 1166 */       float ox = 0.0F;
/* 1167 */       float oy = 0.0F;
/* 1168 */       float glyphOrientationRotation = 0.0F;
/* 1169 */       float glyphRotation = 0.0F;
/*      */ 
/*      */       
/* 1172 */       if (ch != Character.MAX_VALUE) {
/* 1173 */         if (this.vertical) {
/* 1174 */           if (glyphOrientationAuto) {
/* 1175 */             if (isLatinChar(ch)) {
/*      */ 
/*      */               
/* 1178 */               glyphOrientationRotation = 1.5707964F;
/*      */             } else {
/* 1180 */               glyphOrientationRotation = 0.0F;
/*      */             } 
/*      */           } else {
/* 1183 */             glyphOrientationRotation = (float)Math.toRadians(glyphOrientationAngle);
/*      */           } 
/* 1185 */           if (this.textPath != null)
/*      */           {
/* 1187 */             x = null;
/*      */           }
/*      */         } else {
/* 1190 */           glyphOrientationRotation = (float)Math.toRadians(glyphOrientationAngle);
/* 1191 */           if (this.textPath != null)
/*      */           {
/* 1193 */             y = null;
/*      */           }
/*      */         } 
/*      */ 
/*      */         
/* 1198 */         if (rotation == null || rotation.isNaN()) {
/* 1199 */           glyphRotation = glyphOrientationRotation;
/*      */         } else {
/* 1201 */           glyphRotation = rotation.floatValue() + glyphOrientationRotation;
/*      */         } 
/*      */ 
/*      */         
/* 1205 */         if (x != null && !x.isNaN()) {
/* 1206 */           if (i == 0)
/* 1207 */             shift_x_pos = (float)(x.floatValue() - this.offset.getX()); 
/* 1208 */           curr_x_pos = x.floatValue() - shift_x_pos;
/*      */         } 
/* 1210 */         if (dx != null && !dx.isNaN()) {
/* 1211 */           curr_x_pos += dx.floatValue();
/*      */         }
/*      */         
/* 1214 */         if (y != null && !y.isNaN()) {
/* 1215 */           if (i == 0)
/* 1216 */             shift_y_pos = (float)(y.floatValue() - this.offset.getY()); 
/* 1217 */           curr_y_pos = y.floatValue() - shift_y_pos;
/*      */         } 
/* 1219 */         if (dy != null && !dy.isNaN()) {
/* 1220 */           curr_y_pos += dy.floatValue();
/* 1221 */         } else if (i > 0) {
/* 1222 */           curr_y_pos += gp[i * 2 + 1] - gp[i * 2 - 1];
/*      */         } 
/*      */         
/* 1225 */         float baselineAdjust = 0.0F;
/* 1226 */         if (baseline != null) {
/* 1227 */           if (baseline instanceof Integer) {
/* 1228 */             if (baseline == TextAttribute.SUPERSCRIPT_SUPER) {
/* 1229 */               baselineAdjust = baselineAscent * 0.5F;
/* 1230 */             } else if (baseline == TextAttribute.SUPERSCRIPT_SUB) {
/* 1231 */               baselineAdjust = -baselineAscent * 0.5F;
/*      */             } 
/* 1233 */           } else if (baseline instanceof Float) {
/* 1234 */             baselineAdjust = ((Float)baseline).floatValue();
/*      */           } 
/* 1236 */           if (this.vertical) {
/* 1237 */             ox = baselineAdjust;
/*      */           } else {
/* 1239 */             oy = -baselineAdjust;
/*      */           } 
/*      */         } 
/*      */         
/* 1243 */         if (this.vertical) {
/*      */           
/* 1245 */           oy += verticalFirstOffset;
/*      */           
/* 1247 */           if (glyphOrientationAuto) {
/* 1248 */             if (isLatinChar(ch)) {
/* 1249 */               ox += this.metrics.getStrikethroughOffset();
/*      */             } else {
/* 1251 */               Rectangle2D glyphBounds = this.gv.getGlyphVisualBounds(i).getBounds2D();
/*      */               
/* 1253 */               ox -= (float)(glyphBounds.getMaxX() - gp[2 * i] - glyphBounds.getWidth() / 2.0D);
/*      */             }
/*      */           
/*      */           } else {
/*      */             
/* 1258 */             Rectangle2D glyphBounds = this.gv.getGlyphVisualBounds(i).getBounds2D();
/*      */             
/* 1260 */             if (glyphOrientationAngle == 0) {
/* 1261 */               ox -= (float)(glyphBounds.getMaxX() - gp[2 * i] - glyphBounds.getWidth() / 2.0D);
/*      */             }
/* 1263 */             else if (glyphOrientationAngle == 180) {
/* 1264 */               ox += (float)(glyphBounds.getMaxX() - gp[2 * i] - glyphBounds.getWidth() / 2.0D);
/*      */             }
/* 1266 */             else if (glyphOrientationAngle == 90) {
/* 1267 */               ox += this.metrics.getStrikethroughOffset();
/*      */             } else {
/* 1269 */               ox -= this.metrics.getStrikethroughOffset();
/*      */             } 
/*      */           } 
/*      */         } else {
/* 1273 */           ox += horizontalFirstOffset;
/* 1274 */           if (glyphOrientationAngle == 90) {
/* 1275 */             oy -= gm.getHorizontalAdvance();
/* 1276 */           } else if (glyphOrientationAngle == 180) {
/* 1277 */             oy -= this.metrics.getAscent();
/*      */           } 
/*      */         } 
/*      */       } 
/*      */ 
/*      */       
/* 1283 */       pos.x = curr_x_pos + ox;
/* 1284 */       pos.y = curr_y_pos + oy;
/* 1285 */       this.gv.setGlyphPosition(i, pos);
/*      */ 
/*      */       
/* 1288 */       if (ArabicTextHandler.arabicCharTransparent(ch)) {
/* 1289 */         hasArabicTransparent = true;
/*      */       
/*      */       }
/* 1292 */       else if (this.vertical) {
/* 1293 */         float advanceY = 0.0F;
/* 1294 */         if (glyphOrientationAuto) {
/* 1295 */           if (isLatinChar(ch)) {
/* 1296 */             advanceY = gm.getHorizontalAdvance();
/*      */           } else {
/* 1298 */             advanceY = gm.getVerticalAdvance();
/*      */           }
/*      */         
/* 1301 */         } else if (glyphOrientationAngle == 0 || glyphOrientationAngle == 180) {
/*      */           
/* 1303 */           advanceY = gm.getVerticalAdvance();
/* 1304 */         } else if (glyphOrientationAngle == 90) {
/* 1305 */           advanceY = gm.getHorizontalAdvance();
/*      */         } else {
/* 1307 */           advanceY = gm.getHorizontalAdvance();
/*      */ 
/*      */           
/* 1310 */           this.gv.setGlyphTransform(i, AffineTransform.getTranslateInstance(0.0D, advanceY));
/*      */         } 
/*      */ 
/*      */ 
/*      */         
/* 1315 */         curr_y_pos += advanceY;
/*      */       } else {
/* 1317 */         float advanceX = 0.0F;
/* 1318 */         if (glyphOrientationAngle == 0) {
/* 1319 */           advanceX = gm.getHorizontalAdvance();
/* 1320 */         } else if (glyphOrientationAngle == 180) {
/* 1321 */           advanceX = gm.getHorizontalAdvance();
/*      */ 
/*      */           
/* 1324 */           this.gv.setGlyphTransform(i, AffineTransform.getTranslateInstance(advanceX, 0.0D));
/*      */         
/*      */         }
/*      */         else {
/*      */           
/* 1329 */           advanceX = gm.getVerticalAdvance();
/*      */         } 
/* 1331 */         curr_x_pos += advanceX;
/*      */       } 
/*      */ 
/*      */ 
/*      */       
/* 1336 */       if (!epsEQ(glyphRotation, 0.0D)) {
/* 1337 */         AffineTransform affineTransform, glyphTransform = this.gv.getGlyphTransform(i);
/* 1338 */         if (glyphTransform == null) {
/* 1339 */           glyphTransform = new AffineTransform();
/*      */         }
/*      */ 
/*      */ 
/*      */         
/* 1344 */         if (epsEQ(glyphRotation, 1.5707963267948966D)) {
/* 1345 */           affineTransform = new AffineTransform(0.0F, 1.0F, -1.0F, 0.0F, 0.0F, 0.0F);
/* 1346 */         } else if (epsEQ(glyphRotation, Math.PI)) {
/* 1347 */           affineTransform = new AffineTransform(-1.0F, 0.0F, 0.0F, -1.0F, 0.0F, 0.0F);
/* 1348 */         } else if (epsEQ(glyphRotation, 4.71238898038469D)) {
/* 1349 */           affineTransform = new AffineTransform(0.0F, -1.0F, 1.0F, 0.0F, 0.0F, 0.0F);
/*      */         } else {
/* 1351 */           affineTransform = AffineTransform.getRotateInstance(glyphRotation);
/*      */         } 
/* 1353 */         glyphTransform.concatenate(affineTransform);
/* 1354 */         this.gv.setGlyphTransform(i, glyphTransform);
/*      */       } 
/*      */       
/* 1357 */       aciIndex += this.gv.getCharacterCount(i, i);
/* 1358 */       if (aciIndex >= this.charMap.length)
/* 1359 */         aciIndex = this.charMap.length - 1; 
/* 1360 */       ch = this.aci.setIndex(aciIndex + aciStart);
/* 1361 */       i++;
/*      */     } 
/*      */     
/* 1364 */     pos.x = curr_x_pos;
/* 1365 */     pos.y = curr_y_pos;
/* 1366 */     this.gv.setGlyphPosition(i, pos);
/*      */     
/* 1368 */     this.advance = new Point2D.Float((float)(curr_x_pos - this.offset.getX()), (float)(curr_y_pos - this.offset.getY()));
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 1374 */     if (hasArabicTransparent) {
/* 1375 */       ch = this.aci.first();
/* 1376 */       aciIndex = 0;
/* 1377 */       i = 0;
/* 1378 */       int transparentStart = -1;
/* 1379 */       while (i < numGlyphs) {
/* 1380 */         if (ArabicTextHandler.arabicCharTransparent(ch)) {
/* 1381 */           if (transparentStart == -1) transparentStart = i;
/*      */         
/* 1383 */         } else if (transparentStart != -1) {
/* 1384 */           Point2D loc = this.gv.getGlyphPosition(i);
/* 1385 */           GVTGlyphMetrics gm = this.gv.getGlyphMetrics(i);
/* 1386 */           int tyS = 0, txS = 0;
/* 1387 */           float advX = 0.0F, advY = 0.0F;
/* 1388 */           if (this.vertical) {
/* 1389 */             if (glyphOrientationAuto || glyphOrientationAngle == 90) {
/*      */               
/* 1391 */               advY = gm.getHorizontalAdvance();
/* 1392 */             } else if (glyphOrientationAngle == 270) {
/* 1393 */               advY = 0.0F;
/* 1394 */             } else if (glyphOrientationAngle == 0) {
/* 1395 */               advX = gm.getHorizontalAdvance();
/*      */             } else {
/* 1397 */               advX = -gm.getHorizontalAdvance();
/*      */             } 
/* 1399 */           } else if (glyphOrientationAngle == 0) {
/* 1400 */             advX = gm.getHorizontalAdvance();
/* 1401 */           } else if (glyphOrientationAngle == 90) {
/* 1402 */             advY = gm.getHorizontalAdvance();
/* 1403 */           } else if (glyphOrientationAngle == 180) {
/* 1404 */             advX = 0.0F;
/*      */           } else {
/* 1406 */             advY = -gm.getHorizontalAdvance();
/*      */           } 
/* 1408 */           float baseX = (float)(loc.getX() + advX);
/* 1409 */           float baseY = (float)(loc.getY() + advY);
/* 1410 */           for (int j = transparentStart; j < i; j++) {
/* 1411 */             Point2D locT = this.gv.getGlyphPosition(j);
/* 1412 */             GVTGlyphMetrics gmT = this.gv.getGlyphMetrics(j);
/* 1413 */             float locX = (float)locT.getX();
/* 1414 */             float locY = (float)locT.getY();
/* 1415 */             float tx = 0.0F, ty = 0.0F;
/* 1416 */             float advT = gmT.getHorizontalAdvance();
/* 1417 */             if (this.vertical) {
/* 1418 */               if (glyphOrientationAuto || glyphOrientationAngle == 90) {
/*      */                 
/* 1420 */                 locY = baseY - advT;
/* 1421 */               } else if (glyphOrientationAngle == 270) {
/* 1422 */                 locY = baseY + advT;
/* 1423 */               } else if (glyphOrientationAngle == 0) {
/* 1424 */                 locX = baseX - advT;
/*      */               } else {
/* 1426 */                 locX = baseX + advT;
/*      */               } 
/* 1428 */             } else if (glyphOrientationAngle == 0) {
/* 1429 */               locX = baseX - advT;
/* 1430 */             } else if (glyphOrientationAngle == 90) {
/* 1431 */               locY = baseY - advT;
/* 1432 */             } else if (glyphOrientationAngle == 180) {
/* 1433 */               locX = baseX + advT;
/*      */             } else {
/* 1435 */               locY = baseY + advT;
/*      */             } 
/*      */             
/* 1438 */             locT = new Point2D.Double(locX, locY);
/* 1439 */             this.gv.setGlyphPosition(j, locT);
/* 1440 */             if (txS != 0 || tyS != 0) {
/*      */               
/* 1442 */               AffineTransform at = AffineTransform.getTranslateInstance(tx, ty);
/*      */               
/* 1444 */               at.concatenate(this.gv.getGlyphTransform(i));
/* 1445 */               this.gv.setGlyphTransform(i, at);
/*      */             } 
/*      */           } 
/* 1448 */           transparentStart = -1;
/*      */         } 
/*      */         
/* 1451 */         aciIndex += this.gv.getCharacterCount(i, i);
/* 1452 */         if (aciIndex >= this.charMap.length)
/* 1453 */           aciIndex = this.charMap.length - 1; 
/* 1454 */         ch = this.aci.setIndex(aciIndex + aciStart);
/* 1455 */         i++;
/*      */       } 
/*      */     } 
/*      */ 
/*      */ 
/*      */     
/* 1461 */     this.layoutApplied = true;
/* 1462 */     this.spacingApplied = false;
/* 1463 */     this.glyphAdvances = null;
/* 1464 */     this.pathApplied = false;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void adjustTextSpacing() {
/* 1472 */     if (this.spacingApplied) {
/*      */       return;
/*      */     }
/*      */     
/* 1476 */     if (!this.layoutApplied)
/*      */     {
/* 1478 */       doExplicitGlyphLayout();
/*      */     }
/* 1480 */     this.aci.first();
/* 1481 */     Boolean customSpacing = (Boolean)this.aci.getAttribute(GVTAttributedCharacterIterator.TextAttribute.CUSTOM_SPACING);
/*      */     
/* 1483 */     if (customSpacing != null && customSpacing.booleanValue()) {
/* 1484 */       this.advance = doSpacing((Float)this.aci.getAttribute(GVTAttributedCharacterIterator.TextAttribute.KERNING), (Float)this.aci.getAttribute(GVTAttributedCharacterIterator.TextAttribute.LETTER_SPACING), (Float)this.aci.getAttribute(GVTAttributedCharacterIterator.TextAttribute.WORD_SPACING));
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */       
/* 1492 */       this.layoutApplied = false;
/*      */     } 
/*      */ 
/*      */ 
/*      */     
/* 1497 */     applyStretchTransform(!this.adjSpacing);
/*      */     
/* 1499 */     this.spacingApplied = true;
/* 1500 */     this.pathApplied = false;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected Point2D doSpacing(Float kern, Float letterSpacing, Float wordSpacing) {
/* 1514 */     boolean autoKern = true;
/* 1515 */     boolean doWordSpacing = false;
/* 1516 */     boolean doLetterSpacing = false;
/* 1517 */     float kernVal = 0.0F;
/* 1518 */     float letterSpacingVal = 0.0F;
/*      */     
/* 1520 */     if (kern != null && !kern.isNaN()) {
/* 1521 */       kernVal = kern.floatValue();
/* 1522 */       autoKern = false;
/*      */     } 
/*      */     
/* 1525 */     if (letterSpacing != null && !letterSpacing.isNaN()) {
/* 1526 */       letterSpacingVal = letterSpacing.floatValue();
/* 1527 */       doLetterSpacing = true;
/*      */     } 
/*      */     
/* 1530 */     if (wordSpacing != null && !wordSpacing.isNaN()) {
/* 1531 */       doWordSpacing = true;
/*      */     }
/*      */     
/* 1534 */     int numGlyphs = this.gv.getNumGlyphs();
/*      */     
/* 1536 */     float dx = 0.0F;
/* 1537 */     float dy = 0.0F;
/* 1538 */     Point2D[] newPositions = new Point2D[numGlyphs + 1];
/* 1539 */     Point2D prevPos = this.gv.getGlyphPosition(0);
/* 1540 */     int prevCode = this.gv.getGlyphCode(0);
/* 1541 */     float x = (float)prevPos.getX();
/* 1542 */     float y = (float)prevPos.getY();
/*      */     
/* 1544 */     Point2D lastCharAdvance = new Point2D.Double(this.advance.getX() - this.gv.getGlyphPosition(numGlyphs - 1).getX() - x, this.advance.getY() - this.gv.getGlyphPosition(numGlyphs - 1).getY() - y);
/*      */ 
/*      */ 
/*      */     
/*      */     try {
/* 1549 */       GVTFont font = this.gv.getFont();
/*      */       
/* 1551 */       if (numGlyphs > 1 && (doLetterSpacing || !autoKern)) {
/* 1552 */         for (int i = 1; i <= numGlyphs; i++) {
/* 1553 */           Point2D gpos = this.gv.getGlyphPosition(i);
/*      */           
/* 1555 */           int currCode = (i == numGlyphs) ? -1 : this.gv.getGlyphCode(i);
/* 1556 */           dx = (float)gpos.getX() - (float)prevPos.getX();
/* 1557 */           dy = (float)gpos.getY() - (float)prevPos.getY();
/* 1558 */           if (autoKern) {
/* 1559 */             if (this.vertical) { dy += letterSpacingVal; }
/* 1560 */             else { dx += letterSpacingVal; }
/*      */ 
/*      */           
/*      */           }
/* 1564 */           else if (this.vertical) {
/* 1565 */             float vKern = 0.0F;
/* 1566 */             if (currCode != -1)
/* 1567 */               vKern = font.getVKern(prevCode, currCode); 
/* 1568 */             dy += kernVal - vKern + letterSpacingVal;
/*      */           } else {
/* 1570 */             float hKern = 0.0F;
/* 1571 */             if (currCode != -1)
/* 1572 */               hKern = font.getHKern(prevCode, currCode); 
/* 1573 */             dx += kernVal - hKern + letterSpacingVal;
/*      */           } 
/*      */           
/* 1576 */           x += dx;
/* 1577 */           y += dy;
/* 1578 */           newPositions[i] = new Point2D.Float(x, y);
/* 1579 */           prevPos = gpos;
/* 1580 */           prevCode = currCode;
/*      */         } 
/*      */         
/* 1583 */         for (int i = 1; i <= numGlyphs; i++) {
/* 1584 */           if (newPositions[i] != null) {
/* 1585 */             this.gv.setGlyphPosition(i, newPositions[i]);
/*      */           }
/*      */         } 
/*      */       } 
/*      */ 
/*      */       
/* 1591 */       if (this.vertical) {
/* 1592 */         lastCharAdvance.setLocation(lastCharAdvance.getX(), lastCharAdvance.getY() + kernVal + letterSpacingVal);
/*      */       }
/*      */       else {
/*      */         
/* 1596 */         lastCharAdvance.setLocation(lastCharAdvance.getX() + kernVal + letterSpacingVal, lastCharAdvance.getY());
/*      */       } 
/*      */ 
/*      */ 
/*      */ 
/*      */       
/* 1602 */       dx = 0.0F;
/* 1603 */       dy = 0.0F;
/* 1604 */       prevPos = this.gv.getGlyphPosition(0);
/* 1605 */       x = (float)prevPos.getX();
/* 1606 */       y = (float)prevPos.getY();
/*      */       
/* 1608 */       if (numGlyphs > 1 && doWordSpacing) {
/* 1609 */         for (int i = 1; i < numGlyphs; i++) {
/* 1610 */           Point2D gpos = this.gv.getGlyphPosition(i);
/* 1611 */           dx = (float)gpos.getX() - (float)prevPos.getX();
/* 1612 */           dy = (float)gpos.getY() - (float)prevPos.getY();
/* 1613 */           boolean inWS = false;
/*      */           
/* 1615 */           int beginWS = i;
/* 1616 */           int endWS = i;
/* 1617 */           GVTGlyphMetrics gm = this.gv.getGlyphMetrics(i);
/*      */ 
/*      */           
/* 1620 */           while (gm.getBounds2D().getWidth() < 0.01D || gm.isWhitespace()) {
/* 1621 */             if (!inWS) inWS = true; 
/* 1622 */             if (i == numGlyphs - 1) {
/*      */               break;
/*      */             }
/*      */             
/* 1626 */             i++;
/* 1627 */             endWS++;
/* 1628 */             gpos = this.gv.getGlyphPosition(i);
/* 1629 */             gm = this.gv.getGlyphMetrics(i);
/*      */           } 
/*      */           
/* 1632 */           if (inWS) {
/* 1633 */             int nWS = endWS - beginWS;
/* 1634 */             float px = (float)prevPos.getX();
/* 1635 */             float py = (float)prevPos.getY();
/* 1636 */             dx = (float)(gpos.getX() - px) / (nWS + 1);
/* 1637 */             dy = (float)(gpos.getY() - py) / (nWS + 1);
/* 1638 */             if (this.vertical) {
/* 1639 */               dy += wordSpacing.floatValue() / (nWS + 1);
/*      */             } else {
/* 1641 */               dx += wordSpacing.floatValue() / (nWS + 1);
/*      */             } 
/* 1643 */             for (int j = beginWS; j <= endWS; j++) {
/* 1644 */               x += dx;
/* 1645 */               y += dy;
/* 1646 */               newPositions[j] = new Point2D.Float(x, y);
/*      */             } 
/*      */           } else {
/* 1649 */             dx = (float)(gpos.getX() - prevPos.getX());
/* 1650 */             dy = (float)(gpos.getY() - prevPos.getY());
/* 1651 */             x += dx;
/* 1652 */             y += dy;
/* 1653 */             newPositions[i] = new Point2D.Float(x, y);
/*      */           } 
/* 1655 */           prevPos = gpos;
/*      */         } 
/* 1657 */         Point2D gPos = this.gv.getGlyphPosition(numGlyphs);
/* 1658 */         x += (float)(gPos.getX() - prevPos.getX());
/* 1659 */         y += (float)(gPos.getY() - prevPos.getY());
/* 1660 */         newPositions[numGlyphs] = new Point2D.Float(x, y);
/*      */         
/* 1662 */         for (int i = 1; i <= numGlyphs; i++) {
/* 1663 */           if (newPositions[i] != null) {
/* 1664 */             this.gv.setGlyphPosition(i, newPositions[i]);
/*      */           }
/*      */         } 
/*      */       } 
/*      */     } catch (Exception e) {
/*      */       
/* 1670 */       e.printStackTrace();
/*      */     } 
/*      */ 
/*      */     
/* 1674 */     double advX = this.gv.getGlyphPosition(numGlyphs - 1).getX() - this.gv.getGlyphPosition(0).getX();
/*      */     
/* 1676 */     double advY = this.gv.getGlyphPosition(numGlyphs - 1).getY() - this.gv.getGlyphPosition(0).getY();
/*      */     
/* 1678 */     return new Point2D.Double(advX + lastCharAdvance.getX(), advY + lastCharAdvance.getY());
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void applyStretchTransform(boolean stretchGlyphs) {
/* 1690 */     if (this.xScale == 1.0F && this.yScale == 1.0F) {
/*      */       return;
/*      */     }
/* 1693 */     AffineTransform scaleAT = AffineTransform.getScaleInstance(this.xScale, this.yScale);
/*      */ 
/*      */     
/* 1696 */     int numGlyphs = this.gv.getNumGlyphs();
/* 1697 */     float[] gp = this.gv.getGlyphPositions(0, numGlyphs + 1, null);
/*      */     
/* 1699 */     float initX = gp[0];
/* 1700 */     float initY = gp[1];
/* 1701 */     Point2D.Float pos = new Point2D.Float();
/* 1702 */     for (int i = 0; i <= numGlyphs; i++) {
/* 1703 */       float dx = gp[2 * i] - initX;
/* 1704 */       float dy = gp[2 * i + 1] - initY;
/* 1705 */       pos.x = initX + dx * this.xScale;
/* 1706 */       pos.y = initY + dy * this.yScale;
/* 1707 */       this.gv.setGlyphPosition(i, pos);
/*      */       
/* 1709 */       if (stretchGlyphs && i != numGlyphs) {
/*      */         
/* 1711 */         AffineTransform glyphTransform = this.gv.getGlyphTransform(i);
/* 1712 */         if (glyphTransform != null) {
/* 1713 */           glyphTransform.preConcatenate(scaleAT);
/* 1714 */           this.gv.setGlyphTransform(i, glyphTransform);
/*      */         } else {
/* 1716 */           this.gv.setGlyphTransform(i, scaleAT);
/*      */         } 
/*      */       } 
/*      */     } 
/*      */     
/* 1721 */     this.advance = new Point2D.Float((float)(this.advance.getX() * this.xScale), (float)(this.advance.getY() * this.yScale));
/*      */ 
/*      */     
/* 1724 */     this.layoutApplied = false;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected void doPathLayout() {
/*      */     float f2, f1;
/* 1732 */     if (this.pathApplied) {
/*      */       return;
/*      */     }
/* 1735 */     if (!this.spacingApplied)
/*      */     {
/* 1737 */       adjustTextSpacing();
/*      */     }
/* 1739 */     getGlyphAdvances();
/*      */ 
/*      */     
/* 1742 */     if (this.textPath == null) {
/*      */       
/* 1744 */       this.pathApplied = true;
/*      */       
/*      */       return;
/*      */     } 
/*      */     
/* 1749 */     boolean horizontal = !isVertical();
/*      */     
/* 1751 */     boolean glyphOrientationAuto = isGlyphOrientationAuto();
/* 1752 */     int glyphOrientationAngle = 0;
/* 1753 */     if (!glyphOrientationAuto) {
/* 1754 */       glyphOrientationAngle = getGlyphOrientationAngle();
/*      */     }
/*      */     
/* 1757 */     float pathLength = this.textPath.lengthOfPath();
/* 1758 */     float startOffset = this.textPath.getStartOffset();
/* 1759 */     int numGlyphs = this.gv.getNumGlyphs();
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 1764 */     for (int i = 0; i < numGlyphs; i++) {
/* 1765 */       this.gv.setGlyphVisible(i, true);
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 1771 */     if (horizontal) {
/* 1772 */       f1 = (float)this.gv.getLogicalBounds().getWidth();
/*      */     } else {
/* 1774 */       f1 = (float)this.gv.getLogicalBounds().getHeight();
/*      */     } 
/*      */ 
/*      */     
/* 1778 */     if (pathLength == 0.0F || f1 == 0.0F) {
/*      */       
/* 1780 */       this.pathApplied = true;
/* 1781 */       this.textPathAdvance = this.advance;
/*      */ 
/*      */ 
/*      */       
/*      */       return;
/*      */     } 
/*      */ 
/*      */     
/* 1789 */     Point2D firstGlyphPosition = this.gv.getGlyphPosition(0);
/* 1790 */     float glyphOffset = 0.0F;
/*      */     
/* 1792 */     if (horizontal) {
/* 1793 */       glyphOffset = (float)firstGlyphPosition.getY();
/* 1794 */       f2 = (float)(firstGlyphPosition.getX() + startOffset);
/*      */     } else {
/* 1796 */       glyphOffset = (float)firstGlyphPosition.getX();
/* 1797 */       f2 = (float)(firstGlyphPosition.getY() + startOffset);
/*      */     } 
/*      */     
/* 1800 */     char ch = this.aci.first();
/* 1801 */     int start = this.aci.getBeginIndex();
/* 1802 */     int currentChar = 0;
/* 1803 */     int lastGlyphDrawn = -1;
/* 1804 */     float lastGlyphAdvance = 0.0F;
/*      */     
/* 1806 */     for (int i = 0; i < numGlyphs; i++) {
/*      */       float f;
/* 1808 */       Point2D currentGlyphPos = this.gv.getGlyphPosition(i);
/*      */ 
/*      */ 
/*      */ 
/*      */       
/* 1813 */       float glyphAdvance = 0.0F;
/* 1814 */       float nextGlyphOffset = 0.0F;
/* 1815 */       Point2D nextGlyphPosition = this.gv.getGlyphPosition(i + 1);
/* 1816 */       if (horizontal) {
/* 1817 */         glyphAdvance = (float)(nextGlyphPosition.getX() - currentGlyphPos.getX());
/*      */         
/* 1819 */         nextGlyphOffset = (float)(nextGlyphPosition.getY() - currentGlyphPos.getY());
/*      */       } else {
/*      */         
/* 1822 */         glyphAdvance = (float)(nextGlyphPosition.getY() - currentGlyphPos.getY());
/*      */         
/* 1824 */         nextGlyphOffset = (float)(nextGlyphPosition.getX() - currentGlyphPos.getX());
/*      */       } 
/*      */ 
/*      */ 
/*      */       
/* 1829 */       Rectangle2D glyphBounds = this.gv.getGlyphOutline(i).getBounds2D();
/* 1830 */       float glyphWidth = (float)glyphBounds.getWidth();
/* 1831 */       float glyphHeight = (float)glyphBounds.getHeight();
/* 1832 */       float glyphMidX = 0.0F;
/* 1833 */       if (glyphWidth > 0.0F) {
/* 1834 */         glyphMidX = (float)(glyphBounds.getX() + (glyphWidth / 2.0F));
/* 1835 */         glyphMidX -= (float)currentGlyphPos.getX();
/*      */       } 
/*      */       
/* 1838 */       float glyphMidY = 0.0F;
/* 1839 */       if (glyphHeight > 0.0F) {
/* 1840 */         glyphMidY = (float)(glyphBounds.getY() + (glyphHeight / 2.0F));
/* 1841 */         glyphMidY -= (float)currentGlyphPos.getY();
/*      */       } 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */       
/* 1855 */       if (horizontal) {
/* 1856 */         f = f2 + glyphMidX;
/*      */       } else {
/* 1858 */         f = f2 + glyphMidY;
/*      */       } 
/*      */ 
/*      */       
/* 1862 */       Point2D charMidPoint = this.textPath.pointAtLength(f);
/*      */ 
/*      */       
/* 1865 */       if (charMidPoint != null) {
/*      */ 
/*      */         
/* 1868 */         float angle = this.textPath.angleAtLength(f);
/*      */ 
/*      */         
/* 1871 */         AffineTransform glyphPathTransform = new AffineTransform();
/*      */ 
/*      */         
/* 1874 */         if (horizontal) {
/* 1875 */           glyphPathTransform.rotate(angle);
/*      */         } else {
/* 1877 */           glyphPathTransform.rotate(angle - 1.5707963267948966D);
/*      */         } 
/*      */ 
/*      */         
/* 1881 */         if (horizontal) {
/* 1882 */           glyphPathTransform.translate(0.0D, glyphOffset);
/*      */         } else {
/* 1884 */           glyphPathTransform.translate(glyphOffset, 0.0D);
/*      */         } 
/*      */ 
/*      */ 
/*      */         
/* 1889 */         if (horizontal) {
/* 1890 */           glyphPathTransform.translate(-glyphMidX, 0.0D);
/*      */         } else {
/* 1892 */           glyphPathTransform.translate(0.0D, -glyphMidY);
/*      */         } 
/*      */ 
/*      */         
/* 1896 */         AffineTransform glyphTransform = this.gv.getGlyphTransform(i);
/* 1897 */         if (glyphTransform != null) {
/* 1898 */           glyphPathTransform.concatenate(glyphTransform);
/*      */         }
/*      */         
/* 1901 */         this.gv.setGlyphTransform(i, glyphPathTransform);
/* 1902 */         this.gv.setGlyphPosition(i, charMidPoint);
/*      */ 
/*      */         
/* 1905 */         lastGlyphDrawn = i;
/* 1906 */         lastGlyphAdvance = glyphAdvance;
/*      */       }
/*      */       else {
/*      */         
/* 1910 */         this.gv.setGlyphVisible(i, false);
/*      */       } 
/* 1912 */       f2 += glyphAdvance;
/* 1913 */       glyphOffset += nextGlyphOffset;
/* 1914 */       currentChar += this.gv.getCharacterCount(i, i);
/* 1915 */       if (currentChar >= this.charMap.length)
/* 1916 */         currentChar = this.charMap.length - 1; 
/* 1917 */       ch = this.aci.setIndex(currentChar + start);
/*      */     } 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 1923 */     if (lastGlyphDrawn > -1) {
/* 1924 */       Point2D lastGlyphPos = this.gv.getGlyphPosition(lastGlyphDrawn);
/* 1925 */       if (horizontal) {
/* 1926 */         this.textPathAdvance = new Point2D.Double(lastGlyphPos.getX() + lastGlyphAdvance, lastGlyphPos.getY());
/*      */       }
/*      */       else {
/*      */         
/* 1930 */         this.textPathAdvance = new Point2D.Double(lastGlyphPos.getX(), lastGlyphPos.getY() + lastGlyphAdvance);
/*      */       }
/*      */     
/*      */     } else {
/*      */       
/* 1935 */       this.textPathAdvance = new Point2D.Double(0.0D, 0.0D);
/*      */     } 
/*      */ 
/*      */     
/* 1939 */     this.layoutApplied = false;
/*      */     
/* 1941 */     this.spacingApplied = false;
/* 1942 */     this.pathApplied = true;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected boolean isLatinChar(char c) {
/* 1955 */     if (c < 'ÿ' && Character.isLetterOrDigit(c))
/*      */     {
/* 1957 */       return true;
/*      */     }
/*      */     
/* 1960 */     Character.UnicodeBlock block = Character.UnicodeBlock.of(c);
/*      */     
/* 1962 */     if (block == Character.UnicodeBlock.BASIC_LATIN || block == Character.UnicodeBlock.LATIN_1_SUPPLEMENT || block == Character.UnicodeBlock.LATIN_EXTENDED_ADDITIONAL || block == Character.UnicodeBlock.LATIN_EXTENDED_A || block == Character.UnicodeBlock.LATIN_EXTENDED_B || block == Character.UnicodeBlock.ARABIC || block == Character.UnicodeBlock.ARABIC_PRESENTATION_FORMS_A || block == Character.UnicodeBlock.ARABIC_PRESENTATION_FORMS_B)
/*      */     {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */       
/* 1970 */       return true;
/*      */     }
/* 1972 */     return false;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected boolean isGlyphOrientationAuto() {
/* 1979 */     if (!isVertical()) return false; 
/* 1980 */     this.aci.first();
/* 1981 */     Integer vOrient = (Integer)this.aci.getAttribute(VERTICAL_ORIENTATION);
/* 1982 */     if (vOrient != null) {
/* 1983 */       return (vOrient == ORIENTATION_AUTO);
/*      */     }
/* 1985 */     return true;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   protected int getGlyphOrientationAngle() {
/*      */     Float float;
/* 1994 */     int glyphOrientationAngle = 0;
/*      */     
/* 1996 */     this.aci.first();
/*      */ 
/*      */     
/* 1999 */     if (isVertical()) {
/* 2000 */       float = (Float)this.aci.getAttribute(VERTICAL_ORIENTATION_ANGLE);
/*      */     } else {
/* 2002 */       float = (Float)this.aci.getAttribute(HORIZONTAL_ORIENTATION_ANGLE);
/*      */     } 
/*      */     
/* 2005 */     if (float != null) {
/* 2006 */       glyphOrientationAngle = (int)float.floatValue();
/*      */     }
/*      */ 
/*      */     
/* 2010 */     if (glyphOrientationAngle != 0 || glyphOrientationAngle != 90 || glyphOrientationAngle != 180 || glyphOrientationAngle != 270) {
/*      */ 
/*      */       
/* 2013 */       while (glyphOrientationAngle < 0) {
/* 2014 */         glyphOrientationAngle += 360;
/*      */       }
/*      */       
/* 2017 */       while (glyphOrientationAngle >= 360) {
/* 2018 */         glyphOrientationAngle -= 360;
/*      */       }
/*      */       
/* 2021 */       if (glyphOrientationAngle <= 45 || glyphOrientationAngle > 315) {
/*      */         
/* 2023 */         glyphOrientationAngle = 0;
/* 2024 */       } else if (glyphOrientationAngle > 45 && glyphOrientationAngle <= 135) {
/*      */         
/* 2026 */         glyphOrientationAngle = 90;
/* 2027 */       } else if (glyphOrientationAngle > 135 && glyphOrientationAngle <= 225) {
/*      */         
/* 2029 */         glyphOrientationAngle = 180;
/*      */       } else {
/* 2031 */         glyphOrientationAngle = 270;
/*      */       } 
/*      */     } 
/* 2034 */     return glyphOrientationAngle;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public boolean hasCharacterIndex(int index) {
/* 2046 */     for (int n = 0; n < this.charMap.length; n++) {
/* 2047 */       if (index == this.charMap[n])
/* 2048 */         return true; 
/*      */     } 
/* 2050 */     return false;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/* 2058 */   public boolean isAltGlyph() { return this.isAltGlyph; }
/*      */ }


/* Location:              C:\Program Files (x86)\IBM\WebSphere\AppServer\profiles\ctgAppSrv01\installedApps\ctgCell01\MAXIMO.ear\maximouiweb.war\WEB-INF\classes\!\com\ibm\tivoli\maximo\report\control\svgtools\batik-gvt.jar!\org\apache\batik\gvt\text\GlyphLayout.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       1.0.7
 */